corefunctions.cpp

   Copyright 2016 Toby Kenney

   This file is part of Adequate Bootstrap.

   Adequate Bootstrap is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Adequate Bootstrap is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Adequate Bootstrap.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <iostream>
#include <fstream>
#include <cmath>
#include <boost/math/distributions/normal.hpp>
#include <boost/math/distributions/chi_squared.hpp>
#include <cstdlib>
#include <ctime>

#include <boost/math/distributions/binomial.hpp>
#include <boost/random/mersenne_twister.hpp>
#include <boost/random/variate_generator.hpp>
#include <boost/generator_iterator.hpp>
#include <boost/random/binomial_distribution.hpp>
#include <boost/random/mersenne_twister.hpp>
#include <boost/random/normal_distribution.hpp>

boost::math::normal stdnorm(0.0, 1.0);

#define BSSIZE 10000
//Number of bootstraps to perform.


#include "distributions.h"
//#include "tests.h"
//#include "regression.h"
//#include "general.h"
#include "corefunctions.h"

using namespace std;

long double chisqcval;

std::ofstream savedata;

unsigned int jackknife;

unsigned int frequency_table;

unsigned int grouped_data;

unsigned int ngroup;

unsigned int maxbssize;

long double regression_lower=0;
long double regression_upper=1;
unsigned int regression_numstep=100;


//typedef long double (*NP_estimator)(unsigned int n,long double *data,unsigned int *samp);

NPestimator *NPest=NULL;
NPestimator *CILest=NULL;
NPestimator *CIUest=NULL;

void MLEProb(unsigned int n,const unsigned int *outcomes,long double *p){
//Assumes probability is a monotone function of the xvalues. Calculates
//the MLE of the probability for each x value under this constraint.

//outcomes is a matrix whose first column is number of
//         successes and whose last column is number of trials.
//         Rows are sorted in increasing order of xvalue.

  //  long double *minfor=new long double[n];
  long double *sucint=new long double[n*n];
  long double *totint=new long double[n*n];
  for(unsigned int i=0;i<n;i++){
    *(sucint+i*n+i)=*(outcomes+2*i);
    *(totint+i*n+i)=*(outcomes+2*i+1);
    for(unsigned int j=i+1;j<n;j++){
      *(sucint+i*n+j)=*(sucint+i*n+j-1)+*(outcomes+2*j);
      *(totint+i*n+j)=*(totint+i*n+j-1)+*(outcomes+2*j+1);
    };
  };
  long double max=0;

  for(unsigned int i=0;i<n;i++){
    *(p+i)=1;
    for(unsigned int j=i;j<n;j++){
      long double rat=((long double)*(sucint+i*n+j))/((long double)*(totint+i*n+j));
      if(rat<*(p+i)){
	*(p+i)=rat;
      };
    };
    if(*(p+i)<max){
      *(p+i)=max;
    }else{
      max=*(p+i);
    };
  };
  delete[] sucint;
  delete[] totint;
};


inline unsigned int position(long double prob,unsigned int n,const long double *phat){
  unsigned int N=n-1;
  unsigned int Nl=0;
  if(*(phat+N)>prob){
    if(*(phat)<prob){
      while(N-Nl>1){
	unsigned int Nm=(N+Nl)/2;
	if(*(phat+Nm)<prob){
	  Nl=Nm;
	}else{
	  N=Nm;
	};
      };
    }else{
      N=0;
    };
  };
  return N;
};

ofstream outlog;
char plotpower=0;
ostream *plotpowerstream;


void write_to_log(unsigned int n,long double *phat,unsigned int *outcomes ,unsigned int *xvals){
  if(plotpower){
    for(unsigned int i=0;i<n;i++){
      (*plotpowerstream)<<*(xvals+i)<<" "<<*(phat+i)<<" "<<*(outcomes+2*i)<<"/"<<*(outcomes+2*i+1)<<"\n";
    };
  };
};


unsigned int solveprob(unsigned int n,unsigned int *xvals,AdequacyTest &Ad,Distribution& dist,long double prob,Data &dat,unsigned int mxwd=100,unsigned int mxdp=10000){
//xvals are the values to be attempted
//trial is a function which produces a single run for one x value.
//prob is the target probability.
  unsigned int *outcomes=new unsigned int[2*n];
  for(unsigned int i=0;i<n;i++){       
    *(outcomes+2*i+1)=1;
    *(outcomes+2*i)=0;
    //Try to get more accurate measurement.
    for(unsigned int j=0;j<1;j++){
      dat.subsample(*(xvals+i));
      if(Ad.apply(dat,dist)){
	(*(outcomes+2*i))++;
      };
    };
  };
  //  write_to_log(n,outcomes,xvals);
  long double *phat=new long double[n];
  MLEProb(n,outcomes,phat);
  unsigned int N=position(prob,n,phat);
//Now produce more results close to the value N
  for(unsigned int k=1;k<11;k++){
    unsigned int intwidth=(mxwd/k);
    unsigned int intdepth=(mxdp/intwidth);
    for(unsigned int i=(N>intwidth)?(N-intwidth):0;i<N+intwidth&&i<n;i++){
      *(outcomes+2*i+1)+=intdepth;
      for(unsigned int j=0;j<intdepth;j++){
	dat.subsample(xvals[i]);
	if(Ad.apply(dat,dist)){
	  (*(outcomes+2*i))++;
	};
      };
    };
    MLEProb(n,outcomes,phat);
    N=position(prob,n,phat);
    write_to_log(n,phat,outcomes,xvals);
    //    ofstream outlog("outcomes.log",ios_base::app);
    //    if(outlog.is_open()){
    //      for(unsigned int i=0;i<n;i++){
    //	outlog<<*(xvals+i)<<" "<<*(outcomes+2*i)<<" "<<*(outcomes+2*i)<<"\n";
    //      };
    //    };
  };
  write_to_log(n,phat,outcomes,xvals);
  delete[] outcomes;
  delete[] phat;
  return N;
};

unsigned int solveprobfast(unsigned int n,unsigned int *xvals,AdequacyTest &Ad,Distribution &dist,long double prob,Data &dat){
  unsigned int numstep=log10(n)-1;
  unsigned int firststep=pow(10,numstep);
  //First run on every hundredth size
  unsigned int nh=n/firststep;
  unsigned int *xvh=new unsigned int[nh];
  for(unsigned int i=0;i<nh;i++){
    *(xvh+i)=*(xvals+i*firststep);
  };
  unsigned int N=solveprob(nh,xvh,Ad,dist,prob,dat,20,1000);

  unsigned int bt=0;


  unsigned int increasedepth=0;
  if(numstep>1){
      increasedepth=3000/(numstep-1);
  };

  for(;numstep>0;numstep--){
    unsigned int tp=n-1;
    if((N+10)*firststep<tp){
      tp=(N+10)*firststep;
    };
    bt=0;
    if(N>10){
      bt=(N-10)*firststep;
    };
    firststep/=10;

    unsigned int nh2=(tp-bt+1)/firststep;
    if(nh2>nh){
      delete[] xvh;
      xvh=new unsigned int[nh2];
      nh=nh2;
    };
    for(unsigned int i=0;i<nh2;i++){
      *(xvh+i)=*(xvals+bt+i*firststep);
    };
    N=solveprob(nh2,xvh,Ad,dist,prob,dat,40,5000-increasedepth*(numstep-1));
  };
  delete[]xvh;
  return(*(xvals+N+bt));
};









void getpos(unsigned int n,const long double *list,unsigned int numps,const unsigned int *pos,long double *to){
  //pos should be sorted.

  long double *newlist=new long double[n];
  unsigned int nl=0;
  unsigned int nu=1;
  long double pivot=*list;
  for(unsigned int i=1;i<n;i++){
    if(*(list+i)<pivot){
      *(newlist+nl++)=*(list+i);
    }else{
      *(newlist+n-nu++)=*(list+i);
    };
  };
  *(newlist+nl)=pivot;
  unsigned int i;
  for(i=0;i<numps&&pos[i]<nl;i++);
  if(i>0){
    getpos(nl,newlist,i,pos,to);
  };
  if(i<numps){
    if(pos[i]==nl){
      *(to+i)=pivot;    
      if(i+1<numps){
	unsigned int *newpos=new unsigned int[numps-i-1];
	for(unsigned int j=0;j<numps-i-1;j++){
	  newpos[j]=pos[i+j+1]-nl-1;
	};
	getpos(nu-1,newlist+nl+1,numps-i-1,newpos,to+i+1);
	delete[] newpos;
      };
    }else{
      //pos[i]>nl
      unsigned int *newpos=new unsigned int[numps-i];
      for(unsigned int j=0;j<numps-i;j++){
	newpos[j]=pos[i+j]-nl-1;
      };
      getpos(nu-1,newlist+nl+1,numps-i,newpos,to+i);
      delete[] newpos;
    };    
  };
  delete[] newlist;
};


unsigned int solveprobscan(unsigned int n,unsigned int *xvals,AdequacyTest& trial,long double prob,Distribution& dist,Data &dat){
  //Quickly checks whether the range is sensible for the values needed.
  if(n<100){
    return solveprob(n,xvals,trial,dist,prob,dat,15,20);
  };
  unsigned int N=solveprob(100,xvals,trial,dist,prob,dat,15,20);
  if(N>95){
    return solveprobfast(n,xvals,trial,dist,prob,dat);
  };  
  if(N>80){
    if(n>199){
      N=solveprobfast(200,xvals,trial,dist,prob,dat);
      if(N<170){
	return N;
      };
    };
    return solveprobfast(n,xvals,trial,dist,prob,dat);
  };
  if(N>5){
    return solveprobfast(100,xvals,trial,dist,prob,dat);
  };
  return N;
};






void read_option(const char* const optname){
    char *at=NULL;
    char *at2=NULL;
    char *eq=NULL;
    char* name=new char[strlen(optname)+1];
    strcpy(name,optname);
    unsigned int len=strlen(name);
    for(unsigned int i=0;i<len;i++){
      if(*(name+i)=='@'){
	*(name+i)='\0';
	if(at==NULL){
	  at=(name+i+1);
	}else{
	  at2=(name+i+1);
	};
      }else if(*(name+i)=='='){
	*(name+i)='\0';
	eq=(name+i+1);	
      };
    };
    if(!strcmp(name,"seed")){
      if(at!=NULL){
	cout<<"@ option not used for seed option. Ignoring.\n";
      };
      if(eq==NULL){
	cout<<"Seed value unspecified. Ignoring seed setting.\n";
      }else{	
	unsigned int val=atoi(eq);
	cout<<"Seeding random generator with value "<<val<<".\n";
	srand(val);
      };
    }else if(!strcmp(name,"NP")){
      NPest=get_NP_estimator(eq,at);
    }else if(!strcmp(name,"CI")){
      if(eq==NULL){
	cout<<"CI method unspecified. Ignoring CI setting.\n";
      }else{	
	if(*eq!='['){
	  cout<<"first method: \""<<eq<<"\"\n";
	  cout<<"CI format \"CI=[LOWER,UPPER]\".\n";
	}else{
	  eq++;
	  char* eq2=eq;	
	  for(;*eq2!='\0'&&*eq2!=',';eq2++);
	  if(*eq2!=','){
	    if(++eq2==at){
	      for(;*eq2!='\0'&&*eq2!=',';eq2++);
	      if(*eq2!=','){
		cout<<"Arguments:\t\""<<eq<<"\"\n\t\t\""<<at<<"\"\n\t\t\""<<at2<<"\"\n";
		cout<<"CI format \"CI=[LOWER,UPPER]\".\n";
	      };
	    }else{
		cout<<"Arguments:\t\""<<eq<<"\"\n\t\t\""<<at<<"\"\n\t\t\""<<eq2<<"\"\n\t\t\""<<at2<<"\"\n";	      cout<<"CI format \"CI=[LOWER,UPPER]\".\n";
	    };
	  };
	  *(eq2++)='\0';
	  CILest=get_NP_estimator(eq,at);
	  CIUest=get_NP_estimator(eq2,at);
	};
      };
    }else if(!strcmp(name,"jackknife")){
      if(eq==NULL){
	jackknife=1;
      }else{
	if(!strcmp(eq,"1")||!strcmp(eq,"T")||!strcmp(eq,"yes")||!strcmp(eq,"true")||!strcmp(eq,"True")||!strcmp(eq,"Yes")){
	  jackknife=1;
	}else if(!strcmp(eq,"0")||!strcmp(eq,"F")||!strcmp(eq,"no")||!strcmp(eq,"No")||!strcmp(eq,"False")||!strcmp(eq,"false")){
	  jackknife=0;
	}else{
	  cout<<"Could not determine meaning of setting \""<<eq<<"\" for jackknife.\n";
	};
      };      
    }else if(!strcmp(name,"log")){
      if(eq==NULL){
	outlog.open("outcomes.log");
	if(!outlog.is_open()){
	  cout<<"Problem opening default log file \"outcomes.log\"\n";
	};
      }else{
	outlog.open(eq);
	if(!outlog.is_open()){
	  cout<<"Problem opening log file \""<<eq<<"\"\n";
	};	
      };      
    }else if(!strcmp(name,"save")){
      if(eq==NULL){
	eq=(char *)"saved_data";
	//No checking for overwriting file;
      };
      savedata.open(eq);      
      if(!savedata.is_open()){
	cout<<"Unable to open file \""<<eq<<"\".\n";
      };
    }else if(!strcmp(name,"frequency")){
      if(eq==NULL){
	frequency_table=1;
      }else{
	if(!strcmp(eq,"1")||!strcmp(eq,"T")||!strcmp(eq,"yes")||!strcmp(eq,"true")||!strcmp(eq,"True")||!strcmp(eq,"Yes")){
	  frequency_table=1;
	}else if(!strcmp(eq,"0")||!strcmp(eq,"F")||!strcmp(eq,"no")||!strcmp(eq,"No")||!strcmp(eq,"False")||!strcmp(eq,"false")){
	  frequency_table=0;
	}else{
	  cout<<"Could not determine meaning of setting \""<<eq<<"\" for frequency.\n";
	};
      };
    }else if(!strcmp(name,"grouped")){
      if(eq==NULL){
	grouped_data=1;
      }else{
	if(!strcmp(eq,"1")||!strcmp(eq,"T")||!strcmp(eq,"yes")||!strcmp(eq,"true")||!strcmp(eq,"True")||!strcmp(eq,"Yes")){
	 grouped_data=1;
	}else if(!strcmp(eq,"0")||!strcmp(eq,"F")||!strcmp(eq,"no")||!strcmp(eq,"No")||!strcmp(eq,"False")||!strcmp(eq,"false")){
	  grouped_data=0;
	}else{
	  cout<<"Could not determine meaning of setting \""<<eq<<"\" for grouped data.\n";
	};
      };
    }else if(!strcmp(name,"maxbssize")){
      if(eq!=NULL){
	maxbssize=atoi(eq);
      };
    }else if(!strcmp(name,"regression_lower")){
      //lower limit for graphing regression
      if(eq==NULL){
	cout<<"Regression lower limit  unspecified. Using default value 0.\n";
      }else{
	regression_lower=atof(eq);
      };
    }else if(!strcmp(name,"regression_upper")){
      //lower limit for graphing regression
      if(eq==NULL){
	cout<<"Regression upper limit  unspecified. Using default value 1.\n";
      }else{
	regression_upper=atof(eq);
      };
    }else if(!strcmp(name,"regression_num_steps")){
      //lower limit for graphing regression
      if(eq==NULL){
	cout<<"Regression number of steps unspecified. Using default value 100.\n";
      }else{
	regression_numstep=atoi(eq);
      };
    }else if(!strcmp(name,"regression_step_size")){
      //lower limit for graphing regression
      if(eq==NULL){
	cout<<"Regression step size unspecified. Using default value.\n";
      }else{
	long double stepsz=atof(eq);
	regression_numstep=abs((int)((regression_upper-regression_lower)/stepsz));
      };
    }else if(!strcmp(name,"plotpower")){
      //plot power curve
      plotpower=1;
      if(eq==NULL){
	plotpowerstream=&cout;
      }else{
	cout<<"Opening file \""<<eq<<"\" for writing power curve.\n";	
	plotpowerstream=new ofstream(eq);
	if(!(plotpowerstream->good())){
	  cout<<"Could not open file \""<<eq<<"\" for writing power curve.\n";
	  plotpowerstream=&cout;
	};
      };      
    }else{
      cout<<"Option \""<<name<<"\" not currently supported. Ignoring option.\n";
      // long double atval=0;
      // if(at!=NULL){
      // 	atval=atof(at);
      // 	cout<<atval<<"\n";
      // };
    };

    delete[] name;
};





long double chisqlookup(unsigned int n,long double atval){
  //  calculates the critical value of a chi-square distribution with
  //  n degrees of freedom at significance level atval.
  boost::math::chi_squared chisquaredist(n);
  long double answ=boost::math::quantile(chisquaredist,1-atval);
  //  cout<<n<<" "<<atval<<" "<<answ<<"\n";
  return answ;
  long double *vals=new long double[10000];
  for(unsigned int i=0;i<10000;i++){
    *(vals+i)=0;
    for(unsigned int j=0;j<n;j++){
      long double rnd=rnorm();
      *(vals+i)+=rnd*rnd;
    };
  };
  unsigned int cutoff=(1-atval)*10000;
  //  cout<<"cutoff="<<cutoff<<"\n";
  long double ans=0;
  getpos(10000,vals,1,&cutoff,&ans);
  delete[] vals;
  cout<<"Critical value of chi-squared test is "<<ans<<".\n";
  return ans;
};

void cleanup(){
  //Does some necessary clean-up on end of program.


  if(plotpowerstream!=NULL&&plotpowerstream!=&cout){
    ((ofstream*)plotpowerstream)->close();
  };

  
};


#include <cstdlib>
#include <cmath>
#include <iostream>

using namespace std;

#include "general.h"

unsigned int rbinom(unsigned int n,long double p){
  unsigned int cuttoff=p*RAND_MAX;
  unsigned int ans=0;
  for(unsigned int i=0;i<n;i++){
    if((unsigned int)rand()<cuttoff){
      ans++;
    };
  };
  return ans;
};


void makebintable(unsigned int n,long double p,unsigned int min,unsigned int max,long double* to){
  //  cout<<"n="<<n<<", p="<<p<<", min="<<min<<", max="<<max<<"\n";
  long double prob=1;
  *to=1;
  long double a=-p/(1-p);
  long double b=-a*(n+1);
  //  cout<<"a="<<a<<", b="<<b<<"\n";
  for(unsigned int i=min;i<max;i++){
    prob*=a+b/(i+1);
    //    cout<<prob<<" ";
    *(to+i-min+1)=*(to+i-min)+prob;
  };
  for(unsigned int i=0;i<=max-min;i++){
    *(to+i)/=*(to+max-min);
    //        cout<<*(to+i)<<" ";
  };  
  //    cout<<"\n";
};


unsigned int getrndfromlkup(unsigned int min,unsigned int max,const long double *lkup){
  unsigned int mn=0;
  unsigned int mx=max-min;
  long double v=((long double)rand())/RAND_MAX;
  //  cout<<v<<"\n";
  while(mx-mn>1){
    unsigned int test=(mx+mn)/2;
    //      cout<<test<<" "<<*(lkup+test)<<"\n";
    if(v<*(lkup+test)){
      mx=test;
    }else{
      mn=test;
    };    
  };
  if(v<(*(lkup+mn))){
      return min+mn;
    }else{
      return min+mx;
    };
};

unsigned int rbinom_fast(unsigned int n,long double p){
  if(n<400){
    return rbinom(n,p);
  };


  //  cout<<"\n";
  unsigned int N=pow(n,0.666666666666666666667);

  long double mean=N*p;
  long double sd=sqrt(N*p*(1-p));
  long double minplaus=mean-8*sd;
  unsigned int min=minplaus;
  if(minplaus<0){
    min=0;
  };
  long double maxplaus=mean+8*sd;
  unsigned int max=maxplaus;
  if(maxplaus<0){
    max=0;
  };
  long double *lkuptab=new long double[max+1-min];
  makebintable(N,p,min,max,lkuptab);
  //Start by making a lookuptable for size N. Then generate n/N random
  //binomials for N and sum them.

  unsigned int k=n/N;
  unsigned int tot=0;
  //  cout<<"N="<<N<<", k="<<k<<"\n"; 
  for(unsigned int i=0;i<k;i++){
    unsigned int rbinN=getrndfromlkup(min,max,lkuptab);
    tot+=rbinN;
    //    cout<<rbinN<<"  ";
  };
  //  cout<<"\n";
  unsigned int rem=n-k*N;
  delete[] lkuptab;
  //  cout<<"N="<<N<<", k="<<k<<", rem="<<rem<<"\n";
  return tot+rbinom_fast(rem,p);
};


unsigned int rhgeom(unsigned int n,unsigned int success,unsigned int tot){
  unsigned int ans=0;
  for(unsigned int i=0;i<n;i++){
    unsigned int cuttoff=(((long unsigned int)success)*RAND_MAX)/tot;
    if((unsigned int)rand()<cuttoff){
      ans++;
      success--;
    };
    tot--;
  };
  return ans;
};




void Data::subsample(unsigned int size){
  if(!sample){
    sample=new unsigned int[n];
  };
  if(!jackknife){
    if(size<n){    
      for(unsigned int i=0;i<n;i++){
	*(sample+i)=0;
      };
      unsigned int rmx=n*(RAND_MAX/n);
      for(unsigned int i=0;i<size;i++){      
	unsigned int rnd=rand();
	while(rnd>rmx){
	  rnd=rand(); //redraw if too large.
	};
	rnd=rnd%n;
	(*(sample+rnd))++;      
      };  
    }else{
      unsigned int rc=size;
      for(unsigned int i=0;i<n;i++){
	*(sample+i)=rbinom_fast(rc,1/((long double)(n-i)));
	rc-=*(sample+i);
      };
    };
  }else{
    //jackknife Assume size<=n
    //if size>n selects all datapoints
    for(unsigned int i=0;i<n;i++){
      unsigned int rmx=(n-i)*(RAND_MAX/(n-i));
      unsigned int rnd=rand();
      while(rnd>rmx){
	rnd=rand(); //redraw if too large.
      };
      rnd=rnd%(n-i);
      if(rnd<size){
	*(sample+i)=1;
	size--;
      }else{
	*(sample+i)=0;
      };
    };      
  };
};

void Data::clearsubsample(unsigned int size){
  if(sample){
    delete[] sample;
  };
  sample=NULL;
};


void Frequency_Data::subsample(unsigned int size){
  if(!sample){
    sample=new unsigned int[n];
  };
  if(jackknife){
    unsigned int *cumcount=new unsigned int[n];
    *(cumcount+n-1)=*(freq+n-1);
    for(unsigned int i=n-1;i>0;i--){
      *(cumcount+i-1)=*(cumcount+i)+*(freq+i-1);
    };    
    //    unsigned int tc=*(cumcount);
    //    unsigned int rmx=tc*(RAND_MAX/tc);
    unsigned int rc=size;
    for(unsigned int i=0;i<n;i++){
      *(sample+i)=rhgeom(rc,*(freq+i),*(cumcount+i));
      rc-=*(sample+i);
    };
    delete[] cumcount;
  }else{
    if(size<n){
      unsigned int *cumcount=new unsigned int[n];
      for(unsigned int i=0;i<n;i++){
	*(sample+i)=0;
	if(i==0){
	  *cumcount=*freq;
	}else{
	  *(cumcount+i)=*(cumcount+i-1)+*(freq+i);
	};
      };
      unsigned int tc=*(cumcount+n-1);
      unsigned int rmx=tc*(RAND_MAX/tc);
      for(unsigned int i=0;i<size;i++){
	unsigned int rnd=rand();
	while(rnd>rmx){
	  rnd=rand(); //redraw if too large.
	};
	rnd=rnd%tc;
	if(rnd<*cumcount){
	  (*sample)++;
	}else{
	  unsigned int upper=n;
	  unsigned int lower=0;
	  while(upper-lower>1){
	    unsigned int mid=(upper+lower)/2;
	    if(*(cumcount+mid)<=rnd){
	      lower=mid;
	    }else{
	      upper=mid;
	    };
	  };
	  (*(sample+upper))++;
	};
      };  
      delete[] cumcount;
    }else{
      unsigned int *cumcount=new unsigned int[n];
      *(cumcount+n-1)=*(freq+n-1);
      for(unsigned int i=n-1;i>0;i--){
	*(cumcount+i-1)=*(cumcount+i)+*(freq+i-1);
      };    
      //    unsigned int tc=*(cumcount);
      //    unsigned int rmx=tc*(RAND_MAX/tc);
      unsigned int rc=size;
      for(unsigned int i=0;i<n;i++){
	*(sample+i)=rbinom_fast(rc,*(freq+i)/((long double)*(cumcount+i)));
	rc-=*(sample+i);
      };
      delete[] cumcount;
    };
  };
};


const void* Continuous_Frequency_Data::getvalues() const {
  return data;
};


const unsigned int *Continuous_Frequency_Data::getfreqs() const {
  return (sample?sample:freq);
};


const void* Discrete_Frequency_Data::getvalues() const {
  return data;
};


const unsigned int *Discrete_Frequency_Data::getfreqs() const {
  return (sample?sample:freq);
};


const void* Continuous_Individual_Data::getvalues() const {
  return data;
};


const unsigned int *Continuous_Individual_Data::getfreqs() const {
  return sample;
};


const void* Discrete_Individual_Data::getvalues() const {
  return data;
};


const unsigned int *Discrete_Individual_Data::getfreqs() const {
  return sample;
};



const void* Continuous_Grouped_Data::getvalues() const {
  return cutoffs;
};


const unsigned int *Continuous_Grouped_Data::getfreqs() const {
  return (sample?sample:freq);
};


const void* Discrete_Grouped_Data::getvalues() const {
  return cutoffs;
};


const unsigned int *Discrete_Grouped_Data::getfreqs() const {
  return (sample?sample:freq);
};


Data* new_data(const datatype &type){
  if(type.type_of_data&(type.type_of_data-1)){
    //Not valid - data type should be a power of 2
    return NULL;
  };
  if(type.storage&(type.storage-1)){
    //Not valid - data type should be a power of 2
    return NULL;
  };
  switch(type.type_of_data){
  case DISCRETE_DATA:
    switch(type.storage){
    case LIST:
      return new Discrete_Individual_Data();
    case FREQUENCY_TABLE:
      return new Discrete_Frequency_Data();
    case GROUPED:
      return new Discrete_Grouped_Data();
    default:
      return NULL;
    };
  case CONTINUOUS_DATA:
    switch(type.storage){
    case LIST:
      return new Continuous_Individual_Data();
    case FREQUENCY_TABLE:
      return new Continuous_Frequency_Data();
    case GROUPED:
      return new Continuous_Grouped_Data();
    default:
      return NULL;
    };
  default:
    return NULL;
  };
};


void Discrete_Individual_Data::read(std::istream& in){
  long double rd;
  in>>rd;
  uill datlist(rd);
  while(in.good()&&!in.eof()){
    in>>rd;
    if(in.good()&&!in.eof()){
      datlist.insert(rd);
    };
  };
  
  n=datlist.length();
  data=new unsigned int[n];
  datlist.write(data);
  
  this->sort();
};

void Discrete_Frequency_Data::read(std::istream& in){
    long double rd;
    unsigned int count;
    in>>rd;
    in>>count;
    uictll datlist(rd,count);
    while(in.good()&&!in.eof()){
      in>>rd;
      in>>count;
      if(in.good()&&!in.eof()){
	datlist.insert(rd,count);
      };
    };
    
    n=datlist.length();
    data=new unsigned int[n];
    freq=new unsigned int[n];
    unsigned int *tempdata=new unsigned int[n];
    unsigned int *tempcount=new unsigned int[n];
    datlist.write(tempdata,tempcount);
    for(unsigned int i=0;i<n;i++){
      *(freq+i)=*(tempcount+n-1-i);
      *(data+i)=*(tempdata+n-1-i);
    };    
    delete[] tempdata;
    delete[] tempcount;
    this->sort();
};

void Discrete_Grouped_Data::read(std::istream& in){
    unsigned int rd;
    unsigned int count;
    in>>rd;
    char c;
    in>>c;
    if(c=='-'){
      in>>rd;
    };
    in>>count;
    uictll datlist(rd,count);
    while(in.good()&&!in.eof()){
      in>>rd;
      in>>c;
      if(c=='-'){
	in>>rd;
      };
      in>>count;
      if(in.good()&&!in.eof()){
	datlist.insert(rd,count);
      };
    };
    
    n=datlist.length();
    cutoffs=new unsigned int[n];
    freq=new unsigned int[n];
    unsigned int *tempdata=new unsigned int[n];
    unsigned int *tempcount=new unsigned int[n];
    datlist.write(tempdata,tempcount);
    for(unsigned int i=0;i<n;i++){
      *(freq+i)=*(tempcount+n-1-i);
      *(cutoffs+i)=*(tempdata+n-1-i);
    };    
    delete[] tempdata;
    delete[] tempcount;  
    this->sort();
};

void Continuous_Frequency_Data::read(std::istream& in){
    long double rd;
    unsigned int count;
    in>>rd;
    in>>count;
    dblctll datlist(rd,count);
    while(in.good()&&!in.eof()){
      in>>rd;
      in>>count;
      if(in.good()&&!in.eof()){
	datlist.insert(rd,count);
      };
    };
    
    n=datlist.length();
    data=new long double[n];
    freq=new unsigned int[n];
    long double *tempdata=new long double[n];
    unsigned int *tempcount=new unsigned int[n];
    datlist.write(tempdata,tempcount);
    for(unsigned int i=0;i<n;i++){
      *(freq+i)=*(tempcount+n-1-i);
      *(data+i)=*(tempdata+n-1-i);
    };    
    delete[] tempdata;
    delete[] tempcount;
    this->sort();
};

void Continuous_Individual_Data::read(std::istream& in){
  long double rd;
  in>>rd;
  dblll datlist(rd);
  while(in.good()&&!in.eof()){
    in>>rd;
    if(in.good()&&!in.eof()){
      datlist.insert(rd);
    };
  };
    
  n=datlist.length();
  data=new long double[n];
  datlist.write(data);
  
  this->sort();
};

void Continuous_Grouped_Data::read(std::istream& in){
  long double rd;
  unsigned int count;
  in>>rd;
  char c;
  in>>c;
  if(c=='-'){
    in>>rd;
  };
  in>>count;
  dblctll datlist(rd,count);
  while(in.good()&&!in.eof()){
    in>>rd;
    in>>c;
    if(c=='-'){
      in>>rd;
    };
    in>>count;
    if(in.good()&&!in.eof()){
      datlist.insert(rd,count);
    };
  };
     
  n=datlist.length();
  cutoffs=new long double[n];
  freq=new unsigned int[n];
  long double *tempdata=new long double[n];
  unsigned int *tempcount=new unsigned int[n];
  datlist.write(tempdata,tempcount);
  for(unsigned int i=0;i<n;i++){
    *(freq+i)=*(tempcount+n-1-i);
    *(cutoffs+i)=*(tempdata+n-1-i);
  };    
  delete[] tempdata;
  delete[] tempcount;
  this->sort();
};


void Discrete_Individual_Data::concat(const Data *cdat){
  if(cdat){
    if(cdat->get_type().type_of_data&DISCRETE_DATA){
      switch(cdat->get_type().storage){
      case LIST:
	{
	  unsigned int *newdata=new unsigned int[n+cdat->n];
	  unsigned int i=0;
	  unsigned int j=0;
	  const unsigned int*cddata=(const unsigned int*)cdat->getvalues();
	  for(;i<n&&j<cdat->n;){
	    if(*(data+i)<*(cddata+j)){
	      *(newdata+i+j)=*(data+i);
	      i++;
	    }else{
	      *(newdata+i+j)=*(cddata+j);
	      j++;
	    };
	  };
	  for(;i<n;i++){
	    *(newdata+i+j)=*(data+i);	      
	  };
	  for(;j<cdat->n;j++){
	    *(newdata+i+j)=*(cddata+j);
	  };
	  n+=cdat->n;
	  delete[] data;
	  qsort(newdata,n,sizeof(unsigned int),compareUnsignedInt);
	  data=newdata;	  
	  if(sample){
	    delete[] sample;
	  };
	  return;
	};
      case FREQUENCY_TABLE:
	{
	  cout<<"Attempting to concatenate list and frequency table.\n";
	  cout<<"May result in unacceptably long data lists.\n";
	  unsigned int tot=0;
	  const unsigned int *frqs=cdat->getfreqs();
	  for(unsigned int i=0;i<cdat->n;i++){
	    tot+=*(frqs+i);
	  };
	  unsigned int *newdata=new unsigned int[n+tot];
	  unsigned int i=0;
	  unsigned int j=0;
	  unsigned int jj=0;
	  const unsigned int*cddata=(const unsigned int*)cdat->getvalues();
	  for(;i<n&&j<cdat->n;){
	    if(*(data+i)<*(cddata+j)){
	      *(newdata+i+jj)=*(data+i);
	      i++;
	    }else{
	      for(unsigned int k=0;k<*(frqs+j);k++){
		*(newdata+i+jj)=*(cddata+j);
		jj++;
	      };
	      j++;
	    };
	  };
	  for(;i<n;i++){
	    *(newdata+i+jj)=*(data+i);	      
	  };
	  for(;j<cdat->n;j++){
	    for(unsigned int k=0;k<*(frqs+j);k++){
	      *(newdata+i+jj)=*(cddata+j);
	      jj++;
	    };
	  };
	  n+=cdat->n;
	  delete[] data;
	  qsort(newdata,n,sizeof(unsigned int),compareUnsignedInt);
	  data=newdata;	  
	  if(sample){
	    delete[] sample;
	  };
	  return;
	};
      default:
	cout<<"Unable to concatenate data.\n";
	exit(0);
      };
    }else{
      cout<<"Attempting to concatenate discrete and non-discrete data.\n";
      exit(0);
    };   
  };
};

void Discrete_Frequency_Data::concat(const Data *cdat){
  if(cdat){
    if(cdat->get_type().type_of_data&DISCRETE_DATA){
      switch(cdat->get_type().storage){
      case LIST:
      case FREQUENCY_TABLE:
	{
	  const unsigned int *frqs=cdat->getfreqs();
	  const unsigned int*cddata=(const unsigned int*)cdat->getvalues();
	  unsigned int newn=n+cdat->n;
	  unsigned int *newdata=new unsigned int[newn];
	  unsigned int *newfreqs=new unsigned int[newn];
	  unsigned int pos=0;
	  unsigned int i=0;
	  unsigned int j=0;
	  if(frqs){
	    for(;i<n&&j<cdat->n;pos++){
	      if(*(data+i)<*(cddata+j)){
		*(newdata+pos)=*(data+i);
		*(newfreqs+pos)=*(freq+i);
		for(i++;i<n&&*(data+i)==*(newdata+pos);i++){
		  *(newfreqs+pos)+=*(freq+i);
		};
	      }else if(*(cddata+j)<*(data+i) ){
		*(newdata+pos)=*(cddata+j);
		*(newfreqs+pos)=*(frqs+j);
		for(j++;j<cdat->n&&*(cddata+j)==*(newdata+pos);j++){
		  *(newfreqs+pos)+=*(frqs+j);
		};
	      }else{
		*(newdata+pos)=*(cddata+j);
		*(newfreqs+pos)=*(freq+i)+*(frqs+j);
		for(i++;i<n&&*(data+i)==*(newdata+pos);i++){
		  *(newfreqs+pos)+=*(freq+i);
		};
		for(j++;j<cdat->n&&*(cddata+j)==*(newdata+pos);j++){
		  *(newfreqs+pos)+=*(frqs+j);
		};
	      };
	    };
	    for(;i<n;i++){
	      *(newdata+pos)=*(data+i);
	      *(newfreqs+pos)=*(freq+i);
	      for(i++;i<n&&*(data+i)==*(newdata+pos);i++){
		*(newfreqs+pos)+=*(freq+i);
	      };	      
	      pos++;
	    };
	    for(;j<cdat->n;j++){
	      *(newdata+i+j)=*(cddata+j);
	      *(newfreqs+pos)=*(frqs+j);
	      for(j++;j<cdat->n&&*(cddata+j)==*(newdata+pos);i++){
		*(newfreqs+pos)+=*(freq+i);
	      };
	      pos++;
	    };
	    n=pos;
	    delete[] data;
	    delete[] freq;
	    data=newdata;	  
	    freq=newfreqs;
	    if(sample){
	      delete[] sample;
	    };
	    return;
	  }else{
	    for(;i<n&&j<cdat->n;pos++){
	      if(*(data+i)<*(cddata+j)){
		*(newdata+pos)=*(data+i);
		*(newfreqs+pos)=*(freq+i);
		for(i++;i<n&&*(data+i)==*(newdata+pos);i++){
		  *(newfreqs+pos)+=*(freq+i);
		};
	      }else if(*(cddata+j)<*(data+i) ){
		*(newdata+pos)=*(cddata+j);
		*(newfreqs+pos)=1;
		for(j++;j<cdat->n&&*(cddata+j)==*(newdata+pos);j++){
		  (*(newfreqs+pos))++;
		};
	      }else{
		*(newdata+pos)=*(cddata+j);
		*(newfreqs+pos)=*(freq+i)+1;
		for(i++;i<n&&*(data+i)==*(newdata+pos);i++){
		  *(newfreqs+pos)+=*(freq+i);
		};
		for(j++;j<cdat->n&&*(cddata+j)==*(newdata+pos);j++){
		  (*(newfreqs+pos))++;
		};
	      };
	    };
	    for(;i<n;i++){
	      *(newdata+pos)=*(data+i);
	      *(newfreqs+pos)=*(freq+i);
	      for(i++;i<n&&*(data+i)==*(newdata+pos);i++){
		*(newfreqs+pos)+=*(freq+i);
	      };	      
	      pos++;
	    };
	    for(;j<cdat->n;j++){
	      *(newdata+i+j)=*(cddata+j);
	      *(newfreqs+pos)=1;
	      for(j++;j<cdat->n&&*(cddata+j)==*(newdata+pos);i++){
		(*(newfreqs+pos))++;
	      };
	      pos++;
	    };
	    n=pos;
	    delete[] data;
	    delete[] freq;
	    data=newdata;	  
	    freq=newfreqs;
	    if(sample){
	      delete[] sample;
	    };
	    return;
	  };
	};
      default:
	cout<<"Unable to concatenate data.\n";
	exit(0);
      };
    }else{
      cout<<"Attempting to concatenate discrete and non-discrete data.\n";
      exit(0);
    };   
  };
};

void Continuous_Individual_Data::concat(const Data *cdat){
  if(cdat){
    if(cdat->get_type().type_of_data&CONTINUOUS_DATA){
      switch(cdat->get_type().storage){
      case LIST:
	{
	  long double *newdata=new long double[n+cdat->n];
	  unsigned int i=0;
	  unsigned int j=0;
	  const long double*cddata=(const long double*)cdat->getvalues();
	  for(;i<n&&j<cdat->n;){
	    if(*(data+i)<*(cddata+j)){
	      *(newdata+i+j)=*(data+i);
	      i++;
	    }else{
	      *(newdata+i+j)=*(cddata+j);
	      j++;
	    };
	  };
	  for(;i<n;i++){
	    *(newdata+i+j)=*(data+i);	      
	  };
	  for(;j<cdat->n;j++){
	    *(newdata+i+j)=*(cddata+j);
	  };
	  n+=cdat->n;
	  delete[] data;
	  data=newdata;	  
	  if(sample){
	    delete[] sample;
	  };
	  return;
	};
      case FREQUENCY_TABLE:
	{
	  cout<<"Attempting to concatenate list and frequency table.\n";
	  cout<<"May result in unacceptably long data lists.\n";
	  unsigned int tot=0;
	  const unsigned int *frqs=cdat->getfreqs();
	  for(unsigned int i=0;i<cdat->n;i++){
	    tot+=*(frqs+i);
	  };
	  long double *newdata=new long double[n+tot];
	  unsigned int i=0;
	  unsigned int j=0;
	  unsigned int jj=0;
	  const long double*cddata=(const long double*)cdat->getvalues();
	  for(;i<n&&j<cdat->n;){
	    if(*(data+i)<*(cddata+j)){
	      *(newdata+i+jj)=*(data+i);
	      i++;
	    }else{
	      for(unsigned int k=0;k<*(frqs+j);k++){
		*(newdata+i+jj)=*(cddata+j);
		jj++;
	      };
	      j++;
	    };
	  };
	  for(;i<n;i++){
	    *(newdata+i+jj)=*(data+i);	      
	  };
	  for(;j<cdat->n;j++){
	    for(unsigned int k=0;k<*(frqs+j);k++){
	      *(newdata+i+jj)=*(cddata+j);
	      jj++;
	    };
	  };
	  n+=cdat->n;
	  delete[] data;
	  data=newdata;	  
	  if(sample){
	    delete[] sample;
	  };
	  return;
	};
      default:
	cout<<"Unable to concatenate data.\n";
	exit(0);
      };
    }else{
      cout<<"Attempting to concatenate continuous and non-continuous data.\n";
      exit(0);
    };   
  };
};

void Continuous_Frequency_Data::concat(const Data *cdat){
  if(cdat){
    if(cdat->get_type().type_of_data&CONTINUOUS_DATA){
      switch(cdat->get_type().storage){
      case LIST:
      case FREQUENCY_TABLE:
	{
	  const unsigned int *frqs=cdat->getfreqs();
	  const long double*cddata=(const long double*)cdat->getvalues();
	  unsigned int newn=n+cdat->n;
	  long double *newdata=new long double[newn];
	  unsigned int *newfreqs=new unsigned int[newn];
	  unsigned int pos=0;
	  unsigned int i=0;
	  unsigned int j=0;
	  if(frqs){
	    for(;i<n&&j<cdat->n;pos++){
	      if(*(data+i)<*(cddata+j)){
		*(newdata+pos)=*(data+i);
		*(newfreqs+pos)=*(freq+i);
		for(i++;i<n&&*(data+i)==*(newdata+pos);i++){
		  *(newfreqs+pos)+=*(freq+i);
		};
	      }else if(*(cddata+j)<*(data+i) ){
		*(newdata+pos)=*(cddata+j);
		*(newfreqs+pos)=*(frqs+j);
		for(j++;j<cdat->n&&*(cddata+j)==*(newdata+pos);j++){
		  *(newfreqs+pos)+=*(frqs+j);
		};
	      }else{
		*(newdata+pos)=*(cddata+j);
		*(newfreqs+pos)=*(freq+i)+*(frqs+j);
		for(i++;i<n&&*(data+i)==*(newdata+pos);i++){
		  *(newfreqs+pos)+=*(freq+i);
		};
		for(j++;j<cdat->n&&*(cddata+j)==*(newdata+pos);j++){
		  *(newfreqs+pos)+=*(frqs+j);
		};
	      };
	    };
	    for(;i<n;i++){
	      *(newdata+pos)=*(data+i);
	      *(newfreqs+pos)=*(freq+i);
	      for(i++;i<n&&*(data+i)==*(newdata+pos);i++){
		*(newfreqs+pos)+=*(freq+i);
	      };	      
	      pos++;
	    };
	    for(;j<cdat->n;j++){
	      *(newdata+i+j)=*(cddata+j);
	      *(newfreqs+pos)=*(frqs+j);
	      for(j++;j<cdat->n&&*(cddata+j)==*(newdata+pos);i++){
		*(newfreqs+pos)+=*(freq+i);
	      };
	      pos++;
	    };
	    n=pos;
	    delete[] data;
	    delete[] freq;
	    data=newdata;	  
	    freq=newfreqs;
	    if(sample){
	      delete[] sample;
	    };
	    return;
	  }else{
	    for(;i<n&&j<cdat->n;pos++){
	      if(*(data+i)<*(cddata+j)){
		*(newdata+pos)=*(data+i);
		*(newfreqs+pos)=*(freq+i);
		for(i++;i<n&&*(data+i)==*(newdata+pos);i++){
		  *(newfreqs+pos)+=*(freq+i);
		};
	      }else if(*(cddata+j)<*(data+i) ){
		*(newdata+pos)=*(cddata+j);
		*(newfreqs+pos)=1;
		for(j++;j<cdat->n&&*(cddata+j)==*(newdata+pos);j++){
		  (*(newfreqs+pos))++;
		};
	      }else{
		*(newdata+pos)=*(cddata+j);
		*(newfreqs+pos)=*(freq+i)+1;
		for(i++;i<n&&*(data+i)==*(newdata+pos);i++){
		  *(newfreqs+pos)+=*(freq+i);
		};
		for(j++;j<cdat->n&&*(cddata+j)==*(newdata+pos);j++){
		  (*(newfreqs+pos))++;
		};
	      };
	    };
	    for(;i<n;i++){
	      *(newdata+pos)=*(data+i);
	      *(newfreqs+pos)=*(freq+i);
	      for(i++;i<n&&*(data+i)==*(newdata+pos);i++){
		*(newfreqs+pos)+=*(freq+i);
	      };	      
	      pos++;
	    };
	    for(;j<cdat->n;j++){
	      *(newdata+i+j)=*(cddata+j);
	      *(newfreqs+pos)=1;
	      for(j++;j<cdat->n&&*(cddata+j)==*(newdata+pos);i++){
		(*(newfreqs+pos))++;
	      };
	      pos++;
	    };
	    n=pos;
	    delete[] data;
	    delete[] freq;
	    data=newdata;	  
	    freq=newfreqs;
	    if(sample){
	      delete[] sample;
	    };
	    return;
	  };
	};
      default:
	cout<<"Unable to concatenate data.\n";
	exit(0);
      };
    }else{
      cout<<"Attempting to concatenate discrete and non-discrete data.\n";
      exit(0);
    };   
  };
};

void Continuous_Grouped_Data::concat(const Data *cdat){
  if(cdat){
    if(cdat->get_type().type_of_data&CONTINUOUS_DATA){
      switch(cdat->get_type().storage){
      case GROUPED:
      case LIST:
      case FREQUENCY_TABLE:
	{
	};
      default:
	cout<<"Unable to concatenate data.\n";
	exit(0);
      };
    }else{
      cout<<"Attempting to concatenate continuous and non-continuous data.\n";
      exit(0);
    };   
  };
};


void Discrete_Grouped_Data::concat(const Data *cdat){
  if(cdat){
    if(cdat->get_type().type_of_data&DISCRETE_DATA){
    }else{
      cout<<"Attempting to concatenate discrete and non-discrete data.\n";
      exit(0);
    };   
  };
};





void Discrete_Individual_Data::sort(){
  qsort(data,n,sizeof(unsigned int),compareUnsignedInt);    
};

void Continuous_Individual_Data::sort(){
  qsort(data,n,sizeof(long double),compareLongDouble);    
};

void Discrete_Frequency_Data::sort(){
    // Assume frequency table is already sorted
};


void Continuous_Frequency_Data::sort(){
    // Assume frequency table is already sorted
};

void Continuous_Grouped_Data::sort(){
    // Assume frequency table is already sorted
};


void Discrete_Grouped_Data::sort(){
    // Assume frequency table is already sorted
};




/*

   Copyright 2016 Toby Kenney

   This file is part of Adequate Bootstrap.

   Adequate Bootstrap is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Adequate Bootstrap is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Adequate Bootstrap.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "distributions.h"
#include "distortion.h"
#include <cstdlib>
#include <cstring>
#include <cstdio>
#include <exception>

unsigned int rbinom_fast(unsigned int n,long double p);

using namespace std;


class contamination: public Distortion{
  const Distribution *contaminating_distribution;
  long double probability;
public:
  contamination():contaminating_distribution(NULL),probability(0){};
  contamination(long double p,const Distribution *cd):contaminating_distribution(cd),probability(p){};
  void set_contamination(const Distribution *contam);
  void set_prob(long double pr);
  virtual Data *simulate(unsigned int simsize,const Distribution &truedist);
};

Data *contamination::simulate(unsigned int n,const Distribution &main){
  if(contaminating_distribution==NULL){
    return NULL;
  };
  unsigned int numcont=rbinom_fast(n,probability);  
  Data *dattrue=main.simulate(n-numcont);
  Data *datcontam=contaminating_distribution->simulate(numcont);
  dattrue->concat(datcontam);
  delete datcontam;
  return dattrue;
};



class no_distortion: public Distortion{
  virtual Data *simulate(unsigned int simsize,const Distribution &truedist);
};

Data *no_distortion::simulate(unsigned int simsize,const Distribution &truedist){
  return truedist.simulate(simsize);
};





class sample_bias_step: public Distortion{
unsigned int ncut;
long double *cuts;
long double *cum_probs;
public:
  sample_bias_step():ncut(0),cuts(NULL),cum_probs(NULL){};
  sample_bias_step(unsigned int nc,long double *cts,long double *prbs):ncut(0),cuts(cts),cum_probs(prbs){};
  virtual Data *simulate(unsigned int simsize,const Distribution &truedist);
};



Data *sample_bias_step::simulate(unsigned int n,const Distribution &main){
  //Need to fix this!
  //Doesn't work at all.

  if(cum_probs==NULL||cuts==NULL){
    return NULL;
  };
  long double *ans=new long double[n];
  //Assume probs sum to 1.
  unsigned int *nsim=new unsigned int[ncut+1];
  for(unsigned int i=0;i<=ncut;i++){
    *(nsim+i)=0;
  };
  for(unsigned int i=0;i<n;i++){
    long double rnd=((long double)rand())/RAND_MAX;
    unsigned int j=0;
    for(;j<ncut&&*(cum_probs+j)<rnd;j++);
    (*(nsim+j))++;
  };
  unsigned int *nsimdone=new unsigned int[ncut+1];
  for(unsigned int i=0;i<=ncut;i++){
    *(nsimdone+i)=0;
  };
  long double **bucket=new long double *[ncut+1];
  *bucket=ans;
  for(unsigned int i=0;i<ncut;i++){
    *(bucket+i+1)=*(bucket+i)+*(nsim+i);
  };
  unsigned int remaintosim=n;
  Data *dat=main.simulate(remaintosim);    
  while(remaintosim){
    for(unsigned int i=0;i<n;){
      unsigned int sim=0;
      unsigned int j=0;
      for(;j<ncut&&*(cuts+j)<sim;j++);    
      if(*(nsimdone+j)<*(nsim+j)){
	remaintosim--;
	i++;
	*(*(bucket+j)+(*(nsimdone+j))++)=sim;
      };
    };
  };
  
  delete[] bucket;
  delete[] nsim;
  delete[] nsimdone;

  return dat;
};




class distortion_formatting_error: public std::exception{
  static const char*const header;
  char * message;
public:
  distortion_formatting_error(const char *const mess){message=new char[strlen(mess)+1];strcpy(message,mess);};
  distortion_formatting_error(const char *const mess,const char*values,const char *end){message=new char[strlen(mess)+strlen(values)+strlen(end)+1];strcpy(message,mess);strcat(message,values);strcat(message,end);};
  virtual const char* what() const throw() {char *full_message=new char[strlen(header)+strlen(message)+1];strcpy(full_message,header);strcat(full_message,message);return full_message;}; 
};

const char*const distortion_formatting_error::header="Distortion formatting error: ";



Distortion* Read_distortion(unsigned int &argno,unsigned int argc,char *argv[]){
  if(argno>=argc){
    throw distortion_formatting_error("No Distortion specified.\n");
    return NULL;
  };
  char *distorttype=argv[argno++];
  if(!strcmp(distorttype,"contaminate")){
    if(argno>=argc){
      throw distortion_formatting_error("No contaminating probability or distribution specified.\n");
      return NULL;
    };
    long double contamprob=atof(argv[argno++]);
    if(argno>=argc){
      throw distortion_formatting_error("No contaminating distribution specified.\n");
      return NULL;
    };
    Distribution *contamDist=read_distribution(argc,argv,argno);
    if(contamDist==NULL){
      //This shouldn't happen as read_distribution should throw the exception
      throw distortion_formatting_error("Problem reading contaminating distribution.\n");
      return NULL;
    };
    return new contamination(contamprob,contamDist);
  }else if(!strcmp(distorttype,"other")){
    //User specified simulation distribution.
    //Treated as 100% contamination.
    if(argno>=argc){
      throw distortion_formatting_error("No Alternative distribution specified for distortion type \"other\".\n");
    };
    long double contamprob=1;
    Distribution *contamDist=read_distribution(argc,argv,argno);
    return new contamination(contamprob,contamDist);
  }else if(!strcmp(distorttype,"no_distortion")){
    return new no_distortion();
  }else if(!strcmp(distorttype,"sample_bias")){
    if(argno>=argc){
      throw distortion_formatting_error("No sample bias scheme specified.\n");
    };    
    char *sbtype=argv[argno++];
    if(!strcmp(sbtype,"step")){
      if(argno>=argc){
	throw distortion_formatting_error("No sample bias steps specified.\n");
      };        
      char *cuts=argv[argno++];  
      if(argno>=argc){
	throw distortion_formatting_error("No sample bias probabilities specified.\n");
      };        
      char *cutprobs=argv[argno++];  
      unsigned int nc=1;
      unsigned int ncp=1;
      for(unsigned int i=0;*(cuts+i);i++){
	if(*(cuts+i)==','){
	  nc++;
	};
      };
      for(unsigned int i=0;*(cutprobs+i);i++){
	if(*(cutprobs+i)==','){
	  ncp++;
	};
      };
      if(ncp!=nc+1){
	char *message=new char[75];//should be enough.
	sprintf(message,"Number of cuts=%d, Number of probabilities=%d",nc,ncp);
	throw distortion_formatting_error("for sample bias type step, Number of intervals and number of probabilities should match \n",message,"\n\t\tNote: Adequate bootstrap has no localisation - you must use \'.\' as decimal separator and \',\' to separate arguments.\n");
      };
      
      long double *sample_bias_step_cuts=new long double[nc];
      long double *sample_bias_step_cum_probs=new long double[nc+1];

      *sample_bias_step_cuts=atof(cuts);
      nc=1;
      for(unsigned int i=0;*(cuts+i+1);i++){
	if(*(cuts+i)==','){
	  *(sample_bias_step_cuts+nc++)=atof(cuts+i+1);
	};
      };
      ncp=1;
      *(sample_bias_step_cum_probs)=atof(cutprobs);
      long double totprob=*(sample_bias_step_cum_probs);
      for(unsigned int i=0;*(cutprobs+i+1);i++){
	if(*(cutprobs+i)==','){
	  totprob+=atof(cutprobs+i+1);
	  *(sample_bias_step_cum_probs+ncp++)=atof(cutprobs+i+1);
	};
      };      
      long double cumprob=*(sample_bias_step_cum_probs);      
      *(sample_bias_step_cum_probs)=cumprob/totprob;
      for(unsigned int j=1;j<=nc;j++){
	cumprob+=*(sample_bias_step_cum_probs+j);      
	*(sample_bias_step_cum_probs+j)=cumprob/totprob;	
      };
      return new sample_bias_step(nc,sample_bias_step_cuts,sample_bias_step_cum_probs);
    }else{
      throw distortion_formatting_error("Sample bias method \"",sbtype,"\" not currently supported.\n");
    };
  }else{
    throw distortion_formatting_error("Distortion type \"",distorttype,"\" not currently supported.\n");;
  };  
};


//#include "distributions.h"

#include <iostream>
#include <cstdlib>
#include <cstring>
#include <cmath>

using namespace std;

class NP_mean: public NPestimator{
public:  
  NP_mean():NPestimator(datatype(DISCRETE_DATA|CONTINUOUS_DATA,LIST|FREQUENCY_TABLE)){};
  datatype type; //What types of data can this test be applied to
  virtual long double evaluate(const Data& dat);
};


class NP_sd: public NPestimator{
public:  
  NP_sd():NPestimator(datatype(DISCRETE_DATA|CONTINUOUS_DATA,LIST|FREQUENCY_TABLE)){};
  virtual long double evaluate(const Data& dat);
};


class NP_VaR: public NPestimator{
  long double VaR_at;
public:
  NP_VaR():NPestimator(datatype(DISCRETE_DATA|CONTINUOUS_DATA,LIST|FREQUENCY_TABLE)),VaR_at(0.05){};
  NP_VaR(long double at): NPestimator(datatype(DISCRETE_DATA|CONTINUOUS_DATA,LIST|FREQUENCY_TABLE)),VaR_at(at){};  
  virtual long double evaluate(const Data& dat);
};


class NP_Gini: public NPestimator{
public:
  NP_Gini():NPestimator(datatype(DISCRETE_DATA|CONTINUOUS_DATA,LIST|FREQUENCY_TABLE)){};  
  virtual long double evaluate(const Data& dat);
};


long double NP_mean::evaluate(const Data& dat){
  if(dat.get_type().type_of_data&DISCRETE_DATA){
    const unsigned int *values=(const unsigned int *)dat.getvalues();
    const unsigned int *freqs=dat.getfreqs();
    if(freqs!=NULL){
      long double sum=0;
      unsigned int tot=0;
      for(unsigned int i=0;i<dat.n;i++){
	sum+=*(values+i)*(*(freqs+i));
	tot+=*(freqs+i);
      };
      return sum/tot;            
    }else{
      long double sum=0;
      for(unsigned int i=0;i<dat.n;i++){
	sum+=*(values+i);
      };
      return sum/dat.n;      
    };
  }else{
    const long double *values=(const long double *)dat.getvalues();
    const unsigned int *freqs=dat.getfreqs();
    if(freqs){
      long double sum=0;
      unsigned int tot=0;
      for(unsigned int i=0;i<dat.n;i++){
	sum+=*(values+i)*(*(freqs+i));
	tot+=*(freqs+i);
      };
      return sum/tot;            
    }else{
      long double sum=0;
      for(unsigned int i=0;i<dat.n;i++){
	sum+=*(values+i);
      };
      return sum/dat.n;      
    };
  };  
};



long double NP_sd::evaluate(const Data& dat){
  const unsigned int *freqs=dat.getfreqs();
  long double sum=0;
  long double ssq=0;
  if(freqs!=NULL){
    unsigned int tot=0;
    if(dat.get_type().type_of_data&DISCRETE_DATA){
      const unsigned int *values=(const unsigned int *)dat.getvalues();
      for(unsigned int i=0;i<dat.n;i++){
	sum+=*(values+i)*(*(freqs+i));
	ssq+=*(values+i)*(*(freqs+i))*(*(values+i));
	tot+=*(freqs+i);
      };
    }else{
      const long double *values=(const long double *)dat.getvalues();
      for(unsigned int i=0;i<dat.n;i++){
	sum+=*(values+i)*(*(freqs+i));
	ssq+=*(values+i)*(*(freqs+i))*(*(values+i));
	tot+=*(freqs+i);
      };
    };
    return sqrt(ssq/tot-(sum/tot)*(sum/tot));            
  }else{
    if(dat.get_type().type_of_data&DISCRETE_DATA){
      const unsigned int *values=(const unsigned int *)dat.getvalues();
      for(unsigned int i=0;i<dat.n;i++){
	sum+=*(values+i);
	ssq+=*(values+i)*(*(values+i));
      };
    }else{
      const long double *values=(const long double *)dat.getvalues();
      for(unsigned int i=0;i<dat.n;i++){
	sum+=*(values+i);
	ssq+=*(values+i)*(*(values+i));
      };
    };
    return sqrt(ssq/dat.n-(sum/dat.n)*(sum/dat.n));             
  };
};


long double NP_VaR::evaluate(const Data& dat){
  const unsigned int *samp=dat.getfreqs();
  unsigned int i=dat.n*VaR_at;
  if(samp){
    unsigned int tot=0;
    for(unsigned int j=0;j<dat.n;j++){
      tot+=*(samp+j);
    };  
    unsigned int pos=tot*VaR_at;
    unsigned int cumtot=0;
    for(i=0;cumtot<pos;cumtot+=*(samp+i++));
    i--;
  };
  if(dat.get_type().type_of_data&DISCRETE_DATA){
    return *((unsigned int *)dat.getvalues()+i);
  }else{
    return *((long double *)dat.getvalues()+i);
  };
};


long double NP_Gini::evaluate(const Data& dat){
  unsigned int cumtot=0;
  long double U2=0;
  long double U1=0;
  const unsigned int *samp=dat.getfreqs();
  if(samp){
    if(dat.get_type().type_of_data&DISCRETE_DATA){
      unsigned int *data=(unsigned int *)dat.getvalues();
      for(unsigned int i=0;i<dat.n;cumtot+=*(samp+i++)){
	U1+=*(samp+i)*(*(data+i));
	U2+=((*(samp+i))*cumtot+(*(samp+i)*(*(samp+i)-1))/2)*(*(data+i));
      };
    }else{
      long double *data=(long double *)dat.getvalues();
      for(unsigned int i=0;i<dat.n;cumtot+=*(samp+i++)){
	U1+=*(samp+i)*(*(data+i));
	U2+=((*(samp+i))*cumtot+(*(samp+i)*(*(samp+i)-1))/2)*(*(data+i));
      };
    };
  }else{
    cumtot=dat.n;
    if(dat.get_type().type_of_data&DISCRETE_DATA){
      unsigned int *data=(unsigned int *)dat.getvalues();
      for(unsigned int i=0;i<dat.n;i++){
	U1+=(*(data+i));
	U2+=i*(*(data+i));
      };
    }else{
      long double *data=(long double *)dat.getvalues();
      for(unsigned int i=0;i<dat.n;i++){
	U1+=*(data+i);
	U2+=i*(*(data+i));
      };
    };
  };
  return 2*U2/U1/(cumtot-1)-1;
};


NPestimator *get_NP_estimator(const char *eq,const char *at){
  if(eq==NULL){
    cout<<"NP method unspecified. Ignoring NP setting.\n";
    return NULL;
  }else{	
    if(!strcmp(eq,"mean")){
      return new NP_mean();
    }else if(!strcmp(eq,"sd")){
      return new NP_sd();
    }else if(!strcmp(eq,"VaR")){
      long double atval=0.05;
      if(at!=NULL){
	atval=atof(at);
      };
      return new NP_VaR(atval);
    }else if(!strcmp(eq,"Gini")){
      return new NP_Gini();
    }else{
      cout<<"Nonparametric estimator \""<<eq<<"\" not currently supported. Ignoring option.\n";
      return NULL;
    };
  };
};

//*/


//Test code.
//Delete before final version.


/*
extern NPestimator *NPest;
void read_option(const char* const optname);

int main(int argc,char *argv[]){
  read_option(argv[1]);
  unsigned int test[]={3,5,3,5,3};

  long double testCont[]={3.4,5.1,3.5,4.8,3.9,4.7};

  unsigned int freqs[]={1,8,3,4,5,6};
  Discrete_Individual_Data dat(5,test);
  Continuous_Individual_Data datcon(6,testCont);
  Discrete_Frequency_Data datfr(5,freqs,test);
  Continuous_Frequency_Data datconfr(6,freqs,testCont);
  Discrete_Grouped_Data datgr(4,freqs,test);
  Continuous_Grouped_Data datcongr(6,freqs,testCont);


  if(NPest->compatible(dat)){
    long double me=NPest->evaluate(dat);
    cout<<me<<"\n";
  }else{
    cout<<"NP_estimator not available for this type of data.\n";
  };
  if(NPest->compatible(datcon)){
    long double me=NPest->evaluate(datcon);
    cout<<me<<"\n";
    datcon.subsample(1323523);
    me=NPest->evaluate(datcon);
    cout<<me<<"\n";    
  }else{
    cout<<"NP_estimator not available for this type of data.\n";
  };
  if(NPest->compatible(datfr)){
    long double me=NPest->evaluate(datfr);
    cout<<me<<"\n";
  }else{
    cout<<"NP_estimator not available for this type of data.\n";
  };
  if(NPest->compatible(datconfr)){
    long double me=NPest->evaluate(datconfr);
    cout<<me<<"\n";
    datconfr.subsample(132312);
    me=NPest->evaluate(datconfr);
    cout<<me<<"\n";    
  }else{
    cout<<"NP_estimator not available for this type of data.\n";
  };
  if(NPest->compatible(datgr)){
    long double me=NPest->evaluate(datgr);
    cout<<me<<"\n";
    datgr.subsample(132312);
    me=NPest->evaluate(datgr);
    cout<<me<<"\n";    
  }else{
    cout<<"NP_estimator not available for this type of data.\n";
  };
  if(NPest->compatible(datcongr)){
    long double me=NPest->evaluate(datcongr);
    cout<<me<<"\n";
    datcongr.subsample(132312);
    me=NPest->evaluate(datcongr);
    cout<<me<<"\n";    
  }else{
    cout<<"NP_estimator not available for this type of data.\n";
  };
};

*/

#include "distributions.h"

#include <iostream>
#include <fstream>
#include <cmath>
#include <boost/math/distributions/normal.hpp>
#include <boost/math/distributions/binomial.hpp>
#include <boost/math/distributions/non_central_f.hpp>
#include <boost/math/distributions/fisher_f.hpp>
#include <boost/random/mersenne_twister.hpp>
#include <boost/random/variate_generator.hpp>
#include <boost/generator_iterator.hpp>
#include <boost/random/binomial_distribution.hpp>
#include <boost/random/mersenne_twister.hpp>
#include <boost/random/normal_distribution.hpp>

#include <cstdlib>
#include <ctime>

using namespace std;


class AndersonDarling: public AdequacyTest{
private:
  long double critval;
public:
  AndersonDarling():AdequacyTest(datatype(CONTINUOUS_DATA|DISCRETE_DATA,LIST|FREQUENCY_TABLE|GROUPED)),critval(0){};
  AndersonDarling(long double cv):AdequacyTest(datatype(CONTINUOUS_DATA|DISCRETE_DATA,LIST|FREQUENCY_TABLE|GROUPED)),critval(cv){};
  virtual char apply(const Data& dat,const Distribution &dist);  
  virtual long double statistic(const Data& dat,const Distribution &dist);
  virtual void set_level(long double size,const Distribution &dist){critval=dist.get_critval(size,*this);};  
  virtual void set_level(long double cv){critval=cv;};  
};

char AndersonDarling::apply(const Data& dat,const Distribution &dist){
  return this->statistic(dat,dist)>critval;
};  

long double AndersonDarling::statistic(const Data &dat,const Distribution &dist){
  Distribution *tempdist=dist.copy();
  tempdist->estimate(dat);

  long double *distfuns=tempdist->dist(dat);
  
  long double ADstat=-1;
  unsigned int Fn=0;
  long double Fsp=0;
  long double Fsn=0;
  //    long double Fsnc=0;
  const unsigned int *freq=dat.getfreqs();
  unsigned int tot=dat.n;
  if(freq){
    tot=0;
    for(unsigned int i=0;i<dat.n;i++){
      tot+=*(freq+i);
    };
  };
  for(unsigned int j=0;j<dat.n;j++){
    Fsp=distfuns[j];
    Fsn=1-Fsp;
    if(Fsn==0){
      //Too implausible.
      return critval*100;
      //reject
    };
      
      
    ADstat-=(1-((long double)Fn)/dat.n)*(1-((long double)Fn)/dat.n)*log(Fsn);
    ADstat+=(((long double)Fn)/dat.n)*(((long double)Fn)/dat.n)*log(Fsp);
    if(freq){
      Fn+=*(freq+j);
    }else{
      Fn++;
    };
    ADstat+=(1-((long double)Fn)/dat.n)*(1-((long double)Fn)/dat.n)*log(Fsn);
    ADstat-=(((long double)Fn)/dat.n)*(((long double)Fn)/dat.n)*log(Fsp);
  };
  long double ans=ADstat*dat.n;//*tempdist->adjustADstat(n);
  delete[] distfuns;
  delete tempdist;
  return ans;
};



//Kolmogorov-Smirnov test


class KolmogorovSmirnov: public AdequacyTest{
private:
  long double critval;
public:
  KolmogorovSmirnov():AdequacyTest(datatype(CONTINUOUS_DATA|DISCRETE_DATA,LIST|FREQUENCY_TABLE|GROUPED)),critval(0){};
  KolmogorovSmirnov(long double cv):AdequacyTest(datatype(CONTINUOUS_DATA|DISCRETE_DATA,LIST|FREQUENCY_TABLE|GROUPED)),critval(cv){};
  virtual char apply(const Data& dat,const Distribution &dist);  
  virtual long double statistic(const Data& dat,const Distribution &dist);
  virtual void set_level(long double size,const Distribution &dist){critval=dist.get_critval(size,*this);};  
  virtual void set_level(long double cv){critval=cv;};  
};

char KolmogorovSmirnov::apply(const Data &dat , const Distribution &dist){
  return this->statistic(dat,dist)>critval;  
};

long double KolmogorovSmirnov::statistic(const Data &dat , const Distribution &dist){
  Distribution *tempdist=dist.copy();
  tempdist->estimate(dat);
  long double KSstat=0;
  unsigned int Fn=0;
  long double Fsp=0;
  long double *dists=dist.dist(dat);
  const unsigned int *freq=dat.getfreqs();
  unsigned int tot=dat.n;
  if(freq){
    tot=0;
    for(unsigned int i=0;i<dat.n;i++){
      tot+=*(freq+i);
    };
  };
  for(unsigned int j=0;j<dat.n;j++){
    Fsp=tot*(*(dists+j));
    if(Fsp-Fn>KSstat){
      KSstat=Fsp-Fn;
    };
    if(freq){
      Fn+=*(freq+j);
    }else{
      Fn++;
    };
    if(Fn-Fsp>KSstat){
      KSstat=Fn-Fsp;
    };
  };
  delete tempdist;
  delete[] dists;
  return KSstat/sqrt(tot);
};



class ChiSquare: public AdequacyTest{
private:
  long double critval;
public:
  ChiSquare():AdequacyTest(datatype(CONTINUOUS_DATA|DISCRETE_DATA,LIST|FREQUENCY_TABLE|GROUPED)),critval(0){};
  ChiSquare(long double cv):AdequacyTest(datatype(CONTINUOUS_DATA|DISCRETE_DATA,LIST|FREQUENCY_TABLE|GROUPED)),critval(cv){};
  virtual char apply(const Data& dat,const Distribution &dist);  
  virtual long double statistic(const Data& dat,const Distribution &dist);
  virtual void set_level(long double size,const Distribution &dist){critval=dist.get_critval(size,*this);};  
  virtual void set_level(long double cv){critval=cv;};  
};


char ChiSquare::apply(const Data &dat , const Distribution &dist){
  return this->statistic(dat,dist)>critval;  
};

long double ChiSquare::statistic(const Data &dat,const Distribution &dist){
  Distribution *tempdist=dist.copy();
  tempdist->estimate(dat);
  long double ans=0;
  long double ldist=0;

  long double *dists=dist.dist(dat);
  const unsigned int *samp=dat.getfreqs();
  unsigned int tot=dat.n;
  if(samp){
    tot=0;
    for(unsigned int i=0;i<dat.n;i++){
      tot+=*(samp+i);
    };
  };
  
  for(unsigned int i=0;i<dat.n;i++){
    unsigned int bincount=0;
    for(;i<dat.n&&(*(dists+i)-ldist)*tot<20;i++){
      if(samp){
	bincount+=*(samp+i);
      }else{
	bincount++;
      };
    };
    long double dst=1;
    if(i<dat.n-1){
      dst=*(dists+i);
    };
    ans+=(bincount-(dst-ldist)*tot)*(bincount-(dst-ldist)*tot)/((dst-ldist)*tot);
    ldist=dst;
  };
  delete tempdist;
  delete[] dists;
  return ans;  
};




long double chisqlookup(unsigned int n,long double atval);


//*/

AdequacyTest* read_adequacy_test(const char*  testname,const Distribution &dist){
  char *temp=new char[strlen(testname)+1];
  strcpy(temp,testname);
  char *at=NULL;
  unsigned int i=0;
  for(;*(temp+i)!='\0'&&*(temp+i)!='@';i++);
  if(*(temp+i)=='@'){
    at=temp+i+1;
    *(temp+i)='\0';
  };
  long double testatval=0.05;
  if(!strcmp(temp,"AD")){//Anderson-Darling test    
    if(at!=NULL){
      testatval=atof(at);
    };
    AndersonDarling *AD=new AndersonDarling();
    long double adcval=dist.get_critval(testatval,*AD);
    cout<<"Critical value for AD test is "<<adcval<<".\n";
    AD->set_level(adcval);
    delete[] temp;
    return AD;
  }else if(!strcmp(temp,"KS")){//Kolmogorov-Smirnov test
    if(at!=NULL){
      testatval=atof(at);
    };
    KolmogorovSmirnov *KS=new KolmogorovSmirnov();
    long double kscval=dist.get_critval(testatval,*KS);
    cout<<"Critical value for KS test is "<<kscval<<".\n";
    KS->set_level(kscval);
    delete[] temp;
    return KS;
  }else if(!strcmp(temp,"ChiSquare")){//Pearson's Chi-square test
    if(at!=NULL){
      testatval=atof(at);
    };
    delete[] temp;
    ChiSquare *CS=new ChiSquare();
    long double chisqcval=dist.get_critval(testatval,*CS);
    cout<<"Critical value for chi-square test is "<<chisqcval<<".\n";
    CS->set_level(chisqcval);
    return CS;
  }else{
    delete[] temp;
    return NULL;
  };
};

/*

DiscreteAdequacyTest read_discrete_adequacy_test(const char*  testname,const DiscreteDistribution &dist){
  char *temp=new char[strlen(testname)+1];
  strcpy(temp,testname);
  char *at=NULL;
  unsigned int i=0;
  for(;*(temp+i)!='\0'&&*(temp+i)!='@';i++);
  if(*(temp+i)=='@'){
    at=temp+i+1;
    *(temp+i)='\0';
  };
  if(!strcmp(temp,"AD")){//Anderson-Darling test    
    if(at!=NULL){
      testatval=atof(at);
    };
    adcval=dist.get_critval(testatval,CalculateADstat);
    cout<<"Critical value for AD test is "<<adcval<<".\n";
    delete[] temp;
    return ADtest;
  }else if(!strcmp(temp,"AD_jackknife")){//Anderson-Darling test jackknife sample
    if(at!=NULL){
      testatval=atof(at);
    };
    adcval=dist.get_critval(testatval,CalculateADstat);
    cout<<"Critical value for AD test is "<<adcval<<".\n";
    delete[] temp;
    return ADtest_jackknife;
  }else if(!strcmp(temp,"KS")){//Kolmogorov-Smirnov test
    if(at!=NULL){
      testatval=atof(at);
    };
    kscval=dist.get_critval(testatval,CalculateKSstat);
    cout<<"Critical value for KS test is "<<kscval<<".\n";
    delete[] temp;
    return KStest;
  }else if(!strcmp(temp,"ChiSquare")){//Pearson's Chi-square test
    if(at!=NULL){
      testatval=atof(at);
    };
    delete[] temp;
    //    cout<<"testatval="<<testatval<<"\n";
    if(grouped_data){//too early to do this.
      chisqcval=testatval;
      return ChiSqtestGrouped;
    }else{
      chisqcval=dist.get_critval(testatval,CalculateChiSqstat);
      //      cout<<"Critical value for chi-square test is "<<chisqcval<<".\n";
      return ChiSqtest;
    };
  }else if(!strcmp(temp,"LR_sample_bias")){//Likelihood ratio test vs sample bias 
    if(at!=NULL){
      testatval=atof(at);
    };
    delete[] temp;
    adcval=chisqlookup(sample_bias_step_ncut,testatval);
    cout<<"Critical value for likelihood ratio test is "<<adcval<<".\n";
    return LRtestSampleBiasNormal;
  }else{
    delete[] temp;
    return NULL;
  };
};


RegressionAdequacyTest read_regression_test(const char*  testname,const RegressionModel &dist){
  char *temp=new char[strlen(testname)+1];
  strcpy(temp,testname);
  char *at=NULL;
  unsigned int i=0;
  for(;*(temp+i)!='\0'&&*(temp+i)!='@';i++);
  if(*(temp+i)=='@'){
    at=temp+i+1;
    *(temp+i)='\0';
  };
  if(!strcmp(temp,"AD_residual")){
    //Anderson-Darling test for normality on residuals.
    if(at!=NULL){
      testatval=atof(at);
    };
    unsigned int zero=0;
    const char *argv[]={"normal"};
    Distribution& normal=read_distribution(1,argv,zero);
    adcval=normal.get_critval(testatval,CalculateADstat);
    cout<<"Critical value for AD test is "<<adcval<<".\n";
    delete[] temp;
    return ADresidual;
  }else if(!strcmp(temp,"Regression_Without_Replications")){
    //Regression without replicates.
    if(at!=NULL){
      testatval=atof(at);
    };    
    setnoncentralFcritvalsl(atof(at));
    delete[] temp;
    return  LocalGlobalRegression;
  }else{
    delete[] temp;
    return NULL;
  };
};
*/

//#include "distributions.h"

#include <boost/math/distributions/normal.hpp>
extern boost::math::normal stdnorm;


extern long double chisqcval;


class AdequacyTest;
class Data;
class Distribution;
class NPestimator;


void analyse_data(Distribution &truedist,AdequacyTest& adequacy_test,const Data &A);


#define NUMBOOTSTRAP 10000

const long double qchisq95[]={3.841459,  5.991465,  7.814728,  9.487729, 11.070498, 12.591587, 14.067140,  15.507313, 16.918978, 18.307038, 19.675138, 21.026070, 22.362032, 23.684791, 24.995790, 26.296228, 27.587112, 28.869299, 30.143527, 31.410433};


void getpos(unsigned int n,const long double *list,unsigned int numps,const unsigned int *pos,long double *to);

class AdequateBootstrapIntervals{
public:
  unsigned int absz;
  long double abl;
  long double abu;
  long double sbl;
  long double sbu;
  long double nbl;
  long double nbu;
  long double nl;
  long double nu;
};





extern std::ofstream savedata;

extern unsigned int jackknife;

extern unsigned int frequency_table;

extern unsigned int grouped_data;

extern unsigned int ngroup;

extern unsigned int maxbssize;



unsigned int solveprobfast(unsigned int n,unsigned int *xvals,AdequacyTest &Ad,Distribution &dist,long double prob,Data &dat);


void read_option(const char* const optname);

Distribution *read_distribution(int argc,const char* const argv[],unsigned int &start);

//#include "regression.cpp"  //Later, will correct this to regression.h

//For regression contexts, we need to do things a little differently.


inline long double rnorm(){
  return quantile(stdnorm,((long double)rand())/RAND_MAX);
};

long double chisqlookup(unsigned int n,long double atval);

extern long double testatval;

void cleanup();

#define DATA_H
#include <cstdlib>
#include <iosfwd>

class datatype{
  //Describes the different possible datatypes
  //Distributions and datasets should match exactly one datatype
  //Adequacy tests should match a range of possible datatypes;
 public:
  unsigned char type_of_data;
#define CONTINUOUS_DATA 1
#define DISCRETE_DATA 2
#define MULTIVARIATE_CONTINUOUS_DATA 4
#define MULTIVARIATE_DISCRETE_DATA 8
#define REGRESSION_DATA 16
  unsigned char  storage;
#define LIST 1
#define FREQUENCY_TABLE 2
#define GROUPED 4
 datatype(char ty,char st):type_of_data(ty),storage(st){};
 datatype():type_of_data(0),storage(0){};
  char match(const datatype &compare){return type_of_data==compare.type_of_data&&(storage==(storage&compare.storage));};//Compares for an exact match
  char within(const datatype &compare){return (type_of_data==(type_of_data&compare.type_of_data)&&(storage==(storage&compare.storage)));};//Compares for datatype covered
};



class Data{
 protected:
  unsigned int jackknife;
  datatype typ;
 public:
  unsigned int n;
  unsigned int *sample=NULL;
 Data(unsigned int m):jackknife(0),n(m){};
  datatype get_type() const{return typ;};
  virtual const void* getvalues() const=0;
  virtual const unsigned int* getfreqs()const=0;
  void setjackknife(unsigned int j=1){jackknife=j;};
  virtual void subsample(unsigned int size);
  virtual void clearsubsample(unsigned int size);//unlikely to be needed.
  virtual void read(std::istream &in)=0;
  virtual void concat(const Data *cdat)=0;
  virtual void sort()=0;
  virtual ~Data(){};
};


class NPestimator{
 public:
  datatype type; //What types of data can this test be applied to
  virtual long double evaluate(const Data& dat)=0;
 NPestimator(datatype t):type(t){};
 NPestimator():type(datatype(0,0)){};
  unsigned int compatible(const Data &dat){return dat.get_type().within(type);};
  ~NPestimator(){};
};


NPestimator *get_NP_estimator(const char *eq,const char *at);

Data* new_data(const datatype &type);


#endif

#include <cstdlib>

//This could probably be done better with templates.
//Defines different types of data that might be encountered.



class Frequency_Data: public Data{
 public:
  unsigned int *freq;
 Frequency_Data(unsigned int n,unsigned int *f):Data(n),freq(f){};
  virtual void subsample(unsigned int size);  
  virtual void concat(const Data *cdat)=0;
  virtual ~Frequency_Data(){if(freq){delete[] freq;};};
  virtual void sort()=0;
};



class Discrete_Individual_Data: public Data{
 public:
  //  datatype typ;
  //  unsigned int n;
  unsigned int *data;
  virtual const void* getvalues() const;
  virtual const unsigned int* getfreqs() const;
 Discrete_Individual_Data():Data(0),data(NULL){typ=datatype(DISCRETE_DATA,LIST);};
 Discrete_Individual_Data(unsigned int len,unsigned int *vals):Data(len),data(vals){typ=datatype(DISCRETE_DATA,LIST);};
    virtual void read(std::istream &in);
  virtual void concat(const Data *cdat);
  virtual ~Discrete_Individual_Data(){if(data){delete[] data;};};
  virtual void sort();
};


class Continuous_Individual_Data: public Data{
 public:
  //  datatype typ;
  //  unsigned int n;
  long double *data;
  virtual const void* getvalues() const;
  virtual const unsigned int* getfreqs() const;
 Continuous_Individual_Data():Data(0),data(NULL){typ=datatype(CONTINUOUS_DATA,LIST);};
 Continuous_Individual_Data(unsigned int len,long double *vals):Data(len),data(vals){typ=datatype(CONTINUOUS_DATA,LIST);};
  virtual void read(std::istream &in);
  virtual void concat(const Data *cdat);
  virtual ~Continuous_Individual_Data(){if(data){delete[] data;};};
  virtual void sort();
};





class Discrete_Frequency_Data: public Frequency_Data{
 public:
  //  datatype typ;
  //  unsigned int n;
  unsigned int *data;
  virtual const void* getvalues() const;
  virtual const unsigned int* getfreqs() const;
  //  virtual void subsample(unsigned int size);  
 Discrete_Frequency_Data():Frequency_Data(0,NULL),data(NULL){typ=datatype(DISCRETE_DATA,FREQUENCY_TABLE);};
 Discrete_Frequency_Data(unsigned int len,unsigned int *f,unsigned int *vals):Frequency_Data(len,f),data(vals){typ=datatype(DISCRETE_DATA,FREQUENCY_TABLE);};
  virtual void read(std::istream &in);
  virtual void concat(const Data *cdat);
  virtual ~Discrete_Frequency_Data(){if(data){delete[] data;};  };
  virtual void sort();
};


class Continuous_Frequency_Data: public Frequency_Data{
 public:
  //  datatype typ;
  //  unsigned int n;
  long double *data;
  virtual const void* getvalues() const;
  virtual const unsigned int* getfreqs() const;
 Continuous_Frequency_Data():Frequency_Data(0,NULL),data(NULL){typ=datatype(CONTINUOUS_DATA,FREQUENCY_TABLE);};
 Continuous_Frequency_Data(unsigned int len,unsigned int *f,long double *vals):Frequency_Data(len,f),data(vals){typ=datatype(CONTINUOUS_DATA,FREQUENCY_TABLE);};
  virtual void read(std::istream &in);
  virtual void concat(const Data *cdat);
  virtual ~Continuous_Frequency_Data(){if(data){delete[] data;};  };
  virtual void sort();
};




class Discrete_Grouped_Data: public Frequency_Data{
 public:
  //  datatype typ;
  //  unsigned int n;
  unsigned int *cutoffs;
  virtual const void* getvalues() const;
  virtual const unsigned int* getfreqs() const;
  //  virtual void subsample(unsigned int size);  
 Discrete_Grouped_Data():Frequency_Data(0,NULL),cutoffs(NULL){typ=datatype(DISCRETE_DATA,GROUPED);};
 Discrete_Grouped_Data(unsigned int len,unsigned int *f,unsigned int *vals):Frequency_Data(len,f),cutoffs(vals){typ=datatype(DISCRETE_DATA,GROUPED);};
  virtual void read(std::istream &in);
  virtual void concat(const Data *cdat);
  virtual ~Discrete_Grouped_Data(){if(cutoffs){delete[] cutoffs;};      };
  virtual void sort();
};


class Continuous_Grouped_Data: public Frequency_Data{
 public:
  //  datatype typ;
  //  unsigned int n;
  long double *cutoffs;
  virtual const void* getvalues() const;
  virtual const unsigned int* getfreqs() const;
 Continuous_Grouped_Data():Frequency_Data(0,NULL),cutoffs(NULL){typ=datatype(CONTINUOUS_DATA,GROUPED);};
 Continuous_Grouped_Data(unsigned int len,unsigned int *f,long double *vals):Frequency_Data(len,f),cutoffs(vals){typ=datatype(CONTINUOUS_DATA,GROUPED);};
  virtual void read(std::istream &in);
  virtual void concat(const Data *cdat);
  virtual ~Continuous_Grouped_Data(){if(cutoffs){delete[] cutoffs;};  };
  virtual void sort();
};



#define DISTORTION_H

#include "data.h"
class Distribution;

class Distortion{
 public:
  virtual Data *simulate(unsigned int simsize,const Distribution &truedist)=0;
};


Distortion* Read_distortion(unsigned int &argno,unsigned int argc,char *argv[]);

#endif

#define GENERAL_H

#include <cstddef>

#include <cstdlib>
//generally useful functions


inline int compareLongDouble(const void *p1,const void *p2){
  if(*((long double *)p1)<*((long double *)p2)){
    return -1;
  }else if(*((long double *)p1)>*((long double *)p2)){
    return 1;
  }else{
    return 0;
  };
};


inline int compareUnsignedInt(const void *p1,const void *p2){
  if(*((unsigned int *)p1)<*((unsigned int *)p2)){
    return -1;
  }else if(*((unsigned int *)p1)>*((unsigned int *)p2)){
    return 1;
  }else{
    return 0;
  };
};


class dblll{
  long double val;
  dblll *next;
public:
  dblll(long double v){val=v;next=NULL;};
  dblll(long double v,dblll *nxt){val=v;next=nxt;};
  void insert(long double v){dblll *ins=new dblll(val,next);next=ins;val=v;};
  ~dblll(){if(next){delete(next);};};
  unsigned int length(unsigned int ans=0){ans++;if(next==NULL){return(ans);}else{return(next->length(ans));};};
  void write(long double *to){*to=val;if(next!=NULL){next->write(to+1);};};
};


class uill{
  unsigned int val;
  uill *next;
public:
  uill(unsigned int v){val=v;next=NULL;};
  uill(unsigned int v,uill *nxt){val=v;next=nxt;};
  void insert(unsigned int v){uill *ins=new uill(val,next);next=ins;val=v;};
  ~uill(){if(next){delete(next);};};
  unsigned int length(unsigned int ans=0){ans++;if(next==NULL){return(ans);}else{return(next->length(ans));};};
  void write(unsigned int *to){*to=val;if(next!=NULL){next->write(to+1);};};
};

//insertions happen at the front.


class dblctll{
  long double val;
  unsigned int freq;
  dblctll *next;
public:
  dblctll(long double v,unsigned int i){val=v;freq=i;next=NULL;};
  dblctll(long double v,unsigned int i,dblctll *nxt){val=v;freq=i;next=nxt;};
  void insert(long double v,unsigned int i){dblctll *ins=new dblctll(val,freq,next);next=ins;val=v;freq=i;};
  ~dblctll(){if(next){delete(next);};};
  unsigned int length(unsigned int ans=0){ans++;if(next==NULL){return(ans);}else{return(next->length(ans));};};
  void write(long double *to,unsigned int *frq){*to=val;*frq=freq;if(next!=NULL){next->write(to+1,frq+1);};};
};

class uictll{
  unsigned int val;
  unsigned int freq;
  uictll *next;
public:
  uictll(unsigned int v,unsigned int i){val=v;freq=i;next=NULL;};
  uictll(unsigned int v,unsigned int i,uictll *nxt){val=v;freq=i;next=nxt;};
  void insert(unsigned int v,unsigned int i){uictll *ins=new uictll(val,freq,next);next=ins;val=v;freq=i;};
  ~uictll(){if(next){delete(next);};};
  unsigned int length(unsigned int ans=0){ans++;if(next==NULL){return(ans);}else{return(next->length(ans));};};
  void write(unsigned int *to,unsigned int *freqs){*to=val;*freqs=freq;if(next!=NULL){next->write(to+1,freqs+1);};};
};
#endif

#ifndef TESTS_h
#define TESTS_h 1


extern long double adcval;

extern long double kscval;

class Distribution;
class DiscreteDistribution;
class MultivariateDiscreteDistribution;

typedef long double (*AdequacyStat)(unsigned int,const long double *,const Distribution &);


typedef char (*AdequacyTest)(unsigned int,const void*,const Distribution &);

typedef char (*DiscreteAdequacyTest)(unsigned int,const void*,const DiscreteDistribution &);

typedef char (*RegressionAdequacyTest)(unsigned int,const void*,const RegressionModel &);


AdequacyTest read_adequacy_test(const char*  testname,const Distribution &dist);

DiscreteAdequacyTest read_discrete_adequacy_test(const char*  testname,const DiscreteDistribution &dist);


RegressionAdequacyTest read_regression_test(const char*  testname,const RegressionModel &dist);

AdequacyTest switchtojackknife(AdequacyTest adequacy_test);

AdequacyTest switchtogrouped(AdequacyTest adequacy_test);

DiscreteAdequacyTest switchtojackknife(DiscreteAdequacyTest adequacy_test);

DiscreteAdequacyTest switchtogrouped(DiscreteAdequacyTest adequacy_test);

void sortedsubsample(unsigned int n,unsigned int x,unsigned int *to);

void sortedsubsample(unsigned int n,const unsigned int *counts,unsigned int x,unsigned int *to);

void sortedjackknife(unsigned int n,unsigned int x,const long double *data,long double *to);

void sortedjackknife(unsigned int n,unsigned int x,const unsigned int *data,unsigned int *to);

void   sortedjackknife(unsigned int n,const unsigned int *counts,unsigned int x,unsigned int *to);

void jackknifecount(unsigned int n,unsigned int x,unsigned int *to);

unsigned int rbinom(unsigned int n,long double p);

unsigned int rbinom_fast(unsigned int n,long double p);

long double findmle_sample_bias_normal(unsigned int n,long double *samp,unsigned int ncut,long double *cutoffs,long double *nn=NULL);

#endif

#
#   This file is part of Adequate Bootstrap.
#
#   Adequate Bootstrap is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   Adequate Bootstrap is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with Adequate Bootstrap.  If not, see <http://www.gnu.org/licenses/>.


INSTALL=install
INSTALL_PROGRAM=$(INSTALL)
INSTALL_DATA=${INSTALL} -m 666
INSTALL_INFO=install-info

FLAGS=-Wunreachable-code -Wall -ggdb

MAKEPDFINFO=texi2any --pdf

ifdef OPTIMISATIONLEVEL

else
OPTIMISATIONLEVEL=3
endif

installdir=/usr/local/bin
infodir=/usr/share/info

tarfilename=adequatebootstrap.tar
distributiontarfile=Distributions.tar

distcppfiles=adequatebootstrap.cpp  corefunctions.cpp datatypes.cpp distortion.cpp NonParametric.cpp tests.cpp

distheaderfiles= corefunctions.h data.h datatypes.h distortion.h general.h tests.h

maindirdistfiles=$(distcppfiles) $(distheaderfiles) Distributions/$(distributiontarfile) makefile COPYING

alldistfiles=$(maindircppfiles) Distributions/$(distributiontarfile) 






all: adequatebootstrap 

adequatebootstrap: adequatebootstrap.o datatypes.o corefunctions.o tests.o NonParametric.o distortion.o Distributions/AllDistributions.a 
	$(CXX) $(FLAGS) -O$(OPTIMISATIONLEVEL) -o adequatebootstrap   adequatebootstrap.o  datatypes.o NonParametric.o corefunctions.o tests.o distortion.o  Distributions/AllDistributions.a

Distributions/AllDistributions.a:
	$(MAKE) -C Distributions

%.o: %.cpp
	$(CXX) $(FLAGS) -O$(OPTIMISATIONLEVEL) -Wall -ggdb  -o $@ -I. -I Distributions -c $<


.PHONY: all clean maintainer-clean install install-prog config package backup uninstall install-html info Distributions/AllDistributions.a Distributions/$(distributiontarfile)

install: adequatebootstrap.info
	install adequatebootstrap $(installdir)
	$(INSTALL_DATA) adequatebootstrap.info $(infodir)
	$(INSTALL_INFO) adequatebootstrap.info $(infodir)/dir


adequatebootstrap.info:
	$(MAKEINFO) adequatebootstrap.texi

adequatebootstrap.pdf: 
	$(MAKEPDFINFO) adequatebootstrap.texi

info: adequatebootstrap.info 	


dist:   $(tarfilename)
	gzip $(tarfilename)

$(tarfilename): $(alldistfiles)
	tar -cf $(tarfilename) $(maindirdistfiles)
	tar --concatenate --file=$(tarfilename) $(distributiontarfile)

$(distributiontarfile):
	$(MAKE) -C Distributions $(distributiontarfile)

backup: tarfilename=$(backupname)
backup: dist
	mv $(tarfilename).gz $(backupdir)/$(backupname)

uninstall:
	-$(INSTALL-INFO) --delete $(DESTDIR)$(infodir)/adequatebootstrap.info  $(DESTDIR)$(infodir)/dir
	-rm $(installedfiles)

clean:
	$(MAKE) clean -C Distributions
	-rm *.o
	-rm adequatebootstrap

dist-clean: clean

maintainer-clean: dist-clean

                       Version 3, 29 June 2007

 Copyright (C) 2007 Free Software Foundation, Inc. <http://fsf.org/>
 Everyone is permitted to copy and distribute verbatim copies
 of this license document, but changing it is not allowed.

                            Preamble

  The GNU General Public License is a free, copyleft license for
software and other kinds of works.

  The licenses for most software and other practical works are designed
to take away your freedom to share and change the works.  By contrast,
the GNU General Public License is intended to guarantee your freedom to
share and change all versions of a program--to make sure it remains free
software for all its users.  We, the Free Software Foundation, use the
GNU General Public License for most of our software; it applies also to
any other work released this way by its authors.  You can apply it to
your programs, too.

  When we speak of free software, we are referring to freedom, not
price.  Our General Public Licenses are designed to make sure that you
have the freedom to distribute copies of free software (and charge for
them if you wish), that you receive source code or can get it if you
want it, that you can change the software or use pieces of it in new
free programs, and that you know you can do these things.

  To protect your rights, we need to prevent others from denying you
these rights or asking you to surrender the rights.  Therefore, you have
certain responsibilities if you distribute copies of the software, or if
you modify it: responsibilities to respect the freedom of others.

  For example, if you distribute copies of such a program, whether
gratis or for a fee, you must pass on to the recipients the same
freedoms that you received.  You must make sure that they, too, receive
or can get the source code.  And you must show them these terms so they
know their rights.

  Developers that use the GNU GPL protect your rights with two steps:
(1) assert copyright on the software, and (2) offer you this License
giving you legal permission to copy, distribute and/or modify it.

  For the developers' and authors' protection, the GPL clearly explains
that there is no warranty for this free software.  For both users' and
authors' sake, the GPL requires that modified versions be marked as
changed, so that their problems will not be attributed erroneously to
authors of previous versions.

  Some devices are designed to deny users access to install or run
modified versions of the software inside them, although the manufacturer
can do so.  This is fundamentally incompatible with the aim of
protecting users' freedom to change the software.  The systematic
pattern of such abuse occurs in the area of products for individuals to
use, which is precisely where it is most unacceptable.  Therefore, we
have designed this version of the GPL to prohibit the practice for those
products.  If such problems arise substantially in other domains, we
stand ready to extend this provision to those domains in future versions
of the GPL, as needed to protect the freedom of users.

  Finally, every program is threatened constantly by software patents.
States should not allow patents to restrict development and use of
software on general-purpose computers, but in those that do, we wish to
avoid the special danger that patents applied to a free program could
make it effectively proprietary.  To prevent this, the GPL assures that
patents cannot be used to render the program non-free.

  The precise terms and conditions for copying, distribution and
modification follow.

                       TERMS AND CONDITIONS

  0. Definitions.

  "This License" refers to version 3 of the GNU General Public License.

  "Copyright" also means copyright-like laws that apply to other kinds of
works, such as semiconductor masks.

  "The Program" refers to any copyrightable work licensed under this
License.  Each licensee is addressed as "you".  "Licensees" and
"recipients" may be individuals or organizations.

  To "modify" a work means to copy from or adapt all or part of the work
in a fashion requiring copyright permission, other than the making of an
exact copy.  The resulting work is called a "modified version" of the
earlier work or a work "based on" the earlier work.

  A "covered work" means either the unmodified Program or a work based
on the Program.

  To "propagate" a work means to do anything with it that, without
permission, would make you directly or secondarily liable for
infringement under applicable copyright law, except executing it on a
computer or modifying a private copy.  Propagation includes copying,
distribution (with or without modification), making available to the
public, and in some countries other activities as well.

  To "convey" a work means any kind of propagation that enables other
parties to make or receive copies.  Mere interaction with a user through
a computer network, with no transfer of a copy, is not conveying.

  An interactive user interface displays "Appropriate Legal Notices"
to the extent that it includes a convenient and prominently visible
feature that (1) displays an appropriate copyright notice, and (2)
tells the user that there is no warranty for the work (except to the
extent that warranties are provided), that licensees may convey the
work under this License, and how to view a copy of this License.  If
the interface presents a list of user commands or options, such as a
menu, a prominent item in the list meets this criterion.

  1. Source Code.

  The "source code" for a work means the preferred form of the work
for making modifications to it.  "Object code" means any non-source
form of a work.

  A "Standard Interface" means an interface that either is an official
standard defined by a recognized standards body, or, in the case of
interfaces specified for a particular programming language, one that
is widely used among developers working in that language.

  The "System Libraries" of an executable work include anything, other
than the work as a whole, that (a) is included in the normal form of
packaging a Major Component, but which is not part of that Major
Component, and (b) serves only to enable use of the work with that
Major Component, or to implement a Standard Interface for which an
implementation is available to the public in source code form.  A
"Major Component", in this context, means a major essential component
(kernel, window system, and so on) of the specific operating system
(if any) on which the executable work runs, or a compiler used to
produce the work, or an object code interpreter used to run it.

  The "Corresponding Source" for a work in object code form means all
the source code needed to generate, install, and (for an executable
work) run the object code and to modify the work, including scripts to
control those activities.  However, it does not include the work's
System Libraries, or general-purpose tools or generally available free
programs which are used unmodified in performing those activities but
which are not part of the work.  For example, Corresponding Source
includes interface definition files associated with source files for
the work, and the source code for shared libraries and dynamically
linked subprograms that the work is specifically designed to require,
such as by intimate data communication or control flow between those
subprograms and other parts of the work.

  The Corresponding Source need not include anything that users
can regenerate automatically from other parts of the Corresponding
Source.

  The Corresponding Source for a work in source code form is that
same work.

  2. Basic Permissions.

  All rights granted under this License are granted for the term of
copyright on the Program, and are irrevocable provided the stated
conditions are met.  This License explicitly affirms your unlimited
permission to run the unmodified Program.  The output from running a
covered work is covered by this License only if the output, given its
content, constitutes a covered work.  This License acknowledges your
rights of fair use or other equivalent, as provided by copyright law.

  You may make, run and propagate covered works that you do not
convey, without conditions so long as your license otherwise remains
in force.  You may convey covered works to others for the sole purpose
of having them make modifications exclusively for you, or provide you
with facilities for running those works, provided that you comply with
the terms of this License in conveying all material for which you do
not control copyright.  Those thus making or running the covered works
for you must do so exclusively on your behalf, under your direction
and control, on terms that prohibit them from making any copies of
your copyrighted material outside their relationship with you.

  Conveying under any other circumstances is permitted solely under
the conditions stated below.  Sublicensing is not allowed; section 10
makes it unnecessary.

  3. Protecting Users' Legal Rights From Anti-Circumvention Law.

  No covered work shall be deemed part of an effective technological
measure under any applicable law fulfilling obligations under article
11 of the WIPO copyright treaty adopted on 20 December 1996, or
similar laws prohibiting or restricting circumvention of such
measures.

  When you convey a covered work, you waive any legal power to forbid
circumvention of technological measures to the extent such circumvention
is effected by exercising rights under this License with respect to
the covered work, and you disclaim any intention to limit operation or
modification of the work as a means of enforcing, against the work's
users, your or third parties' legal rights to forbid circumvention of
technological measures.

  4. Conveying Verbatim Copies.

  You may convey verbatim copies of the Program's source code as you
receive it, in any medium, provided that you conspicuously and
appropriately publish on each copy an appropriate copyright notice;
keep intact all notices stating that this License and any
non-permissive terms added in accord with section 7 apply to the code;
keep intact all notices of the absence of any warranty; and give all
recipients a copy of this License along with the Program.

  You may charge any price or no price for each copy that you convey,
and you may offer support or warranty protection for a fee.

  5. Conveying Modified Source Versions.

  You may convey a work based on the Program, or the modifications to
produce it from the Program, in the form of source code under the
terms of section 4, provided that you also meet all of these conditions:

    a) The work must carry prominent notices stating that you modified
    it, and giving a relevant date.

    b) The work must carry prominent notices stating that it is
    released under this License and any conditions added under section
    7.  This requirement modifies the requirement in section 4 to
    "keep intact all notices".

    c) You must license the entire work, as a whole, under this
    License to anyone who comes into possession of a copy.  This
    License will therefore apply, along with any applicable section 7
    additional terms, to the whole of the work, and all its parts,
    regardless of how they are packaged.  This License gives no
    permission to license the work in any other way, but it does not
    invalidate such permission if you have separately received it.

    d) If the work has interactive user interfaces, each must display
    Appropriate Legal Notices; however, if the Program has interactive
    interfaces that do not display Appropriate Legal Notices, your
    work need not make them do so.

  A compilation of a covered work with other separate and independent
works, which are not by their nature extensions of the covered work,
and which are not combined with it such as to form a larger program,
in or on a volume of a storage or distribution medium, is called an
"aggregate" if the compilation and its resulting copyright are not
used to limit the access or legal rights of the compilation's users
beyond what the individual works permit.  Inclusion of a covered work
in an aggregate does not cause this License to apply to the other
parts of the aggregate.

  6. Conveying Non-Source Forms.

  You may convey a covered work in object code form under the terms
of sections 4 and 5, provided that you also convey the
machine-readable Corresponding Source under the terms of this License,
in one of these ways:

    a) Convey the object code in, or embodied in, a physical product
    (including a physical distribution medium), accompanied by the
    Corresponding Source fixed on a durable physical medium
    customarily used for software interchange.

    b) Convey the object code in, or embodied in, a physical product
    (including a physical distribution medium), accompanied by a
    written offer, valid for at least three years and valid for as
    long as you offer spare parts or customer support for that product
    model, to give anyone who possesses the object code either (1) a
    copy of the Corresponding Source for all the software in the
    product that is covered by this License, on a durable physical
    medium customarily used for software interchange, for a price no
    more than your reasonable cost of physically performing this
    conveying of source, or (2) access to copy the
    Corresponding Source from a network server at no charge.

    c) Convey individual copies of the object code with a copy of the
    written offer to provide the Corresponding Source.  This
    alternative is allowed only occasionally and noncommercially, and
    only if you received the object code with such an offer, in accord
    with subsection 6b.

    d) Convey the object code by offering access from a designated
    place (gratis or for a charge), and offer equivalent access to the
    Corresponding Source in the same way through the same place at no
    further charge.  You need not require recipients to copy the
    Corresponding Source along with the object code.  If the place to
    copy the object code is a network server, the Corresponding Source
    may be on a different server (operated by you or a third party)
    that supports equivalent copying facilities, provided you maintain
    clear directions next to the object code saying where to find the
    Corresponding Source.  Regardless of what server hosts the
    Corresponding Source, you remain obligated to ensure that it is
    available for as long as needed to satisfy these requirements.

    e) Convey the object code using peer-to-peer transmission, provided
    you inform other peers where the object code and Corresponding
    Source of the work are being offered to the general public at no
    charge under subsection 6d.

  A separable portion of the object code, whose source code is excluded
from the Corresponding Source as a System Library, need not be
included in conveying the object code work.

  A "User Product" is either (1) a "consumer product", which means any
tangible personal property which is normally used for personal, family,
or household purposes, or (2) anything designed or sold for incorporation
into a dwelling.  In determining whether a product is a consumer product,
doubtful cases shall be resolved in favor of coverage.  For a particular
product received by a particular user, "normally used" refers to a
typical or common use of that class of product, regardless of the status
of the particular user or of the way in which the particular user
actually uses, or expects or is expected to use, the product.  A product
is a consumer product regardless of whether the product has substantial
commercial, industrial or non-consumer uses, unless such uses represent
the only significant mode of use of the product.

  "Installation Information" for a User Product means any methods,
procedures, authorization keys, or other information required to install
and execute modified versions of a covered work in that User Product from
a modified version of its Corresponding Source.  The information must
suffice to ensure that the continued functioning of the modified object
code is in no case prevented or interfered with solely because
modification has been made.

  If you convey an object code work under this section in, or with, or
specifically for use in, a User Product, and the conveying occurs as
part of a transaction in which the right of possession and use of the
User Product is transferred to the recipient in perpetuity or for a
fixed term (regardless of how the transaction is characterized), the
Corresponding Source conveyed under this section must be accompanied
by the Installation Information.  But this requirement does not apply
if neither you nor any third party retains the ability to install
modified object code on the User Product (for example, the work has
been installed in ROM).

  The requirement to provide Installation Information does not include a
requirement to continue to provide support service, warranty, or updates
for a work that has been modified or installed by the recipient, or for
the User Product in which it has been modified or installed.  Access to a
network may be denied when the modification itself materially and
adversely affects the operation of the network or violates the rules and
protocols for communication across the network.

  Corresponding Source conveyed, and Installation Information provided,
in accord with this section must be in a format that is publicly
documented (and with an implementation available to the public in
source code form), and must require no special password or key for
unpacking, reading or copying.

  7. Additional Terms.

  "Additional permissions" are terms that supplement the terms of this
License by making exceptions from one or more of its conditions.
Additional permissions that are applicable to the entire Program shall
be treated as though they were included in this License, to the extent
that they are valid under applicable law.  If additional permissions
apply only to part of the Program, that part may be used separately
under those permissions, but the entire Program remains governed by
this License without regard to the additional permissions.

  When you convey a copy of a covered work, you may at your option
remove any additional permissions from that copy, or from any part of
it.  (Additional permissions may be written to require their own
removal in certain cases when you modify the work.)  You may place
additional permissions on material, added by you to a covered work,
for which you have or can give appropriate copyright permission.

  Notwithstanding any other provision of this License, for material you
add to a covered work, you may (if authorized by the copyright holders of
that material) supplement the terms of this License with terms:

    a) Disclaiming warranty or limiting liability differently from the
    terms of sections 15 and 16 of this License; or

    b) Requiring preservation of specified reasonable legal notices or
    author attributions in that material or in the Appropriate Legal
    Notices displayed by works containing it; or

    c) Prohibiting misrepresentation of the origin of that material, or
    requiring that modified versions of such material be marked in
    reasonable ways as different from the original version; or

    d) Limiting the use for publicity purposes of names of licensors or
    authors of the material; or

    e) Declining to grant rights under trademark law for use of some
    trade names, trademarks, or service marks; or

    f) Requiring indemnification of licensors and authors of that
    material by anyone who conveys the material (or modified versions of
    it) with contractual assumptions of liability to the recipient, for
    any liability that these contractual assumptions directly impose on
    those licensors and authors.

  All other non-permissive additional terms are considered "further
restrictions" within the meaning of section 10.  If the Program as you
received it, or any part of it, contains a notice stating that it is
governed by this License along with a term that is a further
restriction, you may remove that term.  If a license document contains
a further restriction but permits relicensing or conveying under this
License, you may add to a covered work material governed by the terms
of that license document, provided that the further restriction does
not survive such relicensing or conveying.

  If you add terms to a covered work in accord with this section, you
must place, in the relevant source files, a statement of the
additional terms that apply to those files, or a notice indicating
where to find the applicable terms.

  Additional terms, permissive or non-permissive, may be stated in the
form of a separately written license, or stated as exceptions;
the above requirements apply either way.

  8. Termination.

  You may not propagate or modify a covered work except as expressly
provided under this License.  Any attempt otherwise to propagate or
modify it is void, and will automatically terminate your rights under
this License (including any patent licenses granted under the third
paragraph of section 11).

  However, if you cease all violation of this License, then your
license from a particular copyright holder is reinstated (a)
provisionally, unless and until the copyright holder explicitly and
finally terminates your license, and (b) permanently, if the copyright
holder fails to notify you of the violation by some reasonable means
prior to 60 days after the cessation.

  Moreover, your license from a particular copyright holder is
reinstated permanently if the copyright holder notifies you of the
violation by some reasonable means, this is the first time you have
received notice of violation of this License (for any work) from that
copyright holder, and you cure the violation prior to 30 days after
your receipt of the notice.

  Termination of your rights under this section does not terminate the
licenses of parties who have received copies or rights from you under
this License.  If your rights have been terminated and not permanently
reinstated, you do not qualify to receive new licenses for the same
material under section 10.

  9. Acceptance Not Required for Having Copies.

  You are not required to accept this License in order to receive or
run a copy of the Program.  Ancillary propagation of a covered work
occurring solely as a consequence of using peer-to-peer transmission
to receive a copy likewise does not require acceptance.  However,
nothing other than this License grants you permission to propagate or
modify any covered work.  These actions infringe copyright if you do
not accept this License.  Therefore, by modifying or propagating a
covered work, you indicate your acceptance of this License to do so.

  10. Automatic Licensing of Downstream Recipients.

  Each time you convey a covered work, the recipient automatically
receives a license from the original licensors, to run, modify and
propagate that work, subject to this License.  You are not responsible
for enforcing compliance by third parties with this License.

  An "entity transaction" is a transaction transferring control of an
organization, or substantially all assets of one, or subdividing an
organization, or merging organizations.  If propagation of a covered
work results from an entity transaction, each party to that
transaction who receives a copy of the work also receives whatever
licenses to the work the party's predecessor in interest had or could
give under the previous paragraph, plus a right to possession of the
Corresponding Source of the work from the predecessor in interest, if
the predecessor has it or can get it with reasonable efforts.

  You may not impose any further restrictions on the exercise of the
rights granted or affirmed under this License.  For example, you may
not impose a license fee, royalty, or other charge for exercise of
rights granted under this License, and you may not initiate litigation
(including a cross-claim or counterclaim in a lawsuit) alleging that
any patent claim is infringed by making, using, selling, offering for
sale, or importing the Program or any portion of it.

  11. Patents.

  A "contributor" is a copyright holder who authorizes use under this
License of the Program or a work on which the Program is based.  The
work thus licensed is called the contributor's "contributor version".

  A contributor's "essential patent claims" are all patent claims
owned or controlled by the contributor, whether already acquired or
hereafter acquired, that would be infringed by some manner, permitted
by this License, of making, using, or selling its contributor version,
but do not include claims that would be infringed only as a
consequence of further modification of the contributor version.  For
purposes of this definition, "control" includes the right to grant
patent sublicenses in a manner consistent with the requirements of
this License.

  Each contributor grants you a non-exclusive, worldwide, royalty-free
patent license under the contributor's essential patent claims, to
make, use, sell, offer for sale, import and otherwise run, modify and
propagate the contents of its contributor version.

  In the following three paragraphs, a "patent license" is any express
agreement or commitment, however denominated, not to enforce a patent
(such as an express permission to practice a patent or covenant not to
sue for patent infringement).  To "grant" such a patent license to a
party means to make such an agreement or commitment not to enforce a
patent against the party.

  If you convey a covered work, knowingly relying on a patent license,
and the Corresponding Source of the work is not available for anyone
to copy, free of charge and under the terms of this License, through a
publicly available network server or other readily accessible means,
then you must either (1) cause the Corresponding Source to be so
available, or (2) arrange to deprive yourself of the benefit of the
patent license for this particular work, or (3) arrange, in a manner
consistent with the requirements of this License, to extend the patent
license to downstream recipients.  "Knowingly relying" means you have
actual knowledge that, but for the patent license, your conveying the
covered work in a country, or your recipient's use of the covered work
in a country, would infringe one or more identifiable patents in that
country that you have reason to believe are valid.

  If, pursuant to or in connection with a single transaction or
arrangement, you convey, or propagate by procuring conveyance of, a
covered work, and grant a patent license to some of the parties
receiving the covered work authorizing them to use, propagate, modify
or convey a specific copy of the covered work, then the patent license
you grant is automatically extended to all recipients of the covered
work and works based on it.

  A patent license is "discriminatory" if it does not include within
the scope of its coverage, prohibits the exercise of, or is
conditioned on the non-exercise of one or more of the rights that are
specifically granted under this License.  You may not convey a covered
work if you are a party to an arrangement with a third party that is
in the business of distributing software, under which you make payment
to the third party based on the extent of your activity of conveying
the work, and under which the third party grants, to any of the
parties who would receive the covered work from you, a discriminatory
patent license (a) in connection with copies of the covered work
conveyed by you (or copies made from those copies), or (b) primarily
for and in connection with specific products or compilations that
contain the covered work, unless you entered into that arrangement,
or that patent license was granted, prior to 28 March 2007.

  Nothing in this License shall be construed as excluding or limiting
any implied license or other defenses to infringement that may
otherwise be available to you under applicable patent law.

  12. No Surrender of Others' Freedom.

  If conditions are imposed on you (whether by court order, agreement or
otherwise) that contradict the conditions of this License, they do not
excuse you from the conditions of this License.  If you cannot convey a
covered work so as to satisfy simultaneously your obligations under this
License and any other pertinent obligations, then as a consequence you may
not convey it at all.  For example, if you agree to terms that obligate you
to collect a royalty for further conveying from those to whom you convey
the Program, the only way you could satisfy both those terms and this
License would be to refrain entirely from conveying the Program.

  13. Use with the GNU Affero General Public License.

  Notwithstanding any other provision of this License, you have
permission to link or combine any covered work with a work licensed
under version 3 of the GNU Affero General Public License into a single
combined work, and to convey the resulting work.  The terms of this
License will continue to apply to the part which is the covered work,
but the special requirements of the GNU Affero General Public License,
section 13, concerning interaction through a network will apply to the
combination as such.

  14. Revised Versions of this License.

  The Free Software Foundation may publish revised and/or new versions of
the GNU General Public License from time to time.  Such new versions will
be similar in spirit to the present version, but may differ in detail to
address new problems or concerns.

  Each version is given a distinguishing version number.  If the
Program specifies that a certain numbered version of the GNU General
Public License "or any later version" applies to it, you have the
option of following the terms and conditions either of that numbered
version or of any later version published by the Free Software
Foundation.  If the Program does not specify a version number of the
GNU General Public License, you may choose any version ever published
by the Free Software Foundation.

  If the Program specifies that a proxy can decide which future
versions of the GNU General Public License can be used, that proxy's
public statement of acceptance of a version permanently authorizes you
to choose that version for the Program.

  Later license versions may give you additional or different
permissions.  However, no additional obligations are imposed on any
author or copyright holder as a result of your choosing to follow a
later version.

  15. Disclaimer of Warranty.

  THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY
APPLICABLE LAW.  EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT
HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY
OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE.  THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM
IS WITH YOU.  SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF
ALL NECESSARY SERVICING, REPAIR OR CORRECTION.

  16. Limitation of Liability.

  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES AND/OR CONVEYS
THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF
DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS),
EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

  17. Interpretation of Sections 15 and 16.

  If the disclaimer of warranty and limitation of liability provided
above cannot be given local legal effect according to their terms,
reviewing courts shall apply local law that most closely approximates
an absolute waiver of all civil liability in connection with the
Program, unless a warranty or assumption of liability accompanies a
copy of the Program in return for a fee.

                     END OF TERMS AND CONDITIONS

            How to Apply These Terms to Your New Programs

  If you develop a new program, and you want it to be of the greatest
possible use to the public, the best way to achieve this is to make it
free software which everyone can redistribute and change under these terms.

  To do so, attach the following notices to the program.  It is safest
to attach them to the start of each source file to most effectively
state the exclusion of warranty; and each file should have at least
the "copyright" line and a pointer to where the full notice is found.

    <one line to give the program's name and a brief idea of what it does.>
    Copyright (C) <year>  <name of author>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

Also add information on how to contact you by electronic and paper mail.

  If the program does terminal interaction, make it output a short
notice like this when it starts in an interactive mode:

    <program>  Copyright (C) <year>  <name of author>
    This program comes with ABSOLUTELY NO WARRANTY; for details type `show w'.
    This is free software, and you are welcome to redistribute it
    under certain conditions; type `show c' for details.

The hypothetical commands `show w' and `show c' should show the appropriate
parts of the General Public License.  Of course, your program's commands
might be different; for a GUI interface, you would use an "about box".

  You should also get your employer (if you work as a programmer) or school,
if any, to sign a "copyright disclaimer" for the program, if necessary.
For more information on this, and how to apply and follow the GNU GPL, see
<http://www.gnu.org/licenses/>.

  The GNU General Public License does not permit incorporating your program
into proprietary programs.  If your program is a subroutine library, you
may consider it more useful to permit linking proprietary applications with
the library.  If this is what you want to do, use the GNU Lesser General
Public License instead of this License.  But first, please read
<http://www.gnu.org/philosophy/why-not-lgpl.html>.

