#include "Options.h"
#include "corefunctions.h"
#include "distributions.h"
#include "distortion.h"
#include <iostream>
#include <fstream>
#include <cstdlib>
#include <cstring>
#include <exception>
#include "data.h"
using namespace std;

class file_error: public exception{
  static const char*const header;
  char *message;
  char *filename;
public:
  file_error(const char *const mess,const char *file);
  virtual const char* what() const throw();   
};


file_error::file_error(const char *const mess,const char *file){
  message=new char[strlen(mess)+1];
  strcpy(message,mess);
  filename=new char[strlen(file)+1];
  strcpy(filename,file);
};

const char* file_error::what() const throw() {
  char *full_message=new char[strlen(header)+strlen(message)+strlen(filename)+12];
  strcpy(full_message,header);
  strcat(full_message,message);
  strcat(full_message," file: \"");
  strcat(full_message,filename);
  strcat(full_message,"\"");
  return full_message;
};   


char dataformat(){
  unsigned int type=*(unsigned int *)(lookup_option("datatype"));
  switch(type){
  case 0:
    return LIST;
  case 1:
    return FREQUENCY_TABLE;    
  case 2:
    return GROUPED;    
  };
  throw option_formatting_error("Internal error in processing data format.");
};



const char*const file_error::header="File Error: ";


option_formatting_error::option_formatting_error(const char *const mess){
  message=new char[strlen(mess)+1];
  strcpy(message,mess);
};

option_formatting_error::option_formatting_error(const char *const mess,const char*values,const char *end){
  message=new char[strlen(mess)+strlen(values)+strlen(end)+1];
  strcpy(message,mess);
  strcat(message,values);
  strcat(message,end);
};


const char* option_formatting_error::what() const throw(){
  char *full_message=new char[strlen(header)+strlen(message)+1];
  strcpy(full_message,header);
  strcat(full_message,message);
  return full_message;
}; 

const char*const option_formatting_error::header="Formatting error: ";

char *firstmatch(char *&c,const char *lower,const char *upper){
  for(;*c;c++){
    unsigned int j=0;    
    for(;lower+j<upper&&*c&&*(c+j)==*(lower+j);j++);
    if(lower+j==upper){
      return c+j;
    };
  };
  c=NULL;
  return NULL;
};

char **matchpattern(char *string,const char *pat){
  //pat is a very basic pattern currently of the form characters
  //that must match exactly or * which can be any pattern.
  //** represents a literal *
  unsigned int parts=0;
  unsigned int laststar=0;
  for(unsigned int i=0;*(pat+i);i++){
    if(*(pat+i)=='*'){
      if(laststar){
	parts--;
	laststar=0;
      }else{
	parts++;
	laststar=1;
      };
    }else{
      laststar=0;
    };
  };
  const char **isplits=new const char*[parts+1];
  unsigned int j=0;
  for(*(isplits+j)=pat;j<parts&&**(isplits+j);(*(isplits+j))++){
    if(*(*(isplits+j))=='*'){
      if(*(*(isplits+j)+1)!='*'){
	*(isplits+j+1)=*(isplits+j)+1;
	j++;
      }else{
	(*(isplits+j))++;
      };
    };
  };
  for(*(isplits+parts)=pat;*(*(isplits+parts));(*(isplits+parts))++);
  
  j=0;
  const char *ch=pat;
  char *c=string;
  for(;ch<*(isplits+j)&&*ch==*c;ch++,c++);
  if(ch!=*(isplits+j)){
    //No match.
    delete[] isplits;
    return NULL;
  };

  char **startmatch=new char*[parts+1];
  *startmatch=string+1;
  for(unsigned int j=0;**(startmatch+j);){
    for(;j<parts;j++){
      *(startmatch+j+1)=firstmatch(*(startmatch+j),*(isplits+j)+1,*(isplits+j+1));
      if(*(startmatch+j)==NULL){
	//No match, decrease j if possible and find next match
	if(j>1){
	  j--;
	  (*(startmatch+j))++;
	  j--;
	}else{
	  delete[] startmatch;
	  delete[] isplits;
	  return NULL;
	};	
      }else{
      };    
    };
    if(strcmp(*(startmatch+parts),*(isplits+parts))){
      //No final match.
      j--;
      (*(startmatch+j))++;
    };
  };
  //
  char **answer=new char *[parts+1];
  *answer=string+(*isplits-pat);
  for(unsigned int j=0;j<parts-1;j++){
    **(startmatch+j)='\0';
    *(answer+j+1)=*(startmatch+j)+(*(isplits+j+1)-*(isplits+j)-1);
  };
  **(startmatch+parts-1)='\0';
  *(answer+parts)=NULL;
  delete[] isplits;
  delete[] startmatch;
  return answer;
};


class PatternMatch{
public:
  const int matchnumber;
  const char *const* matches;
  PatternMatch(char **m,int n):matchnumber(n),matches(m){};
  PatternMatch():matchnumber(-1),matches(NULL){};//error indicator
};


PatternMatch splitstrings(const char *string,const char *const*patterns){
  //string is the string.
  //patterns is a null-terminated array of allowable patterns


  //Need to modify this to return the pattern which matches.
  char *temp=new char[strlen(string)+1];
  strcpy(temp,string);
  for(unsigned int i=0;*(patterns+i);i++){
    char **ans=matchpattern(temp,*(patterns+i));
    if(ans){      
      return PatternMatch(ans,i);
    };
  };
  //No match found.
  delete[] temp;
  return PatternMatch();
};








int Option::checkname(const char* nm) const{
  return strcmp(name,nm);
};


const char * const truevalues[]={"Y","T","y","t","yes","true","Yes","TRUE","1",NULL};
const char * const falsevalues[]={"N","n","F","false","no","No","False","0",NULL};

void TrueFalseOption::setoption(const char *val){
  if(val){
    value=2;
    for(unsigned int i=0;truevalues[i];i++){
      if(!strcmp(val,truevalues[i])){
	value=1;
      };
    };
    for(unsigned int i=0;truevalues[i];i++){
      if(!strcmp(val,falsevalues[i])){
	value=0;
      };
    };
    if(value==2){
      throw option_formatting_error("Could not convert \"",val,"\" to true or false.");
    };
    this->apply();
  }else{
    value=1;//default is to set option.
  };
};



void IntOption::setoption(const char *val){
  if(val){
    value=atol(val);
    this->apply();
  }else{
    char *message=new char[strlen(name)+35];
    sprintf(message,"Value needed for option \"%s\".\n",name);
    throw option_formatting_error(message);
  };
};


void ModeOption::setoption(const char *val){
  if(val){
    unsigned int i=0;
    for(;*(modes+i)&&strcmp(val,*(modes+i));i++);
    if(*(modes+i)){
      value=i;
      this->apply();
    }else{
      unsigned int lenmodes=0;
      for(i=0;*(modes+i)&&strcmp(val,*(modes+i));i++){
	lenmodes+=strlen(*(modes+i))+4;
      };
      if(lenmodes==0){
	throw option_formatting_error("Program error: allowed values for option \"",name,"\" not defined.");      
      };	
      char *supported_modes=new char[lenmodes+1];
      *(supported_modes)='\"';
      *(supported_modes+1)='\0';
      for(i=0;*(modes+i)&&strcmp(val,*(modes+i));i++){
	strcat(supported_modes,*(modes+i));
	strcat(supported_modes,"\", \"");	
      };
      *(supported_modes+lenmodes-3)='\0';
      char *message=new char[strlen(name)+strlen(val)+73+lenmodes];      
      sprintf(message,"Option \"%s\" does not currently support mode \"%s\".\nSupported values are: %s.",name,val,supported_modes);
      delete[] supported_modes;
      throw option_formatting_error(message);      
    };      
  }else{
    char *message=new char[strlen(name)+35];
    sprintf(message,"Value needed for option \"%s\".\n",name);
    throw option_formatting_error(message);
  };
};



void SetSeedOption::apply(){
  srand(value);
};


void PointerOption::setoption(const char *val){
  char *temp=new char[strlen(val)+1];
  strcpy(temp,val);
  value=temp;
  this->apply();
};



void OutputFileStreamOption::apply(){
  if(value==NULL){
    data=&cout;
  }else{
    ofstream *stream=new ofstream(value);
    if(stream->is_open()){
      data=(ostream*)stream;
    }else{
      throw file_error("Could not open ",value);
    };
  };
};

void OutputFileStreamOption::close(){
  if(value){
    ofstream *stream=(ofstream *)data;
    if(stream->is_open()){
      stream->close();
    }else{
      throw file_error("Could not open ",value);
    };
  };
};


void InputFileStreamOption::apply(){
  if(value==NULL||!strcmp(value,"-")){
    data=&cin;
  }else{
    ifstream *stream=new ifstream(value);
    if(stream->is_open()){
      data=(istream*)stream;
    };
  };
};

void InputFileStreamOption::close(){
  if(value){
    ifstream *stream=(ifstream *)data;
    if(stream->is_open()){
      stream->close();
    };
  };
};




void AdequacyTestOption::apply(){
  if(value==NULL){
    throw option_formatting_error("Option \"",name,"\" requires an argument.");
  }else{
    data=read_adequacy_test(value);
  };
};


const char*const atsplit[]={"*@*",NULL};


void NPestimatorOption::apply(){
  if(value==NULL){
    throw option_formatting_error("Option \"",name,"\" requires an argument.");
  }else{
    const char *const*parts=splitstrings(value,atsplit).matches;
    if(parts==NULL||*parts==NULL){
      throw option_formatting_error("Problem parsing argument \"",value,"\" in option NP.");
    };
    data=get_NP_estimator(*parts,*(parts+1));
  };
};

const char*const pairsplit[]={"[*@*,*@*]","[*,*@*]","[*@*,*]","[*,*]",NULL};

void PairNPestimatorOption::apply(){
  if(value==NULL){
    throw option_formatting_error("Option \"",name,"\" requires an argument.");
  }else{
    PatternMatch parts=splitstrings(value,pairsplit);
    if(parts.matchnumber<0||parts.matches==NULL||*parts.matches==NULL){
      throw option_formatting_error("Problem parsing argument \"",value,"\" in option NP.");
    };
    data=new NPestimator*[2];
    const char *const*temp=(parts.matches+1);
    if(parts.matchnumber&1){
      //matches pattern 1 or 3, so first estimator has no @ value.
      *(NPestimator**)data=get_NP_estimator(*parts.matches,NULL);//NULL gives default value.
    }else{//first estimator has an at value
      temp=parts.matches+2;
      *(NPestimator**)data=get_NP_estimator(*parts.matches,*(parts.matches+1));
    };
    if(parts.matchnumber&2){//matches pattern 2 or 3, so second has no at value
      *((NPestimator**)data+1)=get_NP_estimator(*temp,NULL);//NULL gives default value.
    }else{//first estimator has an at
      *((NPestimator**)data+1)=get_NP_estimator(*temp,*(temp+1));
    };
  };
};


void DistributionOption::apply(){
  if(value==NULL){
    throw option_formatting_error("Option \"",name,"\" requires an argument.");
  }else{
    data=get_distribution(value);
    if(data){
      maxpars=((Distribution *)data)->numparameters();
    }else{
      char *alloweddists=listofpermitteddistributions();
      char *message=new char[strlen(alloweddists)+strlen(value)+73];
      strcpy(message,"Distribution \"");
      strcat(message,value);
      strcat(message,"\" not currently supported.\nSupported distributions are: ");
      strcat(message,alloweddists);
      throw option_formatting_error(message);
    };
  };
};


unsigned int DistributionOption::process(const char *na,const char *val){
  if(data){
    Distribution* d=(Distribution *)data;
    if(npset<maxpars){
      if(d->setpar(na,val)){
	npset++;
	return 1;
      };      
    };
  };
  return 0;
};




unsigned int DelayedProcessOption::process(const char *na,const char *val){
  if(value){
    //Only process if option has been defined
    unsigned int i=0;
    for(;*(allowedparameters+i)&&(strcmp(*(allowedparameters+i),na));i++);
    if(*(allowedparameters+i)){
      //option found. Save for later processing.
      *(parameters+numparameters)=na;
      *(values+numparameters)=val;
      numparameters++;
      return 1;
    };
  };
  return 0;
};

const void *DistortionOption::getoptionval(){
  if(data){
    return data;
  }else if(value){
    data=Read_distortion(value,numparameters,parameters,values);
    return data;
  }else{
    return NULL;//should be OK 
  };
};

