#include <R.h>
#include <Rcpp.h>
#include <Rinternals.h>

#include <datatypes.h>

Data *makedata(SEXP X){
  //X is a vector in R, which is to be coerced into a dataset.
  unsigned int len=LENGTH(X);
  if(TYPEOF(X)==REALSXP){
    //continuous data
    long double *vals=new long double[len];

    const double *v=REAL(X);
    
    for(unsigned int i=0;i<len;i++){
      *(vals+i)=v[i];
    };
    Continuous_Individual_Data *xptr=new Continuous_Individual_Data(len,vals);
    return xptr;
  }else if(TYPEOF(X)==INTSXP){
    //discrete data
    unsigned int *vals=new unsigned int[len];

    const int *v=INTEGER(X);
    for(unsigned int i=0;i<len;i++){
      *(vals+i)=v[i];
    };
    Discrete_Individual_Data *xptr=new Discrete_Individual_Data(len,vals);
    return xptr;    
  }else if(TYPEOF(X)==VECSXP){
    //Table
    if(len==2){
      //Frequency
      SEXP Vals=VECTOR_ELT(X,0);
      SEXP Freqs=VECTOR_ELT(X,1);
      len=LENGTH(Vals);
      if(TYPEOF(Vals)==REALSXP){
	long double *vals=new long double[len];
	unsigned int *freqs=new unsigned int[len];
	const double *v=REAL(Vals);
	const int *f=INTEGER(Freqs);
	
	  for(unsigned int i=0;i<len;i++){
	    *(vals+i)=v[i];
	    *(freqs+i)=f[i];
	  };
	Continuous_Frequency_Data *xptr=new Continuous_Frequency_Data(len,freqs,vals);
	return xptr;
      }else if(TYPEOF(Vals)==INTSXP){
	unsigned int *vals=new unsigned int[len];
	unsigned int *freqs=new unsigned int[len];
	const int *v=INTEGER(Vals);
	const int *f=INTEGER(Freqs);
	
	  for(unsigned int i=0;i<len;i++){
	    *(vals+i)=v[i];
	    *(freqs+i)=f[i];
	  };
	Discrete_Frequency_Data *xptr=new Discrete_Frequency_Data(len,freqs,vals);
	return xptr;
      };
    }else if(len==3){
      //Grouped
      SEXP LowerVals=VECTOR_ELT(X,0);
      SEXP UpperVals=VECTOR_ELT(X,1);
      SEXP Freqs=VECTOR_ELT(X,2);
      len=LENGTH(UpperVals);
      if(TYPEOF(UpperVals)==REALSXP){
	long double *vals=new long double[len];
	unsigned int *freqs=new unsigned int[len];
	const double *v=REAL(UpperVals);
	const int *f=INTEGER(Freqs);
	
	  for(unsigned int i=0;i<len;i++){
	    *(vals+i)=v[i];
	    *(freqs+i)=f[i];
	  };
	Continuous_Grouped_Data *xptr=new Continuous_Grouped_Data(len,freqs,vals);
	return xptr;
      }else if(TYPEOF(UpperVals)==INTSXP){
	unsigned int *vals=new unsigned int[len];
	unsigned int *freqs=new unsigned int[len];
	const int *v=INTEGER(UpperVals);
	const int *f=INTEGER(Freqs);
	
	  for(unsigned int i=0;i<len;i++){
	    *(vals+i)=v[i];
	    *(freqs+i)=f[i];
	  };
	Discrete_Grouped_Data *xptr=new Discrete_Grouped_Data(len,freqs,vals);
	return xptr;
      };      
    };
  };  
};
