#include "datatypes.h"
#include <cstdlib>
#include <cmath>
#include <iostream>
#include <iomanip>

using namespace std;

#include "general.h"

unsigned int rbinom(unsigned int n,long double p){
  unsigned int cuttoff=p*RAND_MAX;
  unsigned int ans=0;
  for(unsigned int i=0;i<n;i++){
    if((unsigned int)rand()<cuttoff){
      ans++;
    };
  };
  return ans;
};


void makebintable(unsigned int n,long double p,unsigned int min,unsigned int max,long double* to){
  //  cout<<"n="<<n<<", p="<<p<<", min="<<min<<", max="<<max<<"\n";
  long double prob=1;
  *to=1;
  long double a=-p/(1-p);
  long double b=-a*(n+1);
  //  cout<<"a="<<a<<", b="<<b<<"\n";
  for(unsigned int i=min;i<max;i++){
    prob*=a+b/(i+1);
    //    cout<<prob<<" ";
    *(to+i-min+1)=*(to+i-min)+prob;
  };
  for(unsigned int i=0;i<=max-min;i++){
    *(to+i)/=*(to+max-min);
    //        cout<<*(to+i)<<" ";
  };  
  //    cout<<"\n";
};


unsigned int getrndfromlkup(unsigned int min,unsigned int max,const long double *lkup){
  unsigned int mn=0;
  unsigned int mx=max-min;
  long double v=((long double)rand())/RAND_MAX;
  //  cout<<v<<"\n";
  while(mx-mn>1){
    unsigned int test=(mx+mn)/2;
    //      cout<<test<<" "<<*(lkup+test)<<"\n";
    if(v<*(lkup+test)){
      mx=test;
    }else{
      mn=test;
    };    
  };
  if(v<(*(lkup+mn))){
      return min+mn;
    }else{
      return min+mx;
    };
};

unsigned int rbinom_fast(unsigned int n,long double p){
  if(n<400){
    return rbinom(n,p);
  };


  //  cout<<"\n";
  unsigned int N=pow(n,0.666666666666666666667);

  long double mean=N*p;
  long double sd=sqrt(N*p*(1-p));
  long double minplaus=mean-8*sd;
  unsigned int min=minplaus;
  if(minplaus<0){
    min=0;
  };
  long double maxplaus=mean+8*sd;
  unsigned int max=maxplaus;
  if(maxplaus<0){
    max=0;
  };
  long double *lkuptab=new long double[max+1-min];
  makebintable(N,p,min,max,lkuptab);
  //Start by making a lookuptable for size N. Then generate n/N random
  //binomials for N and sum them.

  unsigned int k=n/N;
  unsigned int tot=0;
  //  cout<<"N="<<N<<", k="<<k<<"\n"; 
  for(unsigned int i=0;i<k;i++){
    unsigned int rbinN=getrndfromlkup(min,max,lkuptab);
    tot+=rbinN;
    //    cout<<rbinN<<"  ";
  };
  //  cout<<"\n";
  unsigned int rem=n-k*N;
  delete[] lkuptab;
  //  cout<<"N="<<N<<", k="<<k<<", rem="<<rem<<"\n";
  return tot+rbinom_fast(rem,p);
};


unsigned int rhgeom(unsigned int n,unsigned int success,unsigned int tot){
  unsigned int ans=0;
  for(unsigned int i=0;i<n;i++){
    unsigned int cuttoff=(((long unsigned int)success)*RAND_MAX)/tot;
    if((unsigned int)rand()<cuttoff){
      ans++;
      success--;
    };
    tot--;
  };
  return ans;
};




void Data::subsample(unsigned int size){
  if(!sample){
    sample=new unsigned int[n];
  };
  if(!jackknife){
    if(size<n){    
      for(unsigned int i=0;i<n;i++){
	*(sample+i)=0;
      };
      unsigned int rmx=n*(RAND_MAX/n);
      for(unsigned int i=0;i<size;i++){      
	unsigned int rnd=rand();
	while(rnd>rmx){
	  rnd=rand(); //redraw if too large.
	};
	rnd=rnd%n;
	(*(sample+rnd))++;      
      };  
    }else{
      unsigned int rc=size;
      for(unsigned int i=0;i<n;i++){
	*(sample+i)=rbinom_fast(rc,1/((long double)(n-i)));
	rc-=*(sample+i);
      };
    };
  }else{
    //jackknife Assume size<=n
    //if size>n selects all datapoints
    for(unsigned int i=0;i<n;i++){
      unsigned int rmx=(n-i)*(RAND_MAX/(n-i));
      unsigned int rnd=rand();
      while(rnd>rmx){
	rnd=rand(); //redraw if too large.
      };
      rnd=rnd%(n-i);
      if(rnd<size){
	*(sample+i)=1;
	size--;
      }else{
	*(sample+i)=0;
      };
    };      
  };
};

void Data::clearsubsample(unsigned int size){
  if(sample){
    delete[] sample;
  };
  sample=NULL;
};


void Frequency_Data::subsample(unsigned int size){
  if(!sample){
    sample=new unsigned int[n];
  };
  if(jackknife){
    unsigned int *cumcount=new unsigned int[n];
    *(cumcount+n-1)=*(freq+n-1);
    for(unsigned int i=n-1;i>0;i--){
      *(cumcount+i-1)=*(cumcount+i)+*(freq+i-1);
    };    
    //    unsigned int tc=*(cumcount);
    //    unsigned int rmx=tc*(RAND_MAX/tc);
    unsigned int rc=size;
    for(unsigned int i=0;i<n;i++){
      *(sample+i)=rhgeom(rc,*(freq+i),*(cumcount+i));
      rc-=*(sample+i);
    };
    delete[] cumcount;
  }else{
    if(size<n){
      unsigned int *cumcount=new unsigned int[n];
      for(unsigned int i=0;i<n;i++){
	*(sample+i)=0;
	if(i==0){
	  *cumcount=*freq;
	}else{
	  *(cumcount+i)=*(cumcount+i-1)+*(freq+i);
	};
      };
      unsigned int tc=*(cumcount+n-1);
      unsigned int rmx=tc*(RAND_MAX/tc);
      for(unsigned int i=0;i<size;i++){
	unsigned int rnd=rand();
	while(rnd>rmx){
	  rnd=rand(); //redraw if too large.
	};
	rnd=rnd%tc;
	if(rnd<*cumcount){
	  (*sample)++;
	}else{
	  unsigned int upper=n;
	  unsigned int lower=0;
	  while(upper-lower>1){
	    unsigned int mid=(upper+lower)/2;
	    if(*(cumcount+mid)<=rnd){
	      lower=mid;
	    }else{
	      upper=mid;
	    };
	  };
	  (*(sample+upper))++;
	};
      };  
      delete[] cumcount;
    }else{
      unsigned int *cumcount=new unsigned int[n];
      *(cumcount+n-1)=*(freq+n-1);
      for(unsigned int i=n-1;i>0;i--){
	*(cumcount+i-1)=*(cumcount+i)+*(freq+i-1);
      };    
      //    unsigned int tc=*(cumcount);
      //    unsigned int rmx=tc*(RAND_MAX/tc);
      unsigned int rc=size;
      for(unsigned int i=0;i<n;i++){
	*(sample+i)=rbinom_fast(rc,*(freq+i)/((long double)*(cumcount+i)));
	rc-=*(sample+i);
      };
      delete[] cumcount;
    };
  };
};


const void* Continuous_Frequency_Data::getvalues() const {
  return data;
};


const unsigned int *Continuous_Frequency_Data::getfreqs() const {
  return (sample?sample:freq);
};


const void* Discrete_Frequency_Data::getvalues() const {
  return data;
};


const unsigned int *Discrete_Frequency_Data::getfreqs() const {
  return (sample?sample:freq);
};


const void* Continuous_Individual_Data::getvalues() const {
  return data;
};


const unsigned int *Continuous_Individual_Data::getfreqs() const {
  return sample;
};


const void* Discrete_Individual_Data::getvalues() const {
  return data;
};


const unsigned int *Discrete_Individual_Data::getfreqs() const {
  return sample;
};



const void* Continuous_Grouped_Data::getvalues() const {
  return cutoffs;
};


const unsigned int *Continuous_Grouped_Data::getfreqs() const {
  return (sample?sample:freq);
};


const void* Discrete_Grouped_Data::getvalues() const {
  return cutoffs;
};


const unsigned int *Discrete_Grouped_Data::getfreqs() const {
  return (sample?sample:freq);
};


Data* new_data(const datatype &type){
  if(type.type_of_data&(type.type_of_data-1)){
    //Not valid - data type should be a power of 2
    return NULL;
  };
  if(type.storage&(type.storage-1)){
    //Not valid - data type should be a power of 2
    return NULL;
  };
  switch(type.type_of_data){
  case DISCRETE_DATA:
    switch(type.storage){
    case LIST:
      return new Discrete_Individual_Data();
    case FREQUENCY_TABLE:
      return new Discrete_Frequency_Data();
    case GROUPED:
      return new Discrete_Grouped_Data();
    default:
      return NULL;
    };
  case CONTINUOUS_DATA:
    switch(type.storage){
    case LIST:
      return new Continuous_Individual_Data();
    case FREQUENCY_TABLE:
      return new Continuous_Frequency_Data();
    case GROUPED:
      return new Continuous_Grouped_Data();
    default:
      return NULL;
    };
  default:
    return NULL;
  };
};


void Discrete_Individual_Data::read(std::istream& in){
  long double rd;
  in>>rd;
  uill datlist(rd);
  while(in.good()&&!in.eof()){
    in>>rd;
    if(in.good()&&!in.eof()){
      datlist.insert(rd);
    };
  };
  
  n=datlist.length();
  data=new unsigned int[n];
  datlist.write(data);
  
  this->sort();
};

void Discrete_Frequency_Data::read(std::istream& in){
    long double rd;
    unsigned int count;
    in>>rd;
    in>>count;
    uictll datlist(rd,count);
    while(in.good()&&!in.eof()){
      in>>rd;
      in>>count;
      if(in.good()&&!in.eof()){
	datlist.insert(rd,count);
      };
    };
    
    n=datlist.length();
    data=new unsigned int[n];
    freq=new unsigned int[n];
    unsigned int *tempdata=new unsigned int[n];
    unsigned int *tempcount=new unsigned int[n];
    datlist.write(tempdata,tempcount);
    for(unsigned int i=0;i<n;i++){
      *(freq+i)=*(tempcount+n-1-i);
      *(data+i)=*(tempdata+n-1-i);
    };    
    delete[] tempdata;
    delete[] tempcount;
    this->sort();
};

void Discrete_Grouped_Data::read(std::istream& in){
    unsigned int rd;
    unsigned int count;
    in>>rd;
    char c;
    in>>c;
    if(c=='-'){
      in>>rd;
    };
    in>>count;
    uictll datlist(rd,count);
    while(in.good()&&!in.eof()){
      in>>rd;
      in>>c;
      if(c=='-'){
	in>>rd;
      };
      in>>count;
      if(in.good()&&!in.eof()){
	datlist.insert(rd,count);
      };
    };
    
    n=datlist.length();
    cutoffs=new unsigned int[n];
    freq=new unsigned int[n];
    unsigned int *tempdata=new unsigned int[n];
    unsigned int *tempcount=new unsigned int[n];
    datlist.write(tempdata,tempcount);
    for(unsigned int i=0;i<n;i++){
      *(freq+i)=*(tempcount+n-1-i);
      *(cutoffs+i)=*(tempdata+n-1-i);
    };    
    delete[] tempdata;
    delete[] tempcount;  
    this->sort();
};

void Continuous_Frequency_Data::read(std::istream& in){
    long double rd;
    unsigned int count;
    in>>rd;
    in>>count;
    dblctll datlist(rd,count);
    while(in.good()&&!in.eof()){
      in>>rd;
      in>>count;
      if(in.good()&&!in.eof()){
	datlist.insert(rd,count);
      };
    };
    
    n=datlist.length();
    data=new long double[n];
    freq=new unsigned int[n];
    long double *tempdata=new long double[n];
    unsigned int *tempcount=new unsigned int[n];
    datlist.write(tempdata,tempcount);
    for(unsigned int i=0;i<n;i++){
      *(freq+i)=*(tempcount+n-1-i);
      *(data+i)=*(tempdata+n-1-i);
    };    
    delete[] tempdata;
    delete[] tempcount;
    this->sort();
};

void Continuous_Individual_Data::read(std::istream& in){
  long double rd;
  in>>rd;
  dblll datlist(rd);
  while(in.good()&&!in.eof()){
    in>>rd;
    if(in.good()&&!in.eof()){
      datlist.insert(rd);
    };
  };
    
  n=datlist.length();
  data=new long double[n];
  datlist.write(data);
  
  this->sort();
};

void Continuous_Grouped_Data::read(std::istream& in){
  long double rd;
  unsigned int count;
  in>>rd;
  char c;
  in>>c;
  if(c=='-'){
    in>>rd;
  };
  in>>count;
  dblctll datlist(rd,count);
  while(in.good()&&!in.eof()){
    in>>rd;
    in>>c;
    if(c=='-'){
      in>>rd;
    };
    in>>count;
    if(in.good()&&!in.eof()){
      datlist.insert(rd,count);
    };
  };
     
  n=datlist.length();
  cutoffs=new long double[n];
  freq=new unsigned int[n];
  long double *tempdata=new long double[n];
  unsigned int *tempcount=new unsigned int[n];
  datlist.write(tempdata,tempcount);
  for(unsigned int i=0;i<n;i++){
    *(freq+i)=*(tempcount+n-1-i);
    *(cutoffs+i)=*(tempdata+n-1-i);
  };    
  delete[] tempdata;
  delete[] tempcount;
  this->sort();
};


void Discrete_Individual_Data::concat(const Data *cdat){
  if(cdat){
    if(cdat->get_type().type_of_data&DISCRETE_DATA){
      switch(cdat->get_type().storage){
      case LIST:
	{
	  unsigned int *newdata=new unsigned int[n+cdat->n];
	  unsigned int i=0;
	  unsigned int j=0;
	  const unsigned int*cddata=(const unsigned int*)cdat->getvalues();
	  for(;i<n&&j<cdat->n;){
	    if(*(data+i)<*(cddata+j)){
	      *(newdata+i+j)=*(data+i);
	      i++;
	    }else{
	      *(newdata+i+j)=*(cddata+j);
	      j++;
	    };
	  };
	  for(;i<n;i++){
	    *(newdata+i+j)=*(data+i);	      
	  };
	  for(;j<cdat->n;j++){
	    *(newdata+i+j)=*(cddata+j);
	  };
	  n+=cdat->n;
	  delete[] data;
	  qsort(newdata,n,sizeof(unsigned int),compareUnsignedInt);
	  data=newdata;	  
	  if(sample){
	    delete[] sample;
	  };
	  return;
	};
      case FREQUENCY_TABLE:
	{
	  cout<<"Attempting to concatenate list and frequency table.\n";
	  cout<<"May result in unacceptably long data lists.\n";
	  unsigned int tot=0;
	  const unsigned int *frqs=cdat->getfreqs();
	  for(unsigned int i=0;i<cdat->n;i++){
	    tot+=*(frqs+i);
	  };
	  unsigned int *newdata=new unsigned int[n+tot];
	  unsigned int i=0;
	  unsigned int j=0;
	  unsigned int jj=0;
	  const unsigned int*cddata=(const unsigned int*)cdat->getvalues();
	  for(;i<n&&j<cdat->n;){
	    if(*(data+i)<*(cddata+j)){
	      *(newdata+i+jj)=*(data+i);
	      i++;
	    }else{
	      for(unsigned int k=0;k<*(frqs+j);k++){
		*(newdata+i+jj)=*(cddata+j);
		jj++;
	      };
	      j++;
	    };
	  };
	  for(;i<n;i++){
	    *(newdata+i+jj)=*(data+i);	      
	  };
	  for(;j<cdat->n;j++){
	    for(unsigned int k=0;k<*(frqs+j);k++){
	      *(newdata+i+jj)=*(cddata+j);
	      jj++;
	    };
	  };
	  n+=cdat->n;
	  delete[] data;
	  qsort(newdata,n,sizeof(unsigned int),compareUnsignedInt);
	  data=newdata;	  
	  if(sample){
	    delete[] sample;
	  };
	  return;
	};
      default:
	cout<<"Unable to concatenate data.\n";
	exit(0);
      };
    }else{
      cout<<"Attempting to concatenate discrete and non-discrete data.\n";
      exit(0);
    };   
  };
};

void Discrete_Frequency_Data::concat(const Data *cdat){
  if(cdat){
    if(cdat->get_type().type_of_data&DISCRETE_DATA){
      switch(cdat->get_type().storage){
      case LIST:
      case FREQUENCY_TABLE:
	{
	  const unsigned int *frqs=cdat->getfreqs();
	  const unsigned int*cddata=(const unsigned int*)cdat->getvalues();
	  unsigned int newn=n+cdat->n;
	  unsigned int *newdata=new unsigned int[newn];
	  unsigned int *newfreqs=new unsigned int[newn];
	  unsigned int pos=0;
	  unsigned int i=0;
	  unsigned int j=0;
	  if(frqs){
	    for(;i<n&&j<cdat->n;pos++){
	      if(*(data+i)<*(cddata+j)){
		*(newdata+pos)=*(data+i);
		*(newfreqs+pos)=*(freq+i);
		for(i++;i<n&&*(data+i)==*(newdata+pos);i++){
		  *(newfreqs+pos)+=*(freq+i);
		};
	      }else if(*(cddata+j)<*(data+i) ){
		*(newdata+pos)=*(cddata+j);
		*(newfreqs+pos)=*(frqs+j);
		for(j++;j<cdat->n&&*(cddata+j)==*(newdata+pos);j++){
		  *(newfreqs+pos)+=*(frqs+j);
		};
	      }else{
		*(newdata+pos)=*(cddata+j);
		*(newfreqs+pos)=*(freq+i)+*(frqs+j);
		for(i++;i<n&&*(data+i)==*(newdata+pos);i++){
		  *(newfreqs+pos)+=*(freq+i);
		};
		for(j++;j<cdat->n&&*(cddata+j)==*(newdata+pos);j++){
		  *(newfreqs+pos)+=*(frqs+j);
		};
	      };
	    };
	    for(;i<n;i++){
	      *(newdata+pos)=*(data+i);
	      *(newfreqs+pos)=*(freq+i);
	      for(i++;i<n&&*(data+i)==*(newdata+pos);i++){
		*(newfreqs+pos)+=*(freq+i);
	      };	      
	      pos++;
	    };
	    for(;j<cdat->n;j++){
	      *(newdata+i+j)=*(cddata+j);
	      *(newfreqs+pos)=*(frqs+j);
	      for(j++;j<cdat->n&&*(cddata+j)==*(newdata+pos);i++){
		*(newfreqs+pos)+=*(freq+i);
	      };
	      pos++;
	    };
	    n=pos;
	    delete[] data;
	    delete[] freq;
	    data=newdata;	  
	    freq=newfreqs;
	    if(sample){
	      delete[] sample;
	    };
	    return;
	  }else{
	    for(;i<n&&j<cdat->n;pos++){
	      if(*(data+i)<*(cddata+j)){
		*(newdata+pos)=*(data+i);
		*(newfreqs+pos)=*(freq+i);
		for(i++;i<n&&*(data+i)==*(newdata+pos);i++){
		  *(newfreqs+pos)+=*(freq+i);
		};
	      }else if(*(cddata+j)<*(data+i) ){
		*(newdata+pos)=*(cddata+j);
		*(newfreqs+pos)=1;
		for(j++;j<cdat->n&&*(cddata+j)==*(newdata+pos);j++){
		  (*(newfreqs+pos))++;
		};
	      }else{
		*(newdata+pos)=*(cddata+j);
		*(newfreqs+pos)=*(freq+i)+1;
		for(i++;i<n&&*(data+i)==*(newdata+pos);i++){
		  *(newfreqs+pos)+=*(freq+i);
		};
		for(j++;j<cdat->n&&*(cddata+j)==*(newdata+pos);j++){
		  (*(newfreqs+pos))++;
		};
	      };
	    };
	    for(;i<n;i++){
	      *(newdata+pos)=*(data+i);
	      *(newfreqs+pos)=*(freq+i);
	      for(i++;i<n&&*(data+i)==*(newdata+pos);i++){
		*(newfreqs+pos)+=*(freq+i);
	      };	      
	      pos++;
	    };
	    for(;j<cdat->n;j++){
	      *(newdata+i+j)=*(cddata+j);
	      *(newfreqs+pos)=1;
	      for(j++;j<cdat->n&&*(cddata+j)==*(newdata+pos);i++){
		(*(newfreqs+pos))++;
	      };
	      pos++;
	    };
	    n=pos;
	    delete[] data;
	    delete[] freq;
	    data=newdata;	  
	    freq=newfreqs;
	    if(sample){
	      delete[] sample;
	    };
	    return;
	  };
	};
      default:
	cout<<"Unable to concatenate data.\n";
	exit(0);
      };
    }else{
      cout<<"Attempting to concatenate discrete and non-discrete data.\n";
      exit(0);
    };   
  };
};

void Continuous_Individual_Data::concat(const Data *cdat){
  if(cdat){
    if(cdat->get_type().type_of_data&CONTINUOUS_DATA){
      switch(cdat->get_type().storage){
      case LIST:
	{
	  long double *newdata=new long double[n+cdat->n];
	  unsigned int i=0;
	  unsigned int j=0;
	  const long double*cddata=(const long double*)cdat->getvalues();
	  for(;i<n&&j<cdat->n;){
	    if(*(data+i)<*(cddata+j)){
	      *(newdata+i+j)=*(data+i);
	      i++;
	    }else{
	      *(newdata+i+j)=*(cddata+j);
	      j++;
	    };
	  };
	  for(;i<n;i++){
	    *(newdata+i+j)=*(data+i);	      
	  };
	  for(;j<cdat->n;j++){
	    *(newdata+i+j)=*(cddata+j);
	  };
	  n+=cdat->n;
	  delete[] data;
	  data=newdata;	  
	  if(sample){
	    delete[] sample;
	  };
	  return;
	};
      case FREQUENCY_TABLE:
	{
	  cout<<"Attempting to concatenate list and frequency table.\n";
	  cout<<"May result in unacceptably long data lists.\n";
	  unsigned int tot=0;
	  const unsigned int *frqs=cdat->getfreqs();
	  for(unsigned int i=0;i<cdat->n;i++){
	    tot+=*(frqs+i);
	  };
	  long double *newdata=new long double[n+tot];
	  unsigned int i=0;
	  unsigned int j=0;
	  unsigned int jj=0;
	  const long double*cddata=(const long double*)cdat->getvalues();
	  for(;i<n&&j<cdat->n;){
	    if(*(data+i)<*(cddata+j)){
	      *(newdata+i+jj)=*(data+i);
	      i++;
	    }else{
	      for(unsigned int k=0;k<*(frqs+j);k++){
		*(newdata+i+jj)=*(cddata+j);
		jj++;
	      };
	      j++;
	    };
	  };
	  for(;i<n;i++){
	    *(newdata+i+jj)=*(data+i);	      
	  };
	  for(;j<cdat->n;j++){
	    for(unsigned int k=0;k<*(frqs+j);k++){
	      *(newdata+i+jj)=*(cddata+j);
	      jj++;
	    };
	  };
	  n+=cdat->n;
	  delete[] data;
	  data=newdata;	  
	  if(sample){
	    delete[] sample;
	  };
	  return;
	};
      default:
	cout<<"Unable to concatenate data.\n";
	exit(0);
      };
    }else{
      cout<<"Attempting to concatenate continuous and non-continuous data.\n";
      exit(0);
    };   
  };
};

void Continuous_Frequency_Data::concat(const Data *cdat){
  if(cdat){
    if(cdat->get_type().type_of_data&CONTINUOUS_DATA){
      switch(cdat->get_type().storage){
      case LIST:
      case FREQUENCY_TABLE:
	{
	  const unsigned int *frqs=cdat->getfreqs();
	  const long double*cddata=(const long double*)cdat->getvalues();
	  unsigned int newn=n+cdat->n;
	  long double *newdata=new long double[newn];
	  unsigned int *newfreqs=new unsigned int[newn];
	  unsigned int pos=0;
	  unsigned int i=0;
	  unsigned int j=0;
	  if(frqs){
	    for(;i<n&&j<cdat->n;pos++){
	      if(*(data+i)<*(cddata+j)){
		*(newdata+pos)=*(data+i);
		*(newfreqs+pos)=*(freq+i);
		for(i++;i<n&&*(data+i)==*(newdata+pos);i++){
		  *(newfreqs+pos)+=*(freq+i);
		};
	      }else if(*(cddata+j)<*(data+i) ){
		*(newdata+pos)=*(cddata+j);
		*(newfreqs+pos)=*(frqs+j);
		for(j++;j<cdat->n&&*(cddata+j)==*(newdata+pos);j++){
		  *(newfreqs+pos)+=*(frqs+j);
		};
	      }else{
		*(newdata+pos)=*(cddata+j);
		*(newfreqs+pos)=*(freq+i)+*(frqs+j);
		for(i++;i<n&&*(data+i)==*(newdata+pos);i++){
		  *(newfreqs+pos)+=*(freq+i);
		};
		for(j++;j<cdat->n&&*(cddata+j)==*(newdata+pos);j++){
		  *(newfreqs+pos)+=*(frqs+j);
		};
	      };
	    };
	    for(;i<n;i++){
	      *(newdata+pos)=*(data+i);
	      *(newfreqs+pos)=*(freq+i);
	      for(i++;i<n&&*(data+i)==*(newdata+pos);i++){
		*(newfreqs+pos)+=*(freq+i);
	      };	      
	      pos++;
	    };
	    for(;j<cdat->n;j++){
	      *(newdata+i+j)=*(cddata+j);
	      *(newfreqs+pos)=*(frqs+j);
	      for(j++;j<cdat->n&&*(cddata+j)==*(newdata+pos);i++){
		*(newfreqs+pos)+=*(freq+i);
	      };
	      pos++;
	    };
	    n=pos;
	    delete[] data;
	    delete[] freq;
	    data=newdata;	  
	    freq=newfreqs;
	    if(sample){
	      delete[] sample;
	    };
	    return;
	  }else{
	    for(;i<n&&j<cdat->n;pos++){
	      if(*(data+i)<*(cddata+j)){
		*(newdata+pos)=*(data+i);
		*(newfreqs+pos)=*(freq+i);
		for(i++;i<n&&*(data+i)==*(newdata+pos);i++){
		  *(newfreqs+pos)+=*(freq+i);
		};
	      }else if(*(cddata+j)<*(data+i) ){
		*(newdata+pos)=*(cddata+j);
		*(newfreqs+pos)=1;
		for(j++;j<cdat->n&&*(cddata+j)==*(newdata+pos);j++){
		  (*(newfreqs+pos))++;
		};
	      }else{
		*(newdata+pos)=*(cddata+j);
		*(newfreqs+pos)=*(freq+i)+1;
		for(i++;i<n&&*(data+i)==*(newdata+pos);i++){
		  *(newfreqs+pos)+=*(freq+i);
		};
		for(j++;j<cdat->n&&*(cddata+j)==*(newdata+pos);j++){
		  (*(newfreqs+pos))++;
		};
	      };
	    };
	    for(;i<n;i++){
	      *(newdata+pos)=*(data+i);
	      *(newfreqs+pos)=*(freq+i);
	      for(i++;i<n&&*(data+i)==*(newdata+pos);i++){
		*(newfreqs+pos)+=*(freq+i);
	      };	      
	      pos++;
	    };
	    for(;j<cdat->n;j++){
	      *(newdata+i+j)=*(cddata+j);
	      *(newfreqs+pos)=1;
	      for(j++;j<cdat->n&&*(cddata+j)==*(newdata+pos);i++){
		(*(newfreqs+pos))++;
	      };
	      pos++;
	    };
	    n=pos;
	    delete[] data;
	    delete[] freq;
	    data=newdata;	  
	    freq=newfreqs;
	    if(sample){
	      delete[] sample;
	    };
	    return;
	  };
	};
      default:
	cout<<"Unable to concatenate data.\n";
	exit(0);
      };
    }else{
      cout<<"Attempting to concatenate discrete and non-discrete data.\n";
      exit(0);
    };   
  };
};

void Continuous_Grouped_Data::concat(const Data *cdat){
  if(cdat){
    if(cdat->get_type().type_of_data&CONTINUOUS_DATA){
      switch(cdat->get_type().storage){
      case GROUPED:
      case LIST:
      case FREQUENCY_TABLE:
	{
	};
      default:
	cout<<"Unable to concatenate data.\n";
	exit(0);
      };
    }else{
      cout<<"Attempting to concatenate continuous and non-continuous data.\n";
      exit(0);
    };   
  };
};


void Discrete_Grouped_Data::concat(const Data *cdat){
  if(cdat){
    if(cdat->get_type().type_of_data&DISCRETE_DATA){
    }else{
      cout<<"Attempting to concatenate discrete and non-discrete data.\n";
      exit(0);
    };   
  };
};





void Discrete_Individual_Data::sort(){
  qsort(data,n,sizeof(unsigned int),compareUnsignedInt);    
};

void Continuous_Individual_Data::sort(){
  qsort(data,n,sizeof(long double),compareLongDouble);    
};

void Discrete_Frequency_Data::sort(){
    // Assume frequency table is already sorted
};


void Continuous_Frequency_Data::sort(){
    // Assume frequency table is already sorted
};

void Continuous_Grouped_Data::sort(){
    // Assume frequency table is already sorted
};


void Discrete_Grouped_Data::sort(){
    // Assume frequency table is already sorted
};


#include <boost/io/ios_state.hpp>

unsigned int getcolwidth(unsigned int n,unsigned int *data){
  //Calculates the width needed to display a given sequence of numbers
  unsigned int max=0;
  for(unsigned int i=0;i<n;i++){
    if(*(data+i)>max){
      max=*(data+i);
    };//for unsigned integers, stringlength is increasing.
  };
  unsigned int columnwidth=2;
  for(unsigned int pten=1;pten<max;pten*=10){
    columnwidth++;
  };
  return columnwidth;
};

unsigned int getcolwidth(unsigned int n,long double *data,ostream &to){
  //calculates the width needed to display numbers
  //also selects a suitable precision value.
  long double max=0;
  long double min=0;
  for(unsigned int i=0;i<n;i++){
    if(abs(*(data+i))>max){
      max=abs(*(data+i));
    };
    if(*(data+i)!=0){
      if(min==0||abs(*(data+i))<min){
	min=abs(*(data+i));
      };
    };
  };//Now range of non-zero data values is from max to min.
  int maxsize=floor(log(max)/log(10));
  int minsize=floor(log(min)/log(10));
  unsigned int columnwidth=0;
  //  cout<<"maxsize="<<maxsize<<"\tminsize="<<minsize<<"\n";
  if((maxsize-minsize>4)||(maxsize>7)||(minsize<-4)){
    to<<scientific;
    to<<setprecision(7);
    columnwidth=16;//-1.1234567e-123 plus a space.
                 //Might need to change this if using types that can produce
                 //4 digit exponents. 
  }else{
    to<<fixed;
    to<<setprecision(7-(int)maxsize);
    columnwidth=11;//-1234.5678 plus a space.
  };
  return columnwidth;
};


void Discrete_Individual_Data::write(ostream &to, unsigned int numcols){
  boost::io::ios_flags_saver resetstateofstream(to);
  unsigned int columnwidth=getcolwidth(n,data);
  for(unsigned int i=0;i<n;i++){
    to<<setw(columnwidth)<<*(data+i);
    if(!((i+1)%numcols)){
      to<<"\n";
    };
  };
  if((n-1)%numcols){//add newline after data.
    to<<"\n";
  };  
};

void Continuous_Individual_Data::write(ostream &to, unsigned int numcols){
  boost::io::ios_flags_saver resetstateofstream(to);
  unsigned int columnwidth=getcolwidth(n,data,to);
  for(unsigned int i=0;i<n;i++){
    to<<setw(columnwidth)<<*(data+i);
    if(!((i+1)%numcols)){
      to<<"\n";
    };
  };
  if((n-1)%numcols){//add newline after data.
    to<<"\n";
  };  
};

void Discrete_Frequency_Data::write(ostream &to, unsigned int numcols){
  //numcols is the number of columns in the table.
  unsigned int numrows=n/numcols;
  unsigned int leftover=n%numcols;
  //Data consists of numrows rows with leftover additional entries.
  //We will sort in columns;
  unsigned int datawidth=getcolwidth(n,data);
  unsigned int freqwidth=getcolwidth(n,freq);
  const char *colsep="   ";//three spaces to separate columns;
  for(unsigned int i=0;i<numrows;i++){
    for(unsigned int j=0;j<leftover;j++){
      to<<setw(datawidth)<<*(data+j*(numrows+1)+i);
      to<<setw(freqwidth)<<*(freq+j*(numrows+1)+i);
      to<<colsep;
    };
    for(unsigned int j=leftover;j<n;j++){
      to<<setw(datawidth)<<*(data+j*numrows+i+leftover);
      to<<setw(freqwidth)<<*(freq+j*numrows+leftover+i);
      to<<colsep;
    };
    to<<"\n";
  };
  for(unsigned int j=0;j<leftover;j++){
    to<<setw(datawidth)<<*(data+(j+1)*(numrows+1)-1);
    to<<setw(freqwidth)<<*(freq+(j+1)*(numrows+1)-1);
    to<<colsep;
  };
  to<<"\n";
};


void Continuous_Frequency_Data::write(ostream &to, unsigned int numcols){
  //numcols is the number of columns in the table.
  unsigned int numrows=n/numcols;
  unsigned int leftover=n%numcols;
  //Data consists of numrows rows with leftover additional entries.
  //We will sort in columns;
  unsigned int datawidth=getcolwidth(n,data,to);
  unsigned int freqwidth=getcolwidth(n,freq);
  const char *colsep="   ";//three spaces to separate columns;
  for(unsigned int i=0;i<numrows;i++){
    for(unsigned int j=0;j<leftover;j++){
      to<<setw(datawidth)<<*(data+j*(numrows+1)+i);
      to<<setw(freqwidth)<<*(freq+j*(numrows+1)+i);
      to<<colsep;
    };
    for(unsigned int j=leftover;j<n;j++){
      to<<setw(datawidth)<<*(data+j*numrows+i+leftover);
      to<<setw(freqwidth)<<*(freq+j*numrows+leftover+i);
      to<<colsep;
    };
    to<<"\n";
  };
  for(unsigned int j=0;j<leftover;j++){
    to<<setw(datawidth)<<*(data+(j+1)*(numrows+1)-1);
    to<<setw(freqwidth)<<*(freq+(j+1)*(numrows+1)-1);
    to<<colsep;
  };
  to<<"\n";
};

void Continuous_Grouped_Data::write(ostream &to, unsigned int numcols){
  //numcols is the number of columns in the table.
  unsigned int numrows=n/numcols;
  unsigned int leftover=n%numcols;
  //Data consists of numrows rows with leftover additional entries.
  //We will sort in columns;
  unsigned int datawidth=getcolwidth(n,cutoffs,to);
  unsigned int freqwidth=getcolwidth(n,freq);
  const char *colsep="   ";//three spaces to separate columns;
  to<<setw(datawidth)<<0<<" - ";//first cutoff set to zero ?????
  //perhaps first cutoff should be a special field for grouped data.
  for(unsigned int i=0;i<numrows;i++){
    for(unsigned int j=0;j<leftover;j++){      
      if(i+j){
	to<<setw(datawidth)<<*(cutoffs+j*(numrows+1)+i-1)<<" - ";
      };
      to<<setw(datawidth)<<*(cutoffs+j*(numrows+1)+i);
      to<<setw(freqwidth)<<*(freq+j*(numrows+1)+i);
      to<<colsep;
    };
    for(unsigned int j=leftover;j<n;j++){
      if(i+j){
	to<<setw(datawidth)<<*(cutoffs+j*(numrows+1)+i-1)<<" - ";
      };
      to<<setw(datawidth)<<*(cutoffs+j*numrows+i+leftover);
      to<<setw(freqwidth)<<*(freq+j*numrows+leftover+i);
      to<<colsep;
    };
    to<<"\n";
  };
  for(unsigned int j=0;j<leftover;j++){
    if(j||numrows){
      to<<setw(datawidth)<<*(cutoffs+(j+1)*(numrows+1)-2)<<" - ";
    };      
    to<<setw(datawidth)<<*(cutoffs+(j+1)*(numrows+1)-1);
    to<<setw(freqwidth)<<*(freq+(j+1)*(numrows+1)-1);
    to<<colsep;
  };
  to<<"\n";
};


void Discrete_Grouped_Data::write(ostream &to, unsigned int numcols){
  //numcols is the number of columns in the table.
  unsigned int numrows=n/numcols;
  unsigned int leftover=n%numcols;
  //Data consists of numrows rows with leftover additional entries.
  //We will sort in columns;
  unsigned int datawidth=getcolwidth(n,cutoffs);
  unsigned int freqwidth=getcolwidth(n,freq);
  const char *colsep="   ";//three spaces to separate columns;
  to<<setw(datawidth)<<0<<" - ";//first cutoff set to zero ?????
  //perhaps first cutoff should be a special field for grouped data.
  for(unsigned int i=0;i<numrows;i++){
    for(unsigned int j=0;j<leftover;j++){      
      if(i+j){
	to<<setw(datawidth)<<*(cutoffs+j*(numrows+1)+i-1)<<" - ";
      };
      to<<setw(datawidth)<<*(cutoffs+j*(numrows+1)+i);
      to<<setw(freqwidth)<<*(freq+j*(numrows+1)+i);
      to<<colsep;
    };
    for(unsigned int j=leftover;j<n;j++){
      if(i+j){
	to<<setw(datawidth)<<*(cutoffs+j*(numrows+1)+i-1)<<" - ";
      };
      to<<setw(datawidth)<<*(cutoffs+j*numrows+i+leftover);
      to<<setw(freqwidth)<<*(freq+j*numrows+leftover+i);
      to<<colsep;
    };
    to<<"\n";
  };
  for(unsigned int j=0;j<leftover;j++){
    if(j||numrows){
      to<<setw(datawidth)<<*(cutoffs+(j+1)*(numrows+1)-2)<<" - ";
    };      
    to<<setw(datawidth)<<*(cutoffs+(j+1)*(numrows+1)-1);
    to<<setw(freqwidth)<<*(freq+(j+1)*(numrows+1)-1);
    to<<colsep;
  };
  to<<"\n";
};

//Copying datasets


Continuous_Individual_Data* Continuous_Individual_Data::copy() const {
  long double *newdata=new long double[n];
  for(unsigned int i=0;i<n;i++){
    *(newdata+i)=*(data+i);
  };
  return new Continuous_Individual_Data(n,newdata);
};


Discrete_Individual_Data* Discrete_Individual_Data::copy() const {
  unsigned int *newdata=new unsigned int[n];
  for(unsigned int i=0;i<n;i++){
    *(newdata+i)=*(data+i);
  };
  return new Discrete_Individual_Data(n,newdata);
};



Continuous_Frequency_Data* Continuous_Frequency_Data::copy() const {
  long double *newdata=new long double[n];
  unsigned int *newfreqs=new unsigned int[n];
  for(unsigned int i=0;i<n;i++){
    *(newdata+i)=*(data+i);
    *(newfreqs+i)=*(freq+i);    
  };
  return new Continuous_Frequency_Data(n,newfreqs,newdata);
};

Discrete_Frequency_Data* Discrete_Frequency_Data::copy() const {
  unsigned int *newdata=new unsigned int[n];
  unsigned int *newfreqs=new unsigned int[n];
  for(unsigned int i=0;i<n;i++){
    *(newdata+i)=*(data+i);
    *(newfreqs+i)=*(freq+i);    
  };
  return new Discrete_Frequency_Data(n,newfreqs,newdata);  
};

Continuous_Grouped_Data* Continuous_Grouped_Data::copy() const {
  long double *newdata=new long double[n];
  unsigned int *newfreqs=new unsigned int[n];
  for(unsigned int i=0;i<n;i++){
    *(newdata+i)=*(cutoffs+i);
    *(newfreqs+i)=*(freq+i);    
  };
  return new Continuous_Grouped_Data(n,newfreqs,newdata);
};


Discrete_Grouped_Data* Discrete_Grouped_Data::copy() const {
  unsigned int *newdata=new unsigned int[n];
  unsigned int *newfreqs=new unsigned int[n];
  for(unsigned int i=0;i<n;i++){
    *(newdata+i)=*(cutoffs+i);
    *(newfreqs+i)=*(freq+i);    
  };
  return new Discrete_Grouped_Data(n,newfreqs,newdata);
};


