#include <iostream>
#include "specific_distributions.h"
#include <cstdlib>
#include <Rcpp.h>

using namespace std;

void NormalDistribution::print() const{
  Rcpp::Rcout<<"normal distribution.\n\n";
  Rcpp::Rcout<<"mu="<<mu;
  if(fixedmu){
    Rcpp::Rcout<<"\t fixed";
  };
  Rcpp::Rcout<<"\nsigma="<<sigma;
  if(fixedsig){
    Rcpp::Rcout<<"\t fixed";
  };
  Rcpp::Rcout<<"\nParameter of interest: ";
  switch(mainpar){
  case 0:
    Rcpp::Rcout<<"mu\n";
    return;
  case 1:
    Rcpp::Rcout<<"sigma\n";
    return;
  case 2:
    Rcpp::Rcout<<"VaR at "<<VaR_at<<" level.\n";
    return;
  default:
    Rcpp::Rcout<<"No main parameter selected. This should not happen.\n";
  };
};


void ParetoType1Distribution::print() const{
  Rcpp::Rcout<<"Pareto distribution.\n\n";
  Rcpp::Rcout<<"alpha="<<alpha;
  if(fixedalpha){
    Rcpp::Rcout<<"\t fixed";
  };
  Rcpp::Rcout<<"\nmin="<<min;
  if(fixedmin){
    Rcpp::Rcout<<"\t fixed";
  };
  Rcpp::Rcout<<"\nParameter of interest: ";
  switch(mainpar){
  case 0:
    Rcpp::Rcout<<"alpha\n";
    return;
  case 1:
    Rcpp::Rcout<<"min\n";
    return;
  case 2:
    Rcpp::Rcout<<"VaR at "<<VaR_at<<" level.\n";
    return;
  default:
    Rcpp::Rcout<<"No main parameter selected. This should not happen.\n";
  };
};


void PoissonDistribution::print() const{
  Rcpp::Rcout<<"Poisson distribution.\n\n";
  Rcpp::Rcout<<"lambda="<<lambda<<"\n";
};



void LogNormalPoissonDistribution::print() const{
  Rcpp::Rcout<<"Log-Normal-Poisson distribution.\n\n";
  Rcpp::Rcout<<"mu="<<mu;
  if(fixedmu){
    Rcpp::Rcout<<"\t fixed";
  };
  Rcpp::Rcout<<"\nsigma="<<sigma;
  if(fixedsig){
    Rcpp::Rcout<<"\t fixed";
  };
  Rcpp::Rcout<<"\nParameter of interest: ";
  switch(mainpar){
  case 0:
    Rcpp::Rcout<<"mu\n";
    return;
  case 1:
    Rcpp::Rcout<<"sigma\n";
    return;
  case 2:
    Rcpp::Rcout<<"VaR at "<<VaR_at<<" level.\n";
    return;
  default:
    Rcpp::Rcout<<"No main parameter selected. This should not happen.\n";
  };
};


void TDistribution::print() const{
  Rcpp::Rcout<<"Student\'s t distribution.\n\n";
  Rcpp::Rcout<<degreesoffreedom<<" degrees of freedom";
  if(fixeddegreesoffreedom){
    Rcpp::Rcout<<"\t fixed";
  };
  Rcpp::Rcout<<"\nmu="<<mu;
  if(fixedmu){
    Rcpp::Rcout<<"\t fixed";
  };
  Rcpp::Rcout<<"\nsigma="<<sigma;
  if(fixedsigma){
    Rcpp::Rcout<<"\t fixed";
  };
  Rcpp::Rcout<<"\nParameter of interest: ";
  switch(mainpar){
  case 0:
    Rcpp::Rcout<<"mu\n";
    return;
  case 1:
    Rcpp::Rcout<<"sigma\n";
    return;
  case 2:
    Rcpp::Rcout<<"degrees of freedom\n";
    return;
  case 3:
    Rcpp::Rcout<<"VaR at "<<VaR_at<<" level.\n";
    return;
  default:
    Rcpp::Rcout<<"No main parameter selected. This should not happen.\n";
  };
};



//Functions for returning a summary of the distribution to main program.


distSummary::distSummary(const char *nm,const char *mp,unsigned int np){
  name=new char[strlen(nm)+1];
  strcpy(name,nm);
  mainpar=new char[strlen(mp)+1];
  strcpy(mainpar,mp);
  numpar=np;
  paramnames=new const char*[np];
  paramvals=new long double[np];
};



distSummary NormalDistribution::Rsummary() const{
  unsigned int fxpars=this->numpars()-this->numestpars();
  const char *mp;
  switch(mainpar){
  case 0:
    mp="mu";
    break;
  case 1:
    mp="sigma";
    break;
  case 2:
    char *temp=new char[30];
    sprintf(temp+7,"VaR at %f level",(double)VaR_at);
    mp=temp;
  }
  distSummary answer("normal",mp,fxpars);
  unsigned int i=0;
  if(fixedmu){
    answer.paramnames[i]="mu";
    answer.paramvals[i++]=mu;
  };
  if(fixedsig){
    answer.paramnames[i]="sigma";
    answer.paramvals[i++]=sigma;
  };
  return(answer);
};


distSummary ParetoType1Distribution::Rsummary() const{
  unsigned int fxpars=this->numpars()-this->numestpars();
  const char *mp;
  switch(mainpar){
  case 0:
    mp="alpha";
    break;
  case 1:
    mp="min";
    break;
  case 2:
    char *temp=new char[30];
    sprintf(temp+7,"VaR at %f level",(double)VaR_at);
    mp=temp;
  }
  distSummary answer("normal",mp,fxpars);
  unsigned int i=0;
  if(fixedalpha){
    answer.paramnames[i]="alpha";
    answer.paramvals[i++]=alpha;
  };
  if(fixedmin){
    answer.paramnames[i]="min";
    answer.paramvals[i++]=min;
  };
  return(answer);  
};


distSummary PoissonDistribution::Rsummary() const{
  distSummary answer("normal","lambda",0); //Only one parameter for Poisson
  return(answer);  
};



distSummary LogNormalPoissonDistribution::Rsummary() const{
  unsigned int fxpars=numpars-this->numestpars();
  const char *mp;
  switch(mainpar){
  case 0:
    mp="mu";
    break;
  case 1:
    mp="sigma";
    break;
  case 2:
    char *temp=new char[30];
    sprintf(temp+7,"VaR at %f level",(double)VaR_at);
    mp=temp;
  }
  distSummary answer("normal",mp,fxpars);
  unsigned int i=0;
  if(fixedmu){
    answer.paramnames[i]="mu";
    answer.paramvals[i++]=mu;
  };
  if(fixedsig){
    answer.paramnames[i]="sigma";
    answer.paramvals[i++]=sigma;
  };
  return(answer);
};


distSummary TDistribution::Rsummary() const{
  unsigned int fxpars=this->numpars()-this->numestpars();
  const char *mp;
  switch(mainpar){
  case 0:
    mp="mu";
    break;
  case 1:
    mp="sigma";
    break;
  case 2:
    mp="degrees of freedom";
    break;
  case 3:
    char *temp=new char[30];
    sprintf(temp+7,"VaR at %f level",(double)VaR_at);
    mp=temp;
  }
  distSummary answer("t",mp,fxpars);
  unsigned int i=0;
  if(fixedmu){
    answer.paramnames[i]="mu";
    answer.paramvals[i++]=mu;
  };
  if(fixedsigma){
    answer.paramnames[i]="sigma";
    answer.paramvals[i++]=sigma;
  };
  if(fixeddegreesoffreedom){
    answer.paramnames[i]="degrees of freedom";
    answer.paramvals[i++]=degreesoffreedom;
  };
  return(answer);
};


