#ifndef LOGNORMALPOISSON_H
#define LOGNORMALPOISSON_H

#include "data.h"
#include "distributions.h"

#define SUPERCONSTRUCT Distribution("LogNormalPoisson",datatype(DISCRETE_DATA,LIST|FREQUENCY_TABLE|GROUPED),2,3)

class LogNormalPoissonDistribution: public Distribution{
protected:
  static const char* const valparnams[];  
  long double mu;
  long double sigma;
  char fixedmu;
  char fixedsig;
  unsigned int mainpar;
  long double VaR_at;
public:
 LogNormalPoissonDistribution(const LogNormalPoissonDistribution &orig):SUPERCONSTRUCT,mu(orig.mu),sigma(orig.sigma),fixedmu(orig.fixedmu),fixedsig(orig.fixedsig),mainpar(orig.mainpar),VaR_at(orig.VaR_at){};
 LogNormalPoissonDistribution(long double m,long double s):SUPERCONSTRUCT,mu(m),sigma(s),fixedmu(0),fixedsig(0),mainpar(0),VaR_at(0){};
 LogNormalPoissonDistribution():SUPERCONSTRUCT,mu(0),sigma(1),fixedmu(0),fixedsig(0),mainpar(0),VaR_at(0){};
  ~LogNormalPoissonDistribution(){};
  virtual const char *const*validparnames() const{return valparnams;};
  virtual void setfixed(unsigned int i,long double val);
  virtual void unfix(unsigned int i);
  virtual void setmain(unsigned int i,long double at);
  virtual long double parval() const;
  virtual long double parval(unsigned int i,long double at) const;
  virtual unsigned int numestpars() const;
  virtual LogNormalPoissonDistribution* copy() const;
  virtual void estimate(const Data &dat);//for grouped data
  virtual Data * simulate(unsigned int n) const;  
  
  virtual long double *dist(const Data &dat) const;
  virtual long double *survive(const Data &dat) const;
  virtual long double *likelihood(const Data &dat) const;
  virtual long double quant(long double p) const;

  virtual void print() const;

  //functions used in estimating parameters by ML.
 private:
  void evalprobstodirect(unsigned int x,long double *answers) const;
  void evalprobsandderivstodirect(unsigned int x,long double *answers) const;
  void evalprobsandderivstodirectsparse(unsigned int x,const unsigned int *values,long double *answers) const;
};

#undef SUPERCONSTRUCT
#endif
