#ifndef NORMALDISTRIBUTION_H
#define NORMALDISTRIBUTION_H


#include "data.h"
#include "distributions.h"


#define SUPERCONSTRUCT Distribution("normal",datatype(CONTINUOUS_DATA,LIST|FREQUENCY_TABLE|GROUPED),2,3)


//Normal Distribution

const char *const normalvalidparnames[]={"mu","sigma","VaR@"};

class NormalDistribution: public Distribution{
protected:
  long double mu;
  long double sigma;
  char fixedmu;
  char fixedsig;
  unsigned int mainpar;
  long double VaR_at;
public:
 NormalDistribution(const NormalDistribution &orig):SUPERCONSTRUCT,mu(orig.mu),sigma(orig.sigma),fixedmu(orig.fixedmu),fixedsig(orig.fixedsig),mainpar(orig.mainpar),VaR_at(orig.VaR_at){};
  NormalDistribution(long double m,long double s):SUPERCONSTRUCT,mu(m),sigma(s),fixedmu(0),fixedsig(0),mainpar(0),VaR_at(0){};
  NormalDistribution():SUPERCONSTRUCT,mu(0),sigma(1),fixedmu(0),fixedsig(0),mainpar(0),VaR_at(0){};
  virtual ~NormalDistribution(){};
  virtual unsigned int numpars() const{return 2;};
  virtual unsigned int numvalidpars() const{return 3;};
  virtual const char *const*validparnames() const{return normalvalidparnames;};
  virtual long double* dist(const Data&dat) const;
  virtual long double* survive(const Data&dat) const;
  virtual long double *likelihood(const Data &dat) const;
  virtual long double quant(long double p) const;
  virtual Data* simulate(unsigned int n) const;  
  virtual long double parval() const;
  virtual long double parval(unsigned int i,long double at) const;
  virtual void setfixed(unsigned int i);
  virtual void setfixed(unsigned int i,long double val);
  virtual void unfix(unsigned int i);
  virtual void setmain(unsigned int i,long double at);
  //  virtual long double get_critval(long double size,AdequacyTest& test)  const;
  //  virtual long double adjustADstat(unsigned int samplesize) const;
  virtual unsigned int numestpars() const;
  virtual NormalDistribution* copy() const;
  virtual void estimate(const Data &dat);
  virtual void print() const;
};

#undef SUPERCONSTRUCT
#endif
