#include "PoissonDistribution.h"
#include <cmath>




const char* const PoissonDistribution::valparnams[]={"lambda"};


long double PoissonDistribution::parval() const{
  return lambda;
};

long double PoissonDistribution::parval(unsigned int i,long double at) const{
  return lambda;
};

PoissonDistribution* PoissonDistribution::copy() const{
  return new PoissonDistribution(lambda);
};

void PoissonDistribution::estimate(const Data &dat){
  if(dat.get_type().storage&(LIST|FREQUENCY_TABLE)){
    const unsigned int *data=(unsigned int *)dat.getvalues();
    const unsigned int *frqs=dat.getfreqs();
    unsigned int sum=0;
    unsigned int tot=0;
    if(frqs){
      for(unsigned int i=0;i<dat.n;i++){
	sum+=*(data+i)*(*(frqs+i));
	tot+=*(frqs+i);
      };
    }else{
      tot=dat.n;
      for(unsigned int i=0;i<dat.n;i++){
	sum+=*(data+i);
      };
    };
    lambda=((long double)sum)/tot;
  }else{
    throw Distribution_error("Poisson Distribution parameter estimation not currently available for data stored in this format.\n");
  };
};


void PoissonDistribution::setfixed(unsigned int i,long double val){
  switch(i){
  case 0:
    lambda=val;
    break;
  default:
    throw Distribution_error("Unknown parameter for Poisson distribution.\n");
  };
};



long double *PoissonDistribution::dist(const Data &dat) const{
  //No checking on suitability of dat. Should be checked by program.
  unsigned int *values=(unsigned int *)dat.getvalues();
  long double exl=exp(lambda);
  long double *answers=new long double[dat.n];
  for(unsigned int i=0;i<dat.n;i++){    
    long double total=1;
    long double current=1;
    for(unsigned int j=1;j<=*(values+i);j++){
      current*=lambda/j;
      total+=current;
    };
    *(answers+i)=total/exl;
  };
  return answers;
};

long double *PoissonDistribution::likelihood(const Data &dat) const{
  //No checking on suitability of dat. Should be checked by program.
  //probably more efficient to save calculations.
  unsigned int *values=(unsigned int *)dat.getvalues();
  long double exl=exp(-lambda);
  long double *answers=new long double[dat.n];
  for(unsigned int i=0;i<dat.n;i++){    
    long double current=exl;
    for(unsigned int j=1;j<=*(values+i);j++){
      current*=lambda/j;
    };
    *(answers+i)=current;
  };
  return answers;
};


long double *PoissonDistribution::survive(const Data& dat) const{
  long double *answers=this->dist(dat);
  for(unsigned int i=0;i<dat.n;i++){
    *(answers+i)=1-*(answers+i);
  };
  return answers;
};


long double PoissonDistribution::quant(long double p) const{
  long double target=p*exp(lambda);
  long double total=1;
  unsigned int i=0;
  long double current=1;
  for(;total<target;i++){
    current*=lambda/(i+1);
    total+=current;
  };
  return i;
};










