//Simulation stuff.

#include "data.h"
#include "datatypes.h"
#include "distributions.h"
#include "specific_distributions.h"
#include <boost/math/distributions/normal.hpp>



using namespace std;

extern boost::math::normal stdnorm;


inline long double pnorm(long double x){
  return erfc((-x)/sqrt(2))/2;
}

inline long double qnorm(long double p){
  return quantile(stdnorm,p);
}

inline long double rnorm(){
  return quantile(stdnorm,((long double)rand())/RAND_MAX);
}



Data* NormalDistribution::simulate(unsigned int n) const{
  long double *values=new long double[n];
  for(unsigned int i=0;i<n;i++){
    *(values+i)=rnorm()*sigma+mu;
  };
  Data* answer=new Continuous_Individual_Data(n,values);
  answer->sort();
  return answer;
};  

Data* ParetoType1Distribution::simulate(unsigned int n) const{
  long double *values=new long double[n];
  long double ninval=-1/alpha;
  for(unsigned int i=0;i<n;i++){
    long double r=((long double)rand())/RAND_MAX;
    *(values+i)=min*pow(r,ninval);
  };
  Data* answer=new Continuous_Individual_Data(n,values);
  answer->sort();
  return answer;
};  



Data* PoissonDistribution::simulate(unsigned int n) const{
  long double expl=exp(lambda);
  unsigned int *values=new unsigned int[n];
  for(unsigned int i=0;i<n;i++){
    long double runif=((long double)rand())/RAND_MAX;
    long double target=runif*expl;
    unsigned int j=0;
    long double total=1;
    long double current=1;
    for(;total<target;j++){
      current*=lambda/(j+1);
      total+=current;
    };
    *(values+i)=j;
  };
  Data *answer=new Discrete_Individual_Data(n,values);
  answer->sort();
  return answer;
};  


Data* LogNormalPoissonDistribution::simulate(unsigned int n) const{
  //Might get better numerical stability for tails
  //by counting backwards.
  //Need to figure out how to implement this.
  unsigned int *values=new unsigned int[n];
  for(unsigned int i=0;i<n;i++){
    long double lambda=exp(rnorm()*sigma+mu);
    //    std::cout<<lambda<<"\t";
    long double runif=((long double)rand())/RAND_MAX;
    long double target=runif*exp(lambda);;
    long double total=1;
    unsigned int j=0;
    long double current=1;
    for(;total<target;j++){
      current*=lambda/(j+1);
      total+=current;
    };
    //    std::cout<<j<<"\n";
    *(values+i)=j;
  };
  Data *answer=new Discrete_Individual_Data(n,values);
  answer->sort();
  return answer;
};  

//#include <iostream>

Data* TDistribution::simulate(unsigned int n) const{
  //Uses a polar method to simulate.
  //Reference:

  //Bailey, R. W. (1994). "Polar Generation of Random Variates with
  //the t-Distribution". Mathematics of Computation. 62 (206):
  //779–781. doi:10.2307/2153537
  long double *values=new long double[n];
  for(unsigned int i=0;i<n;i++){
    long double U, V, W=2;
    while(W>1){
      U=2*((long double)rand()-RAND_MAX/2)/RAND_MAX;
      V=2*((long double)rand()-RAND_MAX/2)/RAND_MAX;
      W=U*U+V*V;
    };
    //U and V are uniformly distributed over the unit dist.
    //    long double costheta=U/sqrt(W);
    long double t=sqrt(degreesoffreedom*(pow(W,-2/degreesoffreedom)-1)*U*U/W);
    if(U<0){
      t=-t;
    };
    //    cout<<"U="<<U<<"\tV="<<V<<"\tW="<<W<<"\tt="<<t<<"\n";
    *(values+i)=t*sigma+mu;
  };
  Data *answer=new Continuous_Individual_Data(n,values);
  answer->sort();
  return answer;
};  

  
  
