#include "datatypes.h"
//#include "distributions.h"

#include <iostream>
#include <cstdlib>
#include <cstring>
#include <cmath>

using namespace std;

class NP_mean: public NPestimator{
public:  
  NP_mean():NPestimator(datatype(DISCRETE_DATA|CONTINUOUS_DATA,LIST|FREQUENCY_TABLE)){};
  datatype type; //What types of data can this test be applied to
  virtual long double evaluate(const Data& dat);
};


class NP_sd: public NPestimator{
public:  
  NP_sd():NPestimator(datatype(DISCRETE_DATA|CONTINUOUS_DATA,LIST|FREQUENCY_TABLE)){};
  virtual long double evaluate(const Data& dat);
};


class NP_VaR: public NPestimator{
  long double VaR_at;
public:
  NP_VaR():NPestimator(datatype(DISCRETE_DATA|CONTINUOUS_DATA,LIST|FREQUENCY_TABLE)),VaR_at(0.05){};
  NP_VaR(long double at): NPestimator(datatype(DISCRETE_DATA|CONTINUOUS_DATA,LIST|FREQUENCY_TABLE)),VaR_at(at){};  
  virtual long double evaluate(const Data& dat);
};


class NP_Gini: public NPestimator{
public:
  NP_Gini():NPestimator(datatype(DISCRETE_DATA|CONTINUOUS_DATA,LIST|FREQUENCY_TABLE)){};  
  virtual long double evaluate(const Data& dat);
};


long double NP_mean::evaluate(const Data& dat){
  if(dat.get_type().type_of_data&DISCRETE_DATA){
    const unsigned int *values=(const unsigned int *)dat.getvalues();
    const unsigned int *freqs=dat.getfreqs();
    if(freqs!=NULL){
      long double sum=0;
      unsigned int tot=0;
      for(unsigned int i=0;i<dat.n;i++){
	sum+=*(values+i)*(*(freqs+i));
	tot+=*(freqs+i);
      };
      return sum/tot;            
    }else{
      long double sum=0;
      for(unsigned int i=0;i<dat.n;i++){
	sum+=*(values+i);
      };
      return sum/dat.n;      
    };
  }else{
    const long double *values=(const long double *)dat.getvalues();
    const unsigned int *freqs=dat.getfreqs();
    if(freqs){
      long double sum=0;
      unsigned int tot=0;
      for(unsigned int i=0;i<dat.n;i++){
	sum+=*(values+i)*(*(freqs+i));
	tot+=*(freqs+i);
      };
      return sum/tot;            
    }else{
      long double sum=0;
      for(unsigned int i=0;i<dat.n;i++){
	sum+=*(values+i);
      };
      return sum/dat.n;      
    };
  };  
};



long double NP_sd::evaluate(const Data& dat){
  const unsigned int *freqs=dat.getfreqs();
  long double sum=0;
  long double ssq=0;
  if(freqs!=NULL){
    unsigned int tot=0;
    if(dat.get_type().type_of_data&DISCRETE_DATA){
      const unsigned int *values=(const unsigned int *)dat.getvalues();
      for(unsigned int i=0;i<dat.n;i++){
	sum+=*(values+i)*(*(freqs+i));
	ssq+=*(values+i)*(*(freqs+i))*(*(values+i));
	tot+=*(freqs+i);
      };
    }else{
      const long double *values=(const long double *)dat.getvalues();
      for(unsigned int i=0;i<dat.n;i++){
	sum+=*(values+i)*(*(freqs+i));
	ssq+=*(values+i)*(*(freqs+i))*(*(values+i));
	tot+=*(freqs+i);
      };
    };
    return sqrt(ssq/tot-(sum/tot)*(sum/tot));            
  }else{
    if(dat.get_type().type_of_data&DISCRETE_DATA){
      const unsigned int *values=(const unsigned int *)dat.getvalues();
      for(unsigned int i=0;i<dat.n;i++){
	sum+=*(values+i);
	ssq+=*(values+i)*(*(values+i));
      };
    }else{
      const long double *values=(const long double *)dat.getvalues();
      for(unsigned int i=0;i<dat.n;i++){
	sum+=*(values+i);
	ssq+=*(values+i)*(*(values+i));
      };
    };
    return sqrt(ssq/dat.n-(sum/dat.n)*(sum/dat.n));             
  };
};


long double NP_VaR::evaluate(const Data& dat){
  const unsigned int *samp=dat.getfreqs();
  unsigned int i=dat.n*VaR_at;
  if(samp){
    unsigned int tot=0;
    for(unsigned int j=0;j<dat.n;j++){
      tot+=*(samp+j);
    };  
    unsigned int pos=tot*VaR_at;
    unsigned int cumtot=0;
    for(i=0;cumtot<pos;cumtot+=*(samp+i++));
    i--;
  };
  if(dat.get_type().type_of_data&DISCRETE_DATA){
    return *((unsigned int *)dat.getvalues()+i);
  }else{
    return *((long double *)dat.getvalues()+i);
  };
};


long double NP_Gini::evaluate(const Data& dat){
  unsigned int cumtot=0;
  long double U2=0;
  long double U1=0;
  const unsigned int *samp=dat.getfreqs();
  if(samp){
    if(dat.get_type().type_of_data&DISCRETE_DATA){
      unsigned int *data=(unsigned int *)dat.getvalues();
      for(unsigned int i=0;i<dat.n;cumtot+=*(samp+i++)){
	U1+=*(samp+i)*(*(data+i));
	U2+=((*(samp+i))*cumtot+(*(samp+i)*(*(samp+i)-1))/2)*(*(data+i));
      };
    }else{
      long double *data=(long double *)dat.getvalues();
      for(unsigned int i=0;i<dat.n;cumtot+=*(samp+i++)){
	U1+=*(samp+i)*(*(data+i));
	U2+=((*(samp+i))*cumtot+(*(samp+i)*(*(samp+i)-1))/2)*(*(data+i));
      };
    };
  }else{
    cumtot=dat.n;
    if(dat.get_type().type_of_data&DISCRETE_DATA){
      unsigned int *data=(unsigned int *)dat.getvalues();
      for(unsigned int i=0;i<dat.n;i++){
	U1+=(*(data+i));
	U2+=i*(*(data+i));
      };
    }else{
      long double *data=(long double *)dat.getvalues();
      for(unsigned int i=0;i<dat.n;i++){
	U1+=*(data+i);
	U2+=i*(*(data+i));
      };
    };
  };
  return 2*U2/U1/(cumtot-1)-1;
};


NPestimator *get_NP_estimator(const char *eq,const char *at){
  if(eq==NULL){
    cout<<"NP method unspecified. Ignoring NP setting.\n";
    return NULL;
  }else{	
    if(!strcmp(eq,"mean")){
      return new NP_mean();
    }else if(!strcmp(eq,"sd")){
      return new NP_sd();
    }else if(!strcmp(eq,"VaR")){
      long double atval=0.05;
      if(at!=NULL){
	atval=atof(at);
      };
      return new NP_VaR(atval);
    }else if(!strcmp(eq,"Gini")){
      return new NP_Gini();
    }else{
      cout<<"Nonparametric estimator \""<<eq<<"\" not currently supported. Ignoring option.\n";
      return NULL;
    };
  };
};

//*/


//Test code.
//Delete before final version.


/*
extern NPestimator *NPest;
void read_option(const char* const optname);

int main(int argc,char *argv[]){
  read_option(argv[1]);
  unsigned int test[]={3,5,3,5,3};

  long double testCont[]={3.4,5.1,3.5,4.8,3.9,4.7};

  unsigned int freqs[]={1,8,3,4,5,6};
  Discrete_Individual_Data dat(5,test);
  Continuous_Individual_Data datcon(6,testCont);
  Discrete_Frequency_Data datfr(5,freqs,test);
  Continuous_Frequency_Data datconfr(6,freqs,testCont);
  Discrete_Grouped_Data datgr(4,freqs,test);
  Continuous_Grouped_Data datcongr(6,freqs,testCont);


  if(NPest->compatible(dat)){
    long double me=NPest->evaluate(dat);
    cout<<me<<"\n";
  }else{
    cout<<"NP_estimator not available for this type of data.\n";
  };
  if(NPest->compatible(datcon)){
    long double me=NPest->evaluate(datcon);
    cout<<me<<"\n";
    datcon.subsample(1323523);
    me=NPest->evaluate(datcon);
    cout<<me<<"\n";    
  }else{
    cout<<"NP_estimator not available for this type of data.\n";
  };
  if(NPest->compatible(datfr)){
    long double me=NPest->evaluate(datfr);
    cout<<me<<"\n";
  }else{
    cout<<"NP_estimator not available for this type of data.\n";
  };
  if(NPest->compatible(datconfr)){
    long double me=NPest->evaluate(datconfr);
    cout<<me<<"\n";
    datconfr.subsample(132312);
    me=NPest->evaluate(datconfr);
    cout<<me<<"\n";    
  }else{
    cout<<"NP_estimator not available for this type of data.\n";
  };
  if(NPest->compatible(datgr)){
    long double me=NPest->evaluate(datgr);
    cout<<me<<"\n";
    datgr.subsample(132312);
    me=NPest->evaluate(datgr);
    cout<<me<<"\n";    
  }else{
    cout<<"NP_estimator not available for this type of data.\n";
  };
  if(NPest->compatible(datcongr)){
    long double me=NPest->evaluate(datcongr);
    cout<<me<<"\n";
    datcongr.subsample(132312);
    me=NPest->evaluate(datcongr);
    cout<<me<<"\n";    
  }else{
    cout<<"NP_estimator not available for this type of data.\n";
  };
};

*/
