//processing commandline options.
#include "Options.h"
#include <cstring>
#include <cstdio>
#include <cstdlib>
#include <exception>

using namespace std;

class OptionList;

class OptionListElement{
  //linked list of option choices
private://No public members.
  Option *op;
  OptionListElement *next;
  Option *find(const char *const name) const ;
  OptionListElement(Option *o,OptionListElement *ne):op(o),next(ne){};
  OptionListElement(Option *o):op(o),next(NULL){};
  friend class OptionList;
  friend OptionListElement *setup_options();
};

Option *OptionListElement::find(const char *const name) const {
  if(!op->checkname(name)){
    return op;
  }else if(next){
    return next->find(name);    
  }else{
    return NULL;
  };
};


OptionListElement *setup_options();

const void *lookup_option(const char *name);

class OptionList{
private:
  //unique containerclass
  //No cleanup on exit.
  static OptionListElement* start;
  //Allow read_option function to set options.
  //Other functions can only read options.
  friend void read_option(const char* const optname);
  friend const void *lookup_option(const char *name);
  Option *find(const char *name){return start->find(name);};
  OptionList(){if(!start){start=setup_options();};};
public:
};

OptionListElement* OptionList::start=NULL;

const void *lookup_option(const char *name){
  static OptionList options;
  const Option *a=options.find(name);
  if(a){
    return a->getoptionval();
  }else{
    return NULL;
  };
};



void read_option(const char* const optname){
  char *at=NULL;
  char *at2=NULL;
  char *eq=NULL;
  char* name=new char[strlen(optname)+1];
  strcpy(name,optname);
  unsigned int len=strlen(name);
  for(unsigned int i=0;i<len;i++){
    if(*(name+i)=='@'){
      *(name+i)='\0';
      if(at==NULL){
	at=(name+i+1);
      }else{
	at2=(name+i+1);
      };
    }else if(*(name+i)=='='){
      *(name+i)='\0';
      eq=(name+i+1);	
    };
  };
  static OptionList options;
  Option *opt=options.find(name);
  if(opt){
    delete[] name;
    opt->setoption(eq,at,at2);
  }else{
    char *error=new char[strlen(name)+56];
    sprintf(error,"Option \"%s\" not currently supported. Ignoring option.\n",name);      
    delete[] name;
    throw new option_formatting_error(error);
  };
};






OptionListElement *setup_options(){
  OptionListElement *next=new OptionListElement(new SetSeedOption("seed"));
  next=new OptionListElement(new NPestimatorOption("NP"),next);
  next=new OptionListElement(new PairNPestimatorOption("CI"),next);
  next=new OptionListElement(new IntOption("jackknife"),next);
  next=new OptionListElement(new IntOption("frequency"),next);
  next=new OptionListElement(new IntOption("grouped"),next);
  next=new OptionListElement(new OutputFileStreamOption("plotpower"),next);
  next=new OptionListElement(new OutputFileStreamOption("save"),next);
  next=new OptionListElement(new OutputFileStreamOption("log"),next);
  return next;
};

/*

    }else if(!strcmp(name,"CI")){
      if(eq==NULL){
	cout<<"CI method unspecified. Ignoring CI setting.\n";
      }else{	
	if(*eq!='['){
	  cout<<"first method: \""<<eq<<"\"\n";
	  cout<<"CI format \"CI=[LOWER,UPPER]\".\n";
	}else{
	  eq++;
	  char* eq2=eq;	
	  for(;*eq2!='\0'&&*eq2!=',';eq2++);
	  if(*eq2!=','){
	    if(++eq2==at){
	      for(;*eq2!='\0'&&*eq2!=',';eq2++);
	      if(*eq2!=','){
		cout<<"Arguments:\t\""<<eq<<"\"\n\t\t\""<<at<<"\"\n\t\t\""<<at2<<"\"\n";
		cout<<"CI format \"CI=[LOWER,UPPER]\".\n";
	      };
	    }else{
		cout<<"Arguments:\t\""<<eq<<"\"\n\t\t\""<<at<<"\"\n\t\t\""<<eq2<<"\"\n\t\t\""<<at2<<"\"\n";	      
                cout<<"CI format \"CI=[LOWER,UPPER]\".\n";
	    };
	  };
	  *(eq2++)='\0';
	  CILest=get_NP_estimator(eq,at);
	  CIUest=get_NP_estimator(eq2,at);
	};
      };
*/
