#include <cstdlib>
#include <exception>

class Option{
protected:
  const char *const name;
public:
  int checkname(const char* nm) const;
  Option(const char* nm):name(nm){};
  virtual void setoption(const char *val,const char *at,const char *at2)=0;
  virtual const void* getoptionval() const=0;
  //calling function should know what type of value is returned.
  virtual void apply(){}; //for deferred application of option.
};

class TrueFalseOption: public Option{
protected:
  unsigned int value;
public:
  TrueFalseOption(const char* nm):Option(nm){};
  virtual void setoption(const char *val,const char *at,const char *at2);
  virtual const void* getoptionval() const {return &value;};
  virtual void apply(){};
};


class IntOption: public Option{
protected:
  unsigned int value;
public:
  IntOption(const char* nm):Option(nm){};
  virtual void setoption(const char *val,const char *at,const char *at2);
  virtual const void* getoptionval() const {return &value;};
  virtual void apply(){};
};


class SetSeedOption: public IntOption{
public:
  SetSeedOption(const char* nm):IntOption(nm){};
  void apply();
};

class PointerOption: public Option{
protected:
  void *data;//create some structure and save a pointer to it.
  const char *value;
  const char *atval;
  const char *atval2;
public:
 PointerOption(const char* nm):Option(nm),data(NULL),value(NULL),atval(NULL),atval2(NULL){};
  void setoption(const char *val,const char *at,const char *at2);
  const void* getoptionval() const {return &value;};
  virtual void apply()=0;
};

class NPestimatorOption:public PointerOption{
public:
 NPestimatorOption(const char *nm):PointerOption(nm){};
  void apply();
};

class PairNPestimatorOption:public PointerOption{
  //Special formatting, may need to do something different
public:
 PairNPestimatorOption(const char *nm):PointerOption(nm){};
  void apply();
};

class OutputFileStreamOption:public PointerOption{
public:
 OutputFileStreamOption(const char *nm):PointerOption(nm){};
  void apply();
};



class option_formatting_error: public std::exception{
  static const char*const header;
  char * message;
public:
  option_formatting_error(const char *const mess);
  option_formatting_error(const char *const mess,const char*values,const char *end);
  virtual const char* what() const throw();
};

