//processing commandline options.
#include "Options.h"
#include <cstring>
#include <cstdio>
#include <cstdlib>
#include <exception>

using namespace std;

class OptionList;

class OptionListElement{
  //linked list of option choices
private://No public members.
  Option *op;
  OptionListElement *next;
  Option *find(const char *const name);//almost const except for lazy evaluation.
  unsigned int process(const char *name,const char *val);
  OptionListElement(Option *o,OptionListElement *ne):op(o),next(ne){};
  OptionListElement(Option *o):op(o),next(NULL){};
  friend class OptionList;
  friend OptionListElement *setup_options();
};

Option *OptionListElement::find(const char *const name){
  if(!op->checkname(name)){
    return op;
  }else if(next){
    return next->find(name);    
  }else{
    return NULL;
  };
};

unsigned int OptionListElement::process(const char *name,const char *val){
  unsigned int pr=op->process(name,val);
  if(pr){
    return pr;
  }else if(next){
    return next->process(name,val);
  }else{
    return 0;
  };
};



OptionListElement *setup_options();

const void *lookup_option(const char *name);

class OptionList{
private:
  //unique containerclass
  //No cleanup on exit.
  static OptionListElement* start;
  //Allow read_option function to set options.
  //Other functions can only read options.
  friend void read_option(const char* const optname);
  friend const void *lookup_option(const char *name);
  friend void close_option(const char *name);
  Option *find(const char *name){return start->find(name);};
  unsigned int process(const char *name,const char *val){return start->process(name,val);};
  OptionList(){if(!start){start=setup_options();};};
public:
};

OptionListElement* OptionList::start=NULL;

void close_option(const char *name){
  static OptionList options;
  Option *a=options.find(name);
  if(a){
    a->close();
  };  
};


const void *lookup_option(const char *name){
  static OptionList options;
  Option *a=options.find(name);
  if(a){
    return a->getoptionval();
  }else{
    return NULL;
  };
};




void read_option(const char* const optname){
  char *eq=NULL;
  char* name=new char[strlen(optname)+1];
  strcpy(name,optname);
  unsigned int len=strlen(name);
  for(unsigned int i=0;i<len;i++){
    if(*(name+i)=='='&&eq==NULL){
      *(name+i)='\0';
      eq=(name+i+1);	
    };
  };
  static OptionList options;
  Option *opt=options.find(name);
  if(opt){
    opt->setoption(eq);
    //Usually directly save name to the option. Could try copying it
    //around to avoid memory leaks.
  }else{
    unsigned int pr=options.process(name,eq);
    //see if any currently assigned option understands this option.
    if(!pr){
      char *error=new char[strlen(name)+56];
      sprintf(error,"Option \"%s\" not currently supported. Ignoring option.\n",name);      
      delete[] name;
      throw option_formatting_error(error);
    };
  };
};





const char *const modes[]={"analysis","simulation","test","discriminant_analysis","discriminant_simulation",NULL};
const char *const datatypemodes[]={"individual","frequency","grouped",NULL};
extern const char *const distortionparameters[];

OptionListElement *setup_options(){
  OptionListElement *next=new OptionListElement(new SetSeedOption("seed"));
  next=new OptionListElement(new NPestimatorOption("NP"),next);
  next=new OptionListElement(new PairNPestimatorOption("CI"),next);
  next=new OptionListElement(new IntOption("save_num_cols"),next);
  next=new OptionListElement(new TrueFalseOption("jackknife"),next);
  next=new OptionListElement(new TrueFalseOption("estimateparams"),next);
  next=new OptionListElement(new IntOption("simsize"),next);
  next=new OptionListElement(new IntOption("simsize2"),next);
  next=new OptionListElement(new ModeOption("mode",modes),next);
  next=new OptionListElement(new ModeOption("datatype",datatypemodes),next);
  next=new OptionListElement(new ModeOption("datatype2",datatypemodes),next);
  next=new OptionListElement(new OutputFileStreamOption("plotpower"),next);
  next=new OptionListElement(new OutputFileStreamOption("save"),next);
  next=new OptionListElement(new OutputFileStreamOption("log"),next);
  next=new OptionListElement(new InputFileStreamOption("file"),next);
  next=new OptionListElement(new InputFileStreamOption("file2"),next);
  next=new OptionListElement(new InputFileStreamOption("testdata"),next);
  next=new OptionListElement(new DistributionOption("distribution"),next);
  next=new OptionListElement(new DistributionOption("distribution2"),next);
  next=new OptionListElement(new DistributionOption("simulation"),next);
  next=new OptionListElement(new DistributionOption("simulation2"),next);
  next=new OptionListElement(new DistributionOption("contamination"),next);
  next=new OptionListElement(new DistortionOption("distortion",distortionparameters),next);
  next=new OptionListElement(new DistortionOption("distortion2",distortionparameters),next);
  next=new OptionListElement(new AdequacyTestOption("adequacy_test"),next);
  return next;
};
