/*

   Copyright 2016 Toby Kenney

   This file is part of Adequate Bootstrap.

   Adequate Bootstrap is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Adequate Bootstrap is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Adequate Bootstrap.  If not, see <http://www.gnu.org/licenses/>.

 */


#include "distributions.h"
#include "distortion.h"
//#include "tests.h"
#include "data.h"
#include "corefunctions.h"
#include <iostream>
#include <fstream>
#include <exception>

/*
#include <cmath>
#include <cstdlib>
#include <ctime>


#include <boost/math/distributions/normal.hpp>
#include <boost/math/distributions/chi_squared.hpp>
#include <boost/math/distributions/binomial.hpp>
#include <boost/random/mersenne_twister.hpp>
#include <boost/random/variate_generator.hpp>
#include <boost/generator_iterator.hpp>
#include <boost/random/binomial_distribution.hpp>
#include <boost/random/mersenne_twister.hpp>
#include <boost/random/normal_distribution.hpp>
*/

#define BSSIZE 10000
//Number of bootstraps to perform.




using namespace std;



#define NUMBOOTSTRAP 10000


//3 modes of operation:
//1st mode: data analysis


void discriminant_analysis(Distribution &dist1,Distribution &dist2,AdequacyTest &adequacy_test,Data &cla1,Data &cla2,Data &test){
  //Fits dist1 to class 1, and dist2 to class 2, using adequate
  //bootstrap Calculates the likelihood of test under each class, and
  //thus gets the probability of datapoints in test being in each class.
  cout<<"Performing discriminant analysis.\n";
  unsigned int mx=(cla1.n>cla2.n)?cla1.n:cla2.n;
  mx-=-19;
  if(cla1.n<20||cla2.n<20){
    cout<<"Adequate Bootstrap cannot currently deal with datasets of size less than 20.\n";
    exit(0);
  };
  unsigned int *xvals=new unsigned int[mx];
  for(unsigned int i=0;i<mx;i++){
    *(xvals+i)=i+20;
  };

  cout<<"Class sizes: "<<cla1.n<<"\t"<<cla2.n<<"\n";
  unsigned int c1bssz=solveprobfast(cla1.n-19,xvals,adequacy_test,dist1,0.5,cla1);
  unsigned int c2bssz=solveprobfast(cla2.n-19,xvals,adequacy_test,dist2,0.5,cla2);

  cout<<"Adequate Bootstrap size:\n Class 1: "<<c1bssz<<"\nClass 2: "<<c2bssz<<"\n\n";
  
  long double *probs=new long double[test.n*BSSIZE];
  long double *stdbsprobs=new long double[test.n*BSSIZE];
  //  long double *probs2=new long double[test2.n*BSSIZE];
  for(unsigned int i=0;i<BSSIZE;i++){
    cla1.subsample(c1bssz);
    dist1.estimate(cla1);
    cla2.subsample(c2bssz);
    dist2.estimate(cla2);
    long double *lik1=dist1.likelihood(test);
    long double *lik2=dist2.likelihood(test);
    cla1.subsample(cla1.n);
    dist1.estimate(cla1);
    cla2.subsample(cla2.n);
    dist2.estimate(cla2);
        long double *lik3=dist1.likelihood(test);
        long double *lik4=dist2.likelihood(test);
    for(unsigned int j=0;j<test.n;j++){
      *(probs+j*BSSIZE+i)=*(lik1+j)/(*(lik1+j)+*(lik2+j));
      *(stdbsprobs+j*BSSIZE+i)=*(lik3+j)/(*(lik3+j)+*(lik4+j));
    };
    //    for(unsigned int j=0;j<test2.n;j++){
    //      *(probs2+j*BSSIZE+i)=*(lik3+j)/(*(lik3+j)+*(lik4+j));
    //    };
    delete[] lik1;
    delete[] lik2;
        delete[] lik3;
        delete[] lik4;
  };
  long double *CI=new long double[2*test.n];
  unsigned int pos[2]={(unsigned int)(0.025*BSSIZE),(unsigned int)(0.975*BSSIZE)};
  for(unsigned int i=0;i<test.n;i++){
    getpos(BSSIZE,probs+i*BSSIZE,2,pos,CI+2*i);
  };
  cout<<"\nAdequate Bootstrap:\n";
  for(unsigned int i=0;i<test.n;i++){
    cout<<*(CI+2*i)<<"\t"<<*(CI+2*i+1)<<"\n";
  };
  cout<<"\nStandard Bootstrap:\n";
  long double *CIstd=new long double[2*test.n];
  for(unsigned int i=0;i<test.n;i++){
    getpos(BSSIZE,stdbsprobs+i*BSSIZE,2,pos,CIstd+2*i);
  };
  for(unsigned int i=0;i<test.n;i++){
    cout<<*(CIstd+2*i)<<"\t"<<*(CIstd+2*i+1)<<"\n";
  };
};



void analyse_data(Distribution &truedist,AdequacyTest &adequacy_test,Data &A){
  //Analyse Data
  truedist.print();

  if(A.n<20){
    cout<<"Adequate Bootstrap cannot currently deal with datasets of size less than 20.\n";
    exit(0);
  };

  truedist.estimate(A);
  
  unsigned int *xvals=new unsigned int[A.n-19];
  for(unsigned int i=0;i<A.n-19;i++){
    *(xvals+i)=i+20;
  };
  unsigned int bssize=solveprobfast(A.n-19,xvals,adequacy_test,truedist,0.5,A);

  delete[] xvals;

  AdequateBootstrapIntervals ans;

  //  cout<<"Adequate Bootstrap size: "<<bssize<<".\n";
  ans.absz=bssize;


  long double *bs=new long double[BSSIZE];
  long double *stdbs=new long double[BSSIZE];
  long double *npbs=new long double[BSSIZE];

  long double *absamp=new long double[bssize];
  long double *bsamp=new long double[A.n];
  unsigned int *sbsmp=new unsigned int[A.n];

  for(unsigned int i=0;i<BSSIZE;i++){
    A.subsample(bssize);
    truedist.estimate(A);
    *(bs+i)=truedist.parval();
  };

  NPestimator *NPest=(NPestimator *)lookup_option("NP");
  
  for(unsigned int i=0;i<BSSIZE;i++){
    A.subsample(A.n);
    truedist.estimate(A);
    *(stdbs+i)=truedist.parval();
    if(NPest!=NULL){
      *(npbs+i)=NPest->evaluate(A);
    };
  };

  unsigned int pos[2]={(unsigned int)(0.025*BSSIZE),(unsigned int)(0.975*BSSIZE)};

  long double ci[2];
  long double stdci[2];
  long double npci[2];

  //  long double confprob[2]={0.025*exp(target),0.975*exp(target)};

  //  cout<<target<<" "<<confprob[0]<<" "<<confprob[1]<<" ";

  getpos(BSSIZE,bs,2,pos,ci);
  getpos(BSSIZE,stdbs,2,pos,stdci);

  //  cout<<posl<<" "<<posu<<"\n";
  
  ans.abl=ci[0];
  ans.abu=ci[1];
  ans.sbl=stdci[0];
  ans.sbu=stdci[1];


  //Initialise to avoid unnecessary compiler warnings.
  ans.nbl=0;
  ans.nbu=0;
  ans.nl=0;
  ans.nu=0;


  if(NPest!=NULL){
    getpos(BSSIZE,npbs,2,pos,npci);
    ans.nbl=npci[0];
    ans.nbu=npci[1];
  };

  NPestimator**CI=(NPestimator**)lookup_option("CI");
  
  if(CI!=NULL&&*CI!=NULL&&*(CI+1)!=NULL){
    for(unsigned int i=0;i<A.n;i++){
      *(sbsmp+i)=1;
    };
    ans.nl=(*CI)->evaluate(A);
    ans.nu=(*(CI+1))->evaluate(A);
    //    cout<<cil_VaR_at<<" "<<ciu_VaR_at;
  };


  delete[] absamp;
  delete[] bsamp;
  delete[] sbsmp;
  delete[] bs;
  delete[] stdbs;
  delete[] npbs;


  cout<<"\nAdequate Bootstrap size: "<<ans.absz<<"\n";

  cout<<"Adequate Bootstrap interval: ["<<ans.abl<<", "<<ans.abu<<"]\n";
  cout<<"Standard Bootstrap interval: ["<<ans.sbl<<", "<<ans.sbu<<"]\n";

  if(NPest!=NULL){
    cout<<"Nonparametric Bootstrap interval: ["<<ans.nbl<<", "<<ans.nbu<<"]\n";
  };

  if(CI!=NULL&&*CI!=NULL&&*(CI+1)!=NULL){
    cout<<"Theoretical Confidence interval: ["<<ans.nl<<", "<<ans.nu<<"]\n";
  };  
};


long double chisqlookup(unsigned int n,long double atval);

extern long double testatval;


void Discriminant_Analysis_mode(){
  //currently performs discriminant analysis between two classes.
  //Should extend to multiclass analysis.
  Distribution *dist1;
  Distribution *dist2;
  AdequacyTest *adequacy_test;
  istream *datafile1;
  istream *datafile2;  
  istream *testdatafile;  
  try{
    dist1=(Distribution*)lookup_option("distribution");
    if(dist1==NULL){
      cout<<"Distribution for analysis not specified";
      exit(0);
    };
    dist1->print();
    dist2=(Distribution*)lookup_option("distribution2");
    if(dist2==NULL){
      cout<<"Distribution for second class not specified. Using same distribution for both classes.";
      dist2=dist1->copy();//Default is to use same distribution for
    };
    dist2->print();

    //Next read adequacy test  (mandatory argument)
    adequacy_test=(AdequacyTest*)lookup_option("adequacy_test");
    if(adequacy_test==NULL){	
      cout<<"Adequacy test not specified.\n";
      exit(0);
    };                    
    datafile1=(istream*)lookup_option("file");
    datafile2=(istream*)lookup_option("file2");
    testdatafile=(istream*)lookup_option("testdata");
    if(datafile1==NULL&&datafile2==NULL){
      cout<<"No data files specified. Cannot read both from standard input.\n";
    };
    if(datafile1==NULL){
      cout<<"No data file specified. Reading data from standard input.\n";
      datafile1=&cin;
    };
    if(datafile2==NULL){
      cout<<"No data file specified. Reading data from standard input.\n";
      datafile2=&cin;
    };
  }catch(exception& e){
    cout<<e.what()<<"\n";
    exit(0);
  };
  datatype type__data(dist1->type_of_data(),dataformat());
  datatype type__data2(dist1->type_of_data(),dataformat());
  Data *data1=new_data(type__data);
  data1->read(*datafile1);
  Data *data2=new_data(type__data2);
  data2->read(*datafile2);
  Data *testdata;
  if(testdatafile==NULL){
    cout<<"No test data file specified. Concatenating input file.\n";
    testdata=data1->copy();
    testdata->concat(data2);
  }else{
    testdata=new_data(type__data);
    testdata->read(*testdatafile);
  };
  discriminant_analysis(*dist1,*dist2,*adequacy_test,*data1,*data2,*testdata);
};


void Discriminant_Simulation_mode(){
  //currently performs discriminant analysis between two classes.
  //Should extend to multiclass analysis.
  Distribution *dist1;
  Distribution *dist2;
  AdequacyTest *adequacy_test;
  Data *data1;
  Data *data2;
  Data *testdata1;
  try{
    const unsigned int *ss=(const unsigned int *)lookup_option("simsize");
    unsigned int simsize=1000;//default value
    if(ss){
      simsize=*ss;
    };
    ss=(const unsigned int *)lookup_option("simsize2");
    unsigned int simsize2=1000;//default value
    if(ss){
      simsize2=*ss;
    };

    dist1=(Distribution*)lookup_option("distribution");
    if(dist1==NULL){
      cout<<"Distribution for analysis not specified";
      exit(0);
    };
    dist1->print();
    dist2=(Distribution*)lookup_option("distribution2");
    if(dist2==NULL){
      cout<<"Distribution for second class not specified. Using same distribution for both classes.";
      dist2=dist1->copy();//Default is to use same distribution for both classes
      //Doesn't seem sensible for simulation.
    };
    dist2->print();


    Distribution *sim1=(Distribution *)lookup_option("simulation");
    Distribution *sim2=(Distribution *)lookup_option("simulation2");
    Distortion* distort=(Distortion*)lookup_option("distortion");
    Distortion* distort2=(Distortion*)lookup_option("distortion2");

    adequacy_test=(AdequacyTest*)lookup_option("adequacy_test");
    if(adequacy_test==NULL){	
      cout<<"Adequacy test not specified.\n";
      exit(0);
    };                    

    if(sim1==NULL||sim2==NULL){
      if(distort==NULL){
	cout<<"No simulation distribution or distotion type specified.\n";
	exit(0);
      };
      if(distort2==NULL){
	//Default apply same distortion to both distributions.
	distort2=distort;
      };
      data1=distort->simulate(simsize,*dist1);
      data2=distort2->simulate(simsize2,*dist2);
    }else{
      cout<<"Simulating first class from:\n";
      sim1->print();
      cout<<"\nSimulating second class from:\n";
      sim2->print();      
      data1=sim1->simulate(simsize);
      data2=sim2->simulate(simsize2);
    };
    
    

    //Next read adequacy test  (mandatory argument)
    istream *testdatafile=(istream *)lookup_option("testdata");
    if(testdatafile){
      datatype type__data(dist1->type_of_data(),dataformat());      
      testdata1=new_data(type__data);
      testdata1->read(*testdatafile);
    }else{
      if(sim1==NULL||sim2==NULL){
	testdata1=distort->simulate(simsize,*dist1);
	testdata1->concat(distort2->simulate(simsize2,*dist2));
      }else{
	testdata1=sim1->simulate(simsize);
	testdata1->concat(sim2->simulate(simsize2));
      };
    };
    ostream *save=(ostream *)lookup_option("save");
    if(save){
      const unsigned int *snc=(const unsigned int *)lookup_option("save_num_cols");
      unsigned int numcols=10;
      if(snc){
	numcols=*snc;
	if(numcols==0){
	  numcols=10;
	};
      };      
      data1->write(*save,numcols);
      data2->write(*save,numcols);
      testdata1->write(*save,numcols);
      close_option("save");
    };
  }catch(exception& e){
    cout<<e.what()<<"\n";
    exit(0);
  };
  discriminant_analysis(*dist1,*dist2,*adequacy_test,*data1,*data2,*testdata1);
};


void Analysis_mode(){
  Distribution *truedist;
  AdequacyTest *adequacy_test;
  istream *datafile;
  try{
    truedist=(Distribution*)lookup_option("distribution");
    if(truedist==NULL){
	cout<<"Distribution for analysis not specified";
    };
    truedist->print();
    //Next read adequacy test  (mandatory argument)
    adequacy_test=(AdequacyTest*)lookup_option("adequacy_test");
    if(adequacy_test==NULL){	
      cout<<"Adequacy test not specified.\n";
      exit(0);
    };
    if(*(unsigned int*)lookup_option("estimateparams")){
      adequacy_test->estimateparameters();
    };    
    datafile=(istream*)lookup_option("file");
    if(datafile==NULL){
      cout<<"No data file specified. Reading data from standard input.\n";
      datafile=&cin;
    };
  }catch(exception& e){
    cout<<e.what()<<"\n";
    exit(0);
  };
  datatype type__data(truedist->type_of_data(),dataformat());
  Data *data=new_data(type__data);
  data->read(*datafile);
  analyse_data(*truedist,*adequacy_test,*data);
};





void Simulation_mode(){
  Distribution *truedist;
  Distortion *distort;
  AdequacyTest *adequacy_test;
  Data *A;
  try{
    const unsigned int *ss=(const unsigned int *)lookup_option("simsize");
    unsigned int simsize=1000;//default value
    if(ss){
      simsize=*ss;
    };
    truedist=(Distribution*)lookup_option("simulation");
    if(truedist==NULL){
      truedist=(Distribution*)lookup_option("distribution");
      if(truedist==NULL){
	cout<<"Distribution for simulation not specified";
      };
    };
      
    truedist->print();
    
    distort=(Distortion*)lookup_option("distortion");
    
    if(distort==NULL){
      cout<<"Distotion type incorrectly specified.\n";
      exit(0);
    };
    
    //Next read adequacy test  (mandatory argument)
    adequacy_test=(AdequacyTest*)lookup_option("adequacy_test");
    if(adequacy_test==NULL){	
      cout<<"Adequacy test not specified.\n";
      exit(0);
    };
    if(*(unsigned int*)lookup_option("estimateparams")){
      adequacy_test->estimateparameters();
    };
    A=distort->simulate(simsize,*truedist);    
  }catch(exception& e){
    cout<<e.what()<<"\n";
    exit(0);
  };
  analyse_data(*truedist,*adequacy_test,*A);
};

//void test_mode(int argc,char *argv[]);


int main(int argc,char* argv[]){
  if(argc==1){
    cout<<"usage: \'"<<argv[0]<<" FILENAME DISTRIBUTION PARAMETERS ADEQUACY_TEST [OPTIONS]\n";
    cout<<"or: \'"<<argv[0]<<" simulation NUMBER DISTRIBUTION PARAMETERS DISTORTION PARAMETERS ADEQUACY_TEST [OPTIONS]\n";
    cout<<"or: \'"<<argv[0]<<" test \'\n";
    exit(0);
  };
//  if(!strcmp(argv[1],"test")){
//    test_mode(argc,argv);
//  }else
  try{
    for(unsigned int i=1;i<(unsigned int)argc;i++){
      read_option(argv[i]);
    };
  }catch(exception &e){
    cout<<e.what()<<"\n";
    exit(0);
  };
  const unsigned int *mode=(const unsigned int *)lookup_option("mode");
  if(mode){
    switch(*mode){
    case 0:
      Analysis_mode();
      break;
    case 1://Simulation
      Simulation_mode();
      break;
    case 2://test
      break;
    case 3://discriminant analysis
      Discriminant_Analysis_mode();
      break;
    case 4://discriminant analysis simulation
      Discriminant_Simulation_mode();
      break;      
    };
  }else{    
    //data mode    
    Analysis_mode();
  };
  cleanup();
};



