/*

   Copyright 2016 Toby Kenney

   This file is part of Adequate Bootstrap.

   Adequate Bootstrap is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Adequate Bootstrap is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Adequate Bootstrap.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <iostream>
#include <fstream>
#include <cmath>
#include <boost/math/distributions/normal.hpp>
#include <boost/math/distributions/chi_squared.hpp>
#include <cstdlib>
#include <ctime>

#include <boost/math/distributions/binomial.hpp>
#include <boost/random/mersenne_twister.hpp>
#include <boost/random/variate_generator.hpp>
#include <boost/generator_iterator.hpp>
#include <boost/random/binomial_distribution.hpp>
#include <boost/random/mersenne_twister.hpp>
#include <boost/random/normal_distribution.hpp>

boost::math::normal stdnorm(0.0, 1.0);

#define BSSIZE 10000
//Number of bootstraps to perform.


#include "distributions.h"
//#include "tests.h"
//#include "regression.h"
//#include "general.h"
#include "corefunctions.h"

using namespace std;

long double chisqcval;

std::ofstream savedata;

unsigned int jackknife;

unsigned int frequency_table;

unsigned int grouped_data;

unsigned int ngroup;

unsigned int maxbssize;

long double regression_lower=0;
long double regression_upper=1;
unsigned int regression_numstep=100;


//typedef long double (*NP_estimator)(unsigned int n,long double *data,unsigned int *samp);

NPestimator *NPest=NULL;
NPestimator *CILest=NULL;
NPestimator *CIUest=NULL;

void MLEProb(unsigned int n,const unsigned int *outcomes,long double *p){
//Assumes probability is a monotone function of the xvalues. Calculates
//the MLE of the probability for each x value under this constraint.

//outcomes is a matrix whose first column is number of
//         successes and whose last column is number of trials.
//         Rows are sorted in increasing order of xvalue.

  //  long double *minfor=new long double[n];
  long double *sucint=new long double[n*n];
  long double *totint=new long double[n*n];
  for(unsigned int i=0;i<n;i++){
    *(sucint+i*n+i)=*(outcomes+2*i);
    *(totint+i*n+i)=*(outcomes+2*i+1);
    for(unsigned int j=i+1;j<n;j++){
      *(sucint+i*n+j)=*(sucint+i*n+j-1)+*(outcomes+2*j);
      *(totint+i*n+j)=*(totint+i*n+j-1)+*(outcomes+2*j+1);
    };
  };
  long double max=0;

  for(unsigned int i=0;i<n;i++){
    *(p+i)=1;
    for(unsigned int j=i;j<n;j++){
      long double rat=((long double)*(sucint+i*n+j))/((long double)*(totint+i*n+j));
      if(rat<*(p+i)){
	*(p+i)=rat;
      };
    };
    if(*(p+i)<max){
      *(p+i)=max;
    }else{
      max=*(p+i);
    };
  };
  delete[] sucint;
  delete[] totint;
};


inline unsigned int position(long double prob,unsigned int n,const long double *phat){
  unsigned int N=n-1;
  unsigned int Nl=0;
  if(*(phat+N)>prob){
    if(*(phat)<prob){
      while(N-Nl>1){
	unsigned int Nm=(N+Nl)/2;
	if(*(phat+Nm)<prob){
	  Nl=Nm;
	}else{
	  N=Nm;
	};
      };
    }else{
      N=0;
    };
  };
  return N;
};

ofstream outlog;
char plotpower=0;
ostream *plotpowerstream;


void write_to_log(unsigned int n,long double *phat,unsigned int *outcomes ,unsigned int *xvals){
  if(plotpower){
    for(unsigned int i=0;i<n;i++){
      (*plotpowerstream)<<*(xvals+i)<<" "<<*(phat+i)<<" "<<*(outcomes+2*i)<<"/"<<*(outcomes+2*i+1)<<"\n";
    };
  };
};


unsigned int solveprob(unsigned int n,unsigned int *xvals,AdequacyTest &Ad,Distribution& dist,long double prob,Data &dat,unsigned int mxwd=100,unsigned int mxdp=10000){
//xvals are the values to be attempted
//trial is a function which produces a single run for one x value.
//prob is the target probability.
  unsigned int *outcomes=new unsigned int[2*n];
  for(unsigned int i=0;i<n;i++){       
    *(outcomes+2*i+1)=1;
    *(outcomes+2*i)=0;
    //Try to get more accurate measurement.
    for(unsigned int j=0;j<1;j++){
      dat.subsample(*(xvals+i));
      if(Ad.apply(dat,dist)){
	(*(outcomes+2*i))++;
      };
    };
  };
  //  write_to_log(n,outcomes,xvals);
  long double *phat=new long double[n];
  MLEProb(n,outcomes,phat);
  unsigned int N=position(prob,n,phat);
//Now produce more results close to the value N
  for(unsigned int k=1;k<11;k++){
    unsigned int intwidth=(mxwd/k);
    unsigned int intdepth=(mxdp/intwidth);
    for(unsigned int i=(N>intwidth)?(N-intwidth):0;i<N+intwidth&&i<n;i++){
      *(outcomes+2*i+1)+=intdepth;
      for(unsigned int j=0;j<intdepth;j++){
	dat.subsample(xvals[i]);
	if(Ad.apply(dat,dist)){
	  (*(outcomes+2*i))++;
	};
      };
    };
    MLEProb(n,outcomes,phat);
    N=position(prob,n,phat);
    write_to_log(n,phat,outcomes,xvals);
    //    ofstream outlog("outcomes.log",ios_base::app);
    //    if(outlog.is_open()){
    //      for(unsigned int i=0;i<n;i++){
    //	outlog<<*(xvals+i)<<" "<<*(outcomes+2*i)<<" "<<*(outcomes+2*i)<<"\n";
    //      };
    //    };
  };
  write_to_log(n,phat,outcomes,xvals);
  delete[] outcomes;
  delete[] phat;
  return N;
};

unsigned int solveprobfast(unsigned int n,unsigned int *xvals,AdequacyTest &Ad,Distribution &dist,long double prob,Data &dat){
  unsigned int numstep=log10(n)-1;
  unsigned int firststep=pow(10,numstep);
  //First run on every hundredth size
  unsigned int nh=n/firststep;
  unsigned int *xvh=new unsigned int[nh];
  for(unsigned int i=0;i<nh;i++){
    *(xvh+i)=*(xvals+i*firststep);
  };
  unsigned int N=solveprob(nh,xvh,Ad,dist,prob,dat,20,1000);

  unsigned int bt=0;


  unsigned int increasedepth=0;
  if(numstep>1){
      increasedepth=3000/(numstep-1);
  };

  for(;numstep>0;numstep--){
    unsigned int tp=n-1;
    if((N+10)*firststep<tp){
      tp=(N+10)*firststep;
    };
    bt=0;
    if(N>10){
      bt=(N-10)*firststep;
    };
    firststep/=10;

    unsigned int nh2=(tp-bt+1)/firststep;
    if(nh2>nh){
      delete[] xvh;
      xvh=new unsigned int[nh2];
      nh=nh2;
    };
    for(unsigned int i=0;i<nh2;i++){
      *(xvh+i)=*(xvals+bt+i*firststep);
    };
    N=solveprob(nh2,xvh,Ad,dist,prob,dat,40,5000-increasedepth*(numstep-1));
  };
  delete[]xvh;
  return(*(xvals+N+bt));
};









void getpos(unsigned int n,const long double *list,unsigned int numps,const unsigned int *pos,long double *to){
  //pos should be sorted.

  long double *newlist=new long double[n];
  unsigned int nl=0;
  unsigned int nu=1;
  long double pivot=*list;
  for(unsigned int i=1;i<n;i++){
    if(*(list+i)<pivot){
      *(newlist+nl++)=*(list+i);
    }else{
      *(newlist+n-nu++)=*(list+i);
    };
  };
  *(newlist+nl)=pivot;
  unsigned int i;
  for(i=0;i<numps&&pos[i]<nl;i++);
  if(i>0){
    getpos(nl,newlist,i,pos,to);
  };
  if(i<numps){
    if(pos[i]==nl){
      *(to+i)=pivot;    
      if(i+1<numps){
	unsigned int *newpos=new unsigned int[numps-i-1];
	for(unsigned int j=0;j<numps-i-1;j++){
	  newpos[j]=pos[i+j+1]-nl-1;
	};
	getpos(nu-1,newlist+nl+1,numps-i-1,newpos,to+i+1);
	delete[] newpos;
      };
    }else{
      //pos[i]>nl
      unsigned int *newpos=new unsigned int[numps-i];
      for(unsigned int j=0;j<numps-i;j++){
	newpos[j]=pos[i+j]-nl-1;
      };
      getpos(nu-1,newlist+nl+1,numps-i,newpos,to+i);
      delete[] newpos;
    };    
  };
  delete[] newlist;
};


unsigned int solveprobscan(unsigned int n,unsigned int *xvals,AdequacyTest& trial,long double prob,Distribution& dist,Data &dat){
  //Quickly checks whether the range is sensible for the values needed.
  if(n<100){
    return solveprob(n,xvals,trial,dist,prob,dat,15,20);
  };
  unsigned int N=solveprob(100,xvals,trial,dist,prob,dat,15,20);
  if(N>95){
    return solveprobfast(n,xvals,trial,dist,prob,dat);
  };  
  if(N>80){
    if(n>199){
      N=solveprobfast(200,xvals,trial,dist,prob,dat);
      if(N<170){
	return N;
      };
    };
    return solveprobfast(n,xvals,trial,dist,prob,dat);
  };
  if(N>5){
    return solveprobfast(100,xvals,trial,dist,prob,dat);
  };
  return N;
};






void read_option(const char* const optname){
    char *at=NULL;
    char *at2=NULL;
    char *eq=NULL;
    char* name=new char[strlen(optname)+1];
    strcpy(name,optname);
    unsigned int len=strlen(name);
    for(unsigned int i=0;i<len;i++){
      if(*(name+i)=='@'){
	*(name+i)='\0';
	if(at==NULL){
	  at=(name+i+1);
	}else{
	  at2=(name+i+1);
	};
      }else if(*(name+i)=='='){
	*(name+i)='\0';
	eq=(name+i+1);	
      };
    };
    if(!strcmp(name,"seed")){
      if(at!=NULL){
	cout<<"@ option not used for seed option. Ignoring.\n";
      };
      if(eq==NULL){
	cout<<"Seed value unspecified. Ignoring seed setting.\n";
      }else{	
	unsigned int val=atoi(eq);
	cout<<"Seeding random generator with value "<<val<<".\n";
	srand(val);
      };
    }else if(!strcmp(name,"NP")){
      NPest=get_NP_estimator(eq,at);
    }else if(!strcmp(name,"CI")){
      if(eq==NULL){
	cout<<"CI method unspecified. Ignoring CI setting.\n";
      }else{	
	if(*eq!='['){
	  cout<<"first method: \""<<eq<<"\"\n";
	  cout<<"CI format \"CI=[LOWER,UPPER]\".\n";
	}else{
	  eq++;
	  char* eq2=eq;	
	  for(;*eq2!='\0'&&*eq2!=',';eq2++);
	  if(*eq2!=','){
	    if(++eq2==at){
	      for(;*eq2!='\0'&&*eq2!=',';eq2++);
	      if(*eq2!=','){
		cout<<"Arguments:\t\""<<eq<<"\"\n\t\t\""<<at<<"\"\n\t\t\""<<at2<<"\"\n";
		cout<<"CI format \"CI=[LOWER,UPPER]\".\n";
	      };
	    }else{
		cout<<"Arguments:\t\""<<eq<<"\"\n\t\t\""<<at<<"\"\n\t\t\""<<eq2<<"\"\n\t\t\""<<at2<<"\"\n";	      cout<<"CI format \"CI=[LOWER,UPPER]\".\n";
	    };
	  };
	  *(eq2++)='\0';
	  CILest=get_NP_estimator(eq,at);
	  CIUest=get_NP_estimator(eq2,at);
	};
      };
    }else if(!strcmp(name,"jackknife")){
      if(eq==NULL){
	jackknife=1;
      }else{
	if(!strcmp(eq,"1")||!strcmp(eq,"T")||!strcmp(eq,"yes")||!strcmp(eq,"true")||!strcmp(eq,"True")||!strcmp(eq,"Yes")){
	  jackknife=1;
	}else if(!strcmp(eq,"0")||!strcmp(eq,"F")||!strcmp(eq,"no")||!strcmp(eq,"No")||!strcmp(eq,"False")||!strcmp(eq,"false")){
	  jackknife=0;
	}else{
	  cout<<"Could not determine meaning of setting \""<<eq<<"\" for jackknife.\n";
	};
      };      
    }else if(!strcmp(name,"log")){
      if(eq==NULL){
	outlog.open("outcomes.log");
	if(!outlog.is_open()){
	  cout<<"Problem opening default log file \"outcomes.log\"\n";
	};
      }else{
	outlog.open(eq);
	if(!outlog.is_open()){
	  cout<<"Problem opening log file \""<<eq<<"\"\n";
	};	
      };      
    }else if(!strcmp(name,"save")){
      if(eq==NULL){
	eq=(char *)"saved_data";
	//No checking for overwriting file;
      };
      savedata.open(eq);      
      if(!savedata.is_open()){
	cout<<"Unable to open file \""<<eq<<"\".\n";
      };
    }else if(!strcmp(name,"frequency")){
      if(eq==NULL){
	frequency_table=1;
      }else{
	if(!strcmp(eq,"1")||!strcmp(eq,"T")||!strcmp(eq,"yes")||!strcmp(eq,"true")||!strcmp(eq,"True")||!strcmp(eq,"Yes")){
	  frequency_table=1;
	}else if(!strcmp(eq,"0")||!strcmp(eq,"F")||!strcmp(eq,"no")||!strcmp(eq,"No")||!strcmp(eq,"False")||!strcmp(eq,"false")){
	  frequency_table=0;
	}else{
	  cout<<"Could not determine meaning of setting \""<<eq<<"\" for frequency.\n";
	};
      };
    }else if(!strcmp(name,"grouped")){
      if(eq==NULL){
	grouped_data=1;
      }else{
	if(!strcmp(eq,"1")||!strcmp(eq,"T")||!strcmp(eq,"yes")||!strcmp(eq,"true")||!strcmp(eq,"True")||!strcmp(eq,"Yes")){
	 grouped_data=1;
	}else if(!strcmp(eq,"0")||!strcmp(eq,"F")||!strcmp(eq,"no")||!strcmp(eq,"No")||!strcmp(eq,"False")||!strcmp(eq,"false")){
	  grouped_data=0;
	}else{
	  cout<<"Could not determine meaning of setting \""<<eq<<"\" for grouped data.\n";
	};
      };
    }else if(!strcmp(name,"maxbssize")){
      if(eq!=NULL){
	maxbssize=atoi(eq);
      };
    }else if(!strcmp(name,"regression_lower")){
      //lower limit for graphing regression
      if(eq==NULL){
	cout<<"Regression lower limit  unspecified. Using default value 0.\n";
      }else{
	regression_lower=atof(eq);
      };
    }else if(!strcmp(name,"regression_upper")){
      //lower limit for graphing regression
      if(eq==NULL){
	cout<<"Regression upper limit  unspecified. Using default value 1.\n";
      }else{
	regression_upper=atof(eq);
      };
    }else if(!strcmp(name,"regression_num_steps")){
      //lower limit for graphing regression
      if(eq==NULL){
	cout<<"Regression number of steps unspecified. Using default value 100.\n";
      }else{
	regression_numstep=atoi(eq);
      };
    }else if(!strcmp(name,"regression_step_size")){
      //lower limit for graphing regression
      if(eq==NULL){
	cout<<"Regression step size unspecified. Using default value.\n";
      }else{
	long double stepsz=atof(eq);
	regression_numstep=abs((int)((regression_upper-regression_lower)/stepsz));
      };
    }else if(!strcmp(name,"plotpower")){
      //plot power curve
      plotpower=1;
      if(eq==NULL){
	plotpowerstream=&cout;
      }else{
	cout<<"Opening file \""<<eq<<"\" for writing power curve.\n";	
	plotpowerstream=new ofstream(eq);
	if(!(plotpowerstream->good())){
	  cout<<"Could not open file \""<<eq<<"\" for writing power curve.\n";
	  plotpowerstream=&cout;
	};
      };      
    }else{
      cout<<"Option \""<<name<<"\" not currently supported. Ignoring option.\n";
      // long double atval=0;
      // if(at!=NULL){
      // 	atval=atof(at);
      // 	cout<<atval<<"\n";
      // };
    };

    delete[] name;
};





long double chisqlookup(unsigned int n,long double atval){
  //  calculates the critical value of a chi-square distribution with
  //  n degrees of freedom at significance level atval.
  boost::math::chi_squared chisquaredist(n);
  long double answ=boost::math::quantile(chisquaredist,1-atval);
  //  cout<<n<<" "<<atval<<" "<<answ<<"\n";
  return answ;
  long double *vals=new long double[10000];
  for(unsigned int i=0;i<10000;i++){
    *(vals+i)=0;
    for(unsigned int j=0;j<n;j++){
      long double rnd=rnorm();
      *(vals+i)+=rnd*rnd;
    };
  };
  unsigned int cutoff=(1-atval)*10000;
  //  cout<<"cutoff="<<cutoff<<"\n";
  long double ans=0;
  getpos(10000,vals,1,&cutoff,&ans);
  delete[] vals;
  cout<<"Critical value of chi-squared test is "<<ans<<".\n";
  return ans;
};

void cleanup(){
  //Does some necessary clean-up on end of program.


  if(plotpowerstream!=NULL&&plotpowerstream!=&cout){
    ((ofstream*)plotpowerstream)->close();
  };

  
};

